@@ -20,20 +20,24 @@ package org.apache.spark.network.util;
 import java.util.concurrent.ThreadFactory;
 
 import io.netty.buffer.PooledByteBufAllocator;
-import io.netty.channel.Channel;
-import io.netty.channel.EventLoopGroup;
-import io.netty.channel.ServerChannel;
-import io.netty.channel.epoll.EpollEventLoopGroup;
+import io.netty.channel.*;
+import io.netty.channel.epoll.Epoll;
+import io.netty.channel.epoll.EpollIoHandler;
 import io.netty.channel.epoll.EpollServerSocketChannel;
 import io.netty.channel.epoll.EpollSocketChannel;
-import io.netty.channel.nio.NioEventLoopGroup;
+import io.netty.channel.kqueue.KQueue;
+import io.netty.channel.kqueue.KQueueIoHandler;
+import io.netty.channel.kqueue.KQueueServerSocketChannel;
+import io.netty.channel.kqueue.KQueueSocketChannel;
+import io.netty.channel.nio.NioIoHandler;
 import io.netty.channel.socket.nio.NioServerSocketChannel;
 import io.netty.channel.socket.nio.NioSocketChannel;
 import io.netty.util.concurrent.DefaultThreadFactory;
 import io.netty.util.internal.PlatformDependent;
 
 /**
- * Utilities for creating various Netty constructs based on whether we're using EPOLL or NIO.
+ * Utilities for creating various Netty constructs based on whether we're using NIO, EPOLL,
+ * , KQUEUE, or AUTO.
  */
 public class NettyUtils {
 
@@ -65,10 +69,21 @@ public class NettyUtils {
   public static EventLoopGroup createEventLoop(IOMode mode, int numThreads, String threadPrefix) {
     ThreadFactory threadFactory = createThreadFactory(threadPrefix);
 
-    return switch (mode) {
-      case NIO -> new NioEventLoopGroup(numThreads, threadFactory);
-      case EPOLL -> new EpollEventLoopGroup(numThreads, threadFactory);
+    IoHandlerFactory handlerFactory = switch (mode) {
+      case NIO -> NioIoHandler.newFactory();
+      case EPOLL -> EpollIoHandler.newFactory();
+      case KQUEUE -> KQueueIoHandler.newFactory();
+      case AUTO -> {
+        if (JavaUtils.isLinux && Epoll.isAvailable()) {
+          yield EpollIoHandler.newFactory();
+        } else if (JavaUtils.isMac && KQueue.isAvailable()) {
+          yield KQueueIoHandler.newFactory();
+        } else {
+          yield NioIoHandler.newFactory();
+        }
+      }
     };
+    return new MultiThreadIoEventLoopGroup(numThreads, threadFactory, handlerFactory);
   }
 
   /** Returns the correct (client) SocketChannel class based on IOMode. */
@@ -76,6 +91,16 @@ public class NettyUtils {
     return switch (mode) {
       case NIO -> NioSocketChannel.class;
       case EPOLL -> EpollSocketChannel.class;
+      case KQUEUE -> KQueueSocketChannel.class;
+      case AUTO -> {
+        if (JavaUtils.isLinux && Epoll.isAvailable()) {
+          yield EpollSocketChannel.class;
+        } else if (JavaUtils.isMac && KQueue.isAvailable()) {
+          yield KQueueSocketChannel.class;
+        } else {
+          yield NioSocketChannel.class;
+        }
+      }
     };
   }
 
@@ -84,6 +109,16 @@ public class NettyUtils {
     return switch (mode) {
       case NIO -> NioServerSocketChannel.class;
       case EPOLL -> EpollServerSocketChannel.class;
+      case KQUEUE -> KQueueServerSocketChannel.class;
+      case AUTO -> {
+        if (JavaUtils.isLinux && Epoll.isAvailable()) {
+          yield EpollServerSocketChannel.class;
+        } else if (JavaUtils.isMac && KQueue.isAvailable()) {
+          yield KQueueServerSocketChannel.class;
+        } else {
+          yield NioServerSocketChannel.class;
+        }
+      }
     };
   }
 
