@@ -28,8 +28,6 @@ import org.junit.jupiter.api.Test;
 import static org.junit.jupiter.api.Assertions.*;
 import static org.mockito.Mockito.*;
 
-import org.apache.commons.lang3.tuple.ImmutablePair;
-import org.apache.commons.lang3.tuple.Pair;
 import org.apache.spark.network.buffer.ManagedBuffer;
 import org.apache.spark.network.client.RpcResponseCallback;
 import org.apache.spark.network.client.TransportClient;
@@ -39,6 +37,7 @@ import org.apache.spark.network.server.OneForOneStreamManager;
 import org.apache.spark.network.server.RpcHandler;
 import org.apache.spark.network.server.StreamManager;
 import org.apache.spark.network.server.TransportRequestHandler;
+import org.apache.spark.util.Pair;
 
 public class TransportRequestHandlerSuite {
 
@@ -53,7 +52,7 @@ public class TransportRequestHandlerSuite {
       .thenAnswer(invocationOnMock0 -> {
         Object response = invocationOnMock0.getArguments()[0];
         ExtendedChannelPromise channelFuture = new ExtendedChannelPromise(channel);
-        responseAndPromisePairs.add(ImmutablePair.of(response, channelFuture));
+        responseAndPromisePairs.add(Pair.of(response, channelFuture));
         return channelFuture;
       });
 
@@ -145,7 +144,7 @@ public class TransportRequestHandlerSuite {
     when(channel.writeAndFlush(any())).thenAnswer(invocationOnMock0 -> {
       Object response = invocationOnMock0.getArguments()[0];
       ExtendedChannelPromise channelFuture = new ExtendedChannelPromise(channel);
-      responseAndPromisePairs.add(ImmutablePair.of(response, channelFuture));
+      responseAndPromisePairs.add(Pair.of(response, channelFuture));
       return channelFuture;
     });
 
