@@ -20,15 +20,13 @@ package org.apache.spark.network.shuffle;
 import java.io.*;
 import java.nio.charset.StandardCharsets;
 import java.util.*;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
 import java.util.stream.Collectors;
 
-import org.apache.commons.lang3.builder.ToStringBuilder;
-import org.apache.commons.lang3.builder.ToStringStyle;
-import org.apache.commons.lang3.tuple.Pair;
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.fasterxml.jackson.databind.ObjectMapper;
@@ -37,7 +35,6 @@ import com.google.common.cache.CacheBuilder;
 import com.google.common.cache.CacheLoader;
 import com.google.common.cache.LoadingCache;
 import com.google.common.cache.Weigher;
-import com.google.common.collect.Maps;
 
 import org.apache.spark.internal.SparkLogger;
 import org.apache.spark.internal.SparkLoggerFactory;
@@ -56,6 +53,7 @@ import org.apache.spark.network.util.DBProvider;
 import org.apache.spark.network.util.JavaUtils;
 import org.apache.spark.network.util.NettyUtils;
 import org.apache.spark.network.util.TransportConf;
+import org.apache.spark.util.Pair;
 
 /**
  * Manages converting shuffle BlockIds into physical segments of local files, from a process outside
@@ -134,11 +132,11 @@ public class ExternalShuffleBlockResolver {
     db = DBProvider.initDB(dbBackend, this.registeredExecutorFile, CURRENT_VERSION, mapper);
     if (db != null) {
       logger.info("Use {} as the implementation of {}",
-        MDC.of(LogKeys.SHUFFLE_DB_BACKEND_NAME$.MODULE$, dbBackend),
-        MDC.of(LogKeys.SHUFFLE_DB_BACKEND_KEY$.MODULE$, Constants.SHUFFLE_SERVICE_DB_BACKEND));
+        MDC.of(LogKeys.SHUFFLE_DB_BACKEND_NAME, dbBackend),
+        MDC.of(LogKeys.SHUFFLE_DB_BACKEND_KEY, Constants.SHUFFLE_SERVICE_DB_BACKEND));
       executors = reloadRegisteredExecutors(db);
     } else {
-      executors = Maps.newConcurrentMap();
+      executors = new ConcurrentHashMap<>();
     }
     this.directoryCleaner = directoryCleaner;
   }
@@ -154,8 +152,8 @@ public class ExternalShuffleBlockResolver {
       ExecutorShuffleInfo executorInfo) {
     AppExecId fullId = new AppExecId(appId, execId);
     logger.info("Registered executor {} with {}",
-      MDC.of(LogKeys.APP_EXECUTOR_ID$.MODULE$, fullId),
-      MDC.of(LogKeys.EXECUTOR_SHUFFLE_INFO$.MODULE$, executorInfo));
+      MDC.of(LogKeys.APP_EXECUTOR_ID, fullId),
+      MDC.of(LogKeys.EXECUTOR_SHUFFLE_INFO, executorInfo));
     try {
       if (db != null && AppsWithRecoveryDisabled.isRecoveryEnabledForApp(appId)) {
         byte[] key = dbAppExecKey(fullId);
@@ -221,8 +219,8 @@ public class ExternalShuffleBlockResolver {
    */
   public void applicationRemoved(String appId, boolean cleanupLocalDirs) {
     logger.info("Application {} removed, cleanupLocalDirs = {}",
-      MDC.of(LogKeys.APP_ID$.MODULE$, appId),
-      MDC.of(LogKeys.CLEANUP_LOCAL_DIRS$.MODULE$, cleanupLocalDirs));
+      MDC.of(LogKeys.APP_ID, appId),
+      MDC.of(LogKeys.CLEANUP_LOCAL_DIRS, cleanupLocalDirs));
     Iterator<Map.Entry<AppExecId, ExecutorShuffleInfo>> it = executors.entrySet().iterator();
     while (it.hasNext()) {
       Map.Entry<AppExecId, ExecutorShuffleInfo> entry = it.next();
@@ -237,14 +235,14 @@ public class ExternalShuffleBlockResolver {
             db.delete(dbAppExecKey(fullId));
           } catch (IOException e) {
             logger.error("Error deleting {} from executor state db", e,
-              MDC.of(LogKeys.APP_ID$.MODULE$, appId));
+              MDC.of(LogKeys.APP_ID, appId));
           }
         }
 
         if (cleanupLocalDirs) {
           logger.info("Cleaning up executor {}'s {} local dirs",
-            MDC.of(LogKeys.APP_EXECUTOR_ID$.MODULE$, fullId),
-            MDC.of(LogKeys.NUM_LOCAL_DIRS$.MODULE$, executor.localDirs.length));
+            MDC.of(LogKeys.APP_EXECUTOR_ID, fullId),
+            MDC.of(LogKeys.NUM_LOCAL_DIRS, executor.localDirs.length));
 
           // Execute the actual deletion in a different thread, as it may take some time.
           directoryCleaner.execute(() -> deleteExecutorDirs(executor.localDirs));
@@ -259,18 +257,18 @@ public class ExternalShuffleBlockResolver {
    */
   public void executorRemoved(String executorId, String appId) {
     logger.info("Clean up non-shuffle and non-RDD files associated with the finished executor {}",
-      MDC.of(LogKeys.EXECUTOR_ID$.MODULE$, executorId));
+      MDC.of(LogKeys.EXECUTOR_ID, executorId));
     AppExecId fullId = new AppExecId(appId, executorId);
     final ExecutorShuffleInfo executor = executors.get(fullId);
     if (executor == null) {
       // Executor not registered, skip clean up of the local directories.
       logger.info("Executor is not registered (appId={}, execId={})",
-        MDC.of(LogKeys.APP_ID$.MODULE$, appId),
-        MDC.of(LogKeys.EXECUTOR_ID$.MODULE$, executorId));
+        MDC.of(LogKeys.APP_ID, appId),
+        MDC.of(LogKeys.EXECUTOR_ID, executorId));
     } else {
       logger.info("Cleaning up non-shuffle and non-RDD files in executor {}'s {} local dirs",
-        MDC.of(LogKeys.APP_EXECUTOR_ID$.MODULE$, fullId),
-        MDC.of(LogKeys.NUM_LOCAL_DIRS$.MODULE$, executor.localDirs.length));
+        MDC.of(LogKeys.APP_EXECUTOR_ID, fullId),
+        MDC.of(LogKeys.NUM_LOCAL_DIRS, executor.localDirs.length));
 
       // Execute the actual deletion in a different thread, as it may take some time.
       directoryCleaner.execute(() -> deleteNonShuffleServiceServedFiles(executor.localDirs));
@@ -288,7 +286,7 @@ public class ExternalShuffleBlockResolver {
         logger.debug("Successfully cleaned up directory: {}", localDir);
       } catch (Exception e) {
         logger.error("Failed to delete directory: {}", e,
-          MDC.of(LogKeys.PATH$.MODULE$, localDir));
+          MDC.of(LogKeys.PATH, localDir));
       }
     }
   }
@@ -311,7 +309,7 @@ public class ExternalShuffleBlockResolver {
           localDir);
       } catch (Exception e) {
         logger.error("Failed to delete files not served by shuffle service in directory: {}", e,
-          MDC.of(LogKeys.PATH$.MODULE$, localDir));
+          MDC.of(LogKeys.PATH, localDir));
       }
     }
   }
@@ -384,7 +382,7 @@ public class ExternalShuffleBlockResolver {
         numRemovedBlocks++;
       } else {
         logger.warn("Failed to delete block: {}",
-          MDC.of(LogKeys.PATH$.MODULE$, file.getAbsolutePath()));
+          MDC.of(LogKeys.PATH, file.getAbsolutePath()));
       }
     }
     return numRemovedBlocks;
@@ -400,7 +398,7 @@ public class ExternalShuffleBlockResolver {
         }
         return Pair.of(exec, info.localDirs);
       })
-      .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
+      .collect(Collectors.toMap(Pair::getLeft, Pair::getRight));
   }
 
   /**
@@ -451,10 +449,7 @@ public class ExternalShuffleBlockResolver {
 
     @Override
     public String toString() {
-      return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
-        .append("appId", appId)
-        .append("execId", execId)
-        .toString();
+      return "ExternalShuffleBlockResolver[appId=" + appId + ",execId=" + execId + "]";
     }
   }
 
@@ -477,7 +472,7 @@ public class ExternalShuffleBlockResolver {
   @VisibleForTesting
   static ConcurrentMap<AppExecId, ExecutorShuffleInfo> reloadRegisteredExecutors(DB db)
       throws IOException {
-    ConcurrentMap<AppExecId, ExecutorShuffleInfo> registeredExecutors = Maps.newConcurrentMap();
+    ConcurrentMap<AppExecId, ExecutorShuffleInfo> registeredExecutors = new ConcurrentHashMap<>();
     if (db != null) {
       try (DBIterator itr = db.iterator()) {
         itr.seek(APP_KEY_PREFIX.getBytes(StandardCharsets.UTF_8));
@@ -489,7 +484,7 @@ public class ExternalShuffleBlockResolver {
           }
           AppExecId id = parseDbAppExecKey(key);
           logger.info("Reloading registered executors: {}",
-            MDC.of(LogKeys.APP_EXECUTOR_ID$.MODULE$, id));
+            MDC.of(LogKeys.APP_EXECUTOR_ID, id));
           ExecutorShuffleInfo shuffleInfo =
             mapper.readValue(e.getValue(), ExecutorShuffleInfo.class);
           registeredExecutors.put(id, shuffleInfo);
