@@ -23,8 +23,6 @@ import java.util.Objects;
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import io.netty.buffer.ByteBuf;
-import org.apache.commons.lang3.builder.ToStringBuilder;
-import org.apache.commons.lang3.builder.ToStringStyle;
 
 import org.apache.spark.network.protocol.Encodable;
 import org.apache.spark.network.protocol.Encoders;
@@ -60,11 +58,8 @@ public class ExecutorShuffleInfo implements Encodable {
 
   @Override
   public String toString() {
-    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
-      .append("localDirs", Arrays.toString(localDirs))
-      .append("subDirsPerLocalDir", subDirsPerLocalDir)
-      .append("shuffleManager", shuffleManager)
-      .toString();
+    return "ExecutorShuffleInfo[localDirs=" + Arrays.toString(localDirs) +
+        ",subDirsPerLocalDir=" + subDirsPerLocalDir + ",shuffleManager=" + shuffleManager + "]";
   }
 
   @Override
