@@ -17,7 +17,8 @@
 
 package org.apache.spark.unsafe.types
 
-import scala.collection.parallel.immutable.ParSeq
+import java.util.stream.IntStream
+
 import scala.jdk.CollectionConverters.MapHasAsScala
 
 import com.ibm.icu.util.ULocale
@@ -27,13 +28,14 @@ import org.apache.spark.sql.catalyst.util.CollationFactory.fetchCollation
 // scalastyle:off
 import org.scalatest.funsuite.AnyFunSuite
 import org.scalatest.matchers.must.Matchers
+import com.ibm.icu.util.VersionInfo.ICU_VERSION
 
 import org.apache.spark.sql.catalyst.util.CollationFactory._
 import org.apache.spark.unsafe.types.UTF8String.{fromString => toUTF8}
 
 class CollationFactorySuite extends AnyFunSuite with Matchers { // scalastyle:ignore funsuite
 
-  val currentIcuVersion: String = "76.1"
+  val currentIcuVersion: String = s"${ICU_VERSION.getMajor}.${ICU_VERSION.getMinor}"
 
   test("collationId stability") {
     assert(INDETERMINATE_COLLATION_ID == -1)
@@ -292,7 +294,7 @@ class CollationFactorySuite extends AnyFunSuite with Matchers { // scalastyle:ig
     (0 to 10).foreach(_ => {
       val collator = fetchCollation("UNICODE").getCollator
 
-      ParSeq(0 to 100).foreach { _ =>
+      IntStream.rangeClosed(0, 100).parallel().forEach { _ =>
         collator.getCollationKey("aaa")
       }
     })
