@@ -36,8 +36,8 @@ import org.apache.hadoop.security.{Credentials, UserGroupInformation}
 import org.apache.hadoop.security.token.{Token, TokenIdentifier}
 import org.apache.hadoop.security.token.delegation.AbstractDelegationTokenIdentifier
 
-import org.apache.spark.{SparkConf, SparkException}
-import org.apache.spark.internal.{Logging, LogKeys, MDC}
+import org.apache.spark.{ReadOnlySparkConf, SparkConf, SparkException}
+import org.apache.spark.internal.{Logging, LogKeys}
 import org.apache.spark.internal.config.BUFFER_SIZE
 import org.apache.spark.util.ArrayImplicits._
 import org.apache.spark.util.Utils
@@ -90,7 +90,7 @@ private[spark] class SparkHadoopUtil extends Logging {
    * Appends spark.hadoop.* configurations from a [[SparkConf]] to a Hadoop
    * configuration without the spark.hadoop. prefix.
    */
-  def appendSparkHadoopConfigs(conf: SparkConf, hadoopConf: Configuration): Unit = {
+  def appendSparkHadoopConfigs(conf: ReadOnlySparkConf, hadoopConf: Configuration): Unit = {
     SparkHadoopUtil.appendSparkHadoopConfigs(conf, hadoopConf)
   }
 
@@ -430,14 +430,14 @@ private[spark] object SparkHadoopUtil extends Logging {
    * and if found on the classpath, those of core-site.xml.
    * This is done before the spark overrides are applied.
    */
-  private[spark] def newConfiguration(conf: SparkConf): Configuration = {
+  private[spark] def newConfiguration(conf: ReadOnlySparkConf): Configuration = {
     val hadoopConf = new Configuration()
     appendS3AndSparkHadoopHiveConfigurations(conf, hadoopConf)
     hadoopConf
   }
 
   private def appendS3AndSparkHadoopHiveConfigurations(
-      conf: SparkConf,
+      conf: ReadOnlySparkConf,
       hadoopConf: Configuration): Unit = {
     // Note: this null check is around more than just access to the "conf" object to maintain
     // the behavior of the old implementation of this code, for backwards compatibility.
@@ -514,7 +514,7 @@ private[spark] object SparkHadoopUtil extends Logging {
     }
   }
 
-  private def appendSparkHadoopConfigs(conf: SparkConf, hadoopConf: Configuration): Unit = {
+  private def appendSparkHadoopConfigs(conf: ReadOnlySparkConf, hadoopConf: Configuration): Unit = {
     // Copy any "spark.hadoop.foo=bar" spark properties into conf as "foo=bar"
     for ((key, value) <- conf.getAll if key.startsWith("spark.hadoop.")) {
       hadoopConf.set(key.substring("spark.hadoop.".length), value,
@@ -532,7 +532,7 @@ private[spark] object SparkHadoopUtil extends Logging {
     }
   }
 
-  private def appendSparkHiveConfigs(conf: SparkConf, hadoopConf: Configuration): Unit = {
+  private def appendSparkHiveConfigs(conf: ReadOnlySparkConf, hadoopConf: Configuration): Unit = {
     // Copy any "spark.hive.foo=bar" spark properties into conf as "hive.foo=bar"
     for ((key, value) <- conf.getAll if key.startsWith("spark.hive.")) {
       hadoopConf.set(key.substring("spark.".length), value, SOURCE_SPARK_HIVE)
