@@ -18,7 +18,7 @@ package org.apache.spark.executor
 
 import java.lang.Long.{MAX_VALUE => LONG_MAX_VALUE}
 import java.util.concurrent.{ConcurrentHashMap, TimeUnit}
-import java.util.concurrent.atomic.{AtomicLong, AtomicLongArray}
+import java.util.concurrent.atomic.AtomicLongArray
 
 import scala.collection.mutable.HashMap
 
@@ -53,7 +53,7 @@ private[spark] class ExecutorMetricsPoller(
 
   type StageKey = (Int, Int)
   // Task Count and Metric Peaks
-  private[executor] case class TCMP(count: AtomicLong, peaks: AtomicLongArray)
+  private[executor] case class TCMP(count: Long, peaks: AtomicLongArray)
 
   // Map of (stageId, stageAttemptId) to (count of running tasks, executor metric peaks)
   private[executor] val stageTCMP = new ConcurrentHashMap[StageKey, TCMP]
@@ -112,10 +112,13 @@ private[spark] class ExecutorMetricsPoller(
 
     // Put a new entry in stageTCMP for the stage if there isn't one already.
     // Increment the task count.
-    val countAndPeaks = stageTCMP.computeIfAbsent((stageId, stageAttemptId),
-      _ => TCMP(new AtomicLong(0), new AtomicLongArray(ExecutorMetricType.numMetrics)))
-    val stageCount = countAndPeaks.count.incrementAndGet()
-    logDebug(s"stageTCMP: ($stageId, $stageAttemptId) -> $stageCount")
+    val countAndPeaks = stageTCMP.compute((stageId, stageAttemptId), (k: StageKey, v: TCMP) =>
+      if (v == null) {
+        TCMP(1L, new AtomicLongArray(ExecutorMetricType.numMetrics))
+      } else {
+        TCMP(v.count + 1, v.peaks)
+      })
+    logDebug(s"stageTCMP: ($stageId, $stageAttemptId) -> ${countAndPeaks.count}")
   }
 
   /**
@@ -126,10 +129,10 @@ private[spark] class ExecutorMetricsPoller(
     // Decrement the task count.
 
     def decrementCount(stage: StageKey, countAndPeaks: TCMP): TCMP = {
-      val countValue = countAndPeaks.count.decrementAndGet()
+      val countValue = countAndPeaks.count - 1
       assert(countValue >= 0, "task count shouldn't below 0")
       logDebug(s"stageTCMP: (${stage._1}, ${stage._2}) -> " + countValue)
-      countAndPeaks
+      TCMP(countValue, countAndPeaks.peaks)
     }
 
     stageTCMP.computeIfPresent((stageId, stageAttemptId), decrementCount)
@@ -172,7 +175,7 @@ private[spark] class ExecutorMetricsPoller(
     stageTCMP.replaceAll(getUpdateAndResetPeaks)
 
     def removeIfInactive(k: StageKey, v: TCMP): TCMP = {
-      if (v.count.get == 0) {
+      if (v.count == 0) {
         logDebug(s"removing (${k._1}, ${k._2}) from stageTCMP")
         null
       } else {
