@@ -18,7 +18,8 @@
 package org.apache.spark.api.python
 
 import java.io.{ByteArrayOutputStream, DataOutputStream, File}
-import java.net.{InetAddress, Socket}
+import java.net.{InetAddress, InetSocketAddress}
+import java.nio.channels.SocketChannel
 import java.nio.charset.StandardCharsets
 import java.util
 
@@ -33,6 +34,7 @@ import org.apache.hadoop.mapreduce.lib.input.FileInputFormat
 
 import org.apache.spark.{LocalSparkContext, SparkConf, SparkContext, SparkFunSuite}
 import org.apache.spark.api.java.JavaSparkContext
+import org.apache.spark.internal.config.Python.PYTHON_UNIX_DOMAIN_SOCKET_ENABLED
 import org.apache.spark.rdd.{HadoopRDD, RDD}
 import org.apache.spark.security.{SocketAuthHelper, SocketAuthServer}
 import org.apache.spark.util.Utils
@@ -76,10 +78,14 @@ class PythonRDDSuite extends SparkFunSuite with LocalSparkContext {
   }
 
   test("python server error handling") {
-    val authHelper = new SocketAuthHelper(new SparkConf())
+    val conf = new SparkConf()
+    conf.set(PYTHON_UNIX_DOMAIN_SOCKET_ENABLED.key, false.toString)
+    val authHelper = new SocketAuthHelper(conf)
     val errorServer = new ExceptionPythonServer(authHelper)
-    val client = new Socket(InetAddress.getLoopbackAddress(), errorServer.port)
-    authHelper.authToServer(client)
+    val socketChannel = SocketChannel.open(
+      new InetSocketAddress(InetAddress.getLoopbackAddress(),
+        errorServer.connInfo.asInstanceOf[Int]))
+    authHelper.authToServer(socketChannel)
     val ex = intercept[Exception] { errorServer.getResult(Duration(1, "second")) }
     assert(ex.getCause().getMessage().contains("exception within handleConnection"))
   }
@@ -87,7 +93,7 @@ class PythonRDDSuite extends SparkFunSuite with LocalSparkContext {
   class ExceptionPythonServer(authHelper: SocketAuthHelper)
       extends SocketAuthServer[Unit](authHelper, "error-server") {
 
-    override def handleConnection(sock: Socket): Unit = {
+    override def handleConnection(sock: SocketChannel): Unit = {
       throw new Exception("exception within handleConnection")
     }
   }
