@@ -21,7 +21,7 @@ import java.lang.reflect.Modifier
 import java.nio.file.{FileSystems, Paths}
 import java.util.Locale
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 import scala.util.Try
 
 import com.google.common.reflect.ClassPath
