@@ -26,6 +26,7 @@ import org.apache.hadoop.io.Text
 import org.apache.hadoop.io.compress.{CompressionCodecFactory, GzipCodec}
 
 import org.apache.spark.{SparkConf, SparkContext, SparkFunSuite}
+import org.apache.spark.io.ZStdCompressionCodec
 
 /**
  * Tests the correctness of
@@ -36,6 +37,10 @@ class WholeTextFileRecordReaderSuite extends SparkFunSuite {
   private var sc: SparkContext = _
   private var factory: CompressionCodecFactory = _
 
+  def getSparkConf(): SparkConf = {
+    new SparkConf()
+  }
+
   override def beforeAll(): Unit = {
     // Hadoop's FileSystem caching does not use the Configuration as part of its cache key, which
     // can cause Filesystem.get(Configuration) to return a cached instance created with a different
@@ -44,7 +49,7 @@ class WholeTextFileRecordReaderSuite extends SparkFunSuite {
     // the new value of "fs.local.block.size" (see SPARK-5227 and SPARK-5679). To work around this,
     // we disable FileSystem caching in this suite.
     super.beforeAll()
-    val conf = new SparkConf().set("spark.hadoop.fs.file.impl.disable.cache", "true")
+    val conf = getSparkConf().set("spark.hadoop.fs.file.impl.disable.cache", "true")
 
     sc = new SparkContext("local", "test", conf)
 
@@ -63,13 +68,25 @@ class WholeTextFileRecordReaderSuite extends SparkFunSuite {
     }
   }
 
-  private def createNativeFile(inputDir: File, fileName: String, contents: Array[Byte],
-                               compress: Boolean) = {
-    val out = if (compress) {
+  import WholeTextFileRecordReaderSuite.CompressionType
+
+  def createNativeFile(inputDir: File, fileName: String, contents: Array[Byte],
+    compressionType: CompressionType.CompressionType = CompressionType.NONE): Unit = {
+    val out = if (compressionType == CompressionType.GZIP ||
+      compressionType == CompressionType.GZ) {
       val codec = new GzipCodec
       codec.setConf(new Configuration())
-      val path = s"${inputDir.toString}/$fileName${codec.getDefaultExtension}"
+      val extension = if (compressionType == CompressionType.GZIP) {
+        ".gzip" // Try with non-standard extension
+      } else {
+        codec.getDefaultExtension
+      }
+      val path = s"${inputDir.toString}/$fileName${extension}"
       codec.createOutputStream(new DataOutputStream(new FileOutputStream(path)))
+    } else if (compressionType == CompressionType.ZSTD || compressionType == CompressionType.ZST) {
+      val extension = if (compressionType == CompressionType.ZSTD) ".zstd" else ".zst"
+      val path = s"${inputDir.toString}/${fileName}${extension}"
+      new ZStdCompressionCodec(sc.conf).compressedOutputStream(new FileOutputStream(path))
     } else {
       val path = s"${inputDir.toString}/$fileName"
       new DataOutputStream(new FileOutputStream(path))
@@ -86,48 +103,29 @@ class WholeTextFileRecordReaderSuite extends SparkFunSuite {
    *   3) Does the contents be the same.
    */
   test("Correctness of WholeTextFileRecordReader.") {
-    withTempDir { dir =>
-      logInfo(s"Local disk address is ${dir.toString}.")
+    CompressionType.values.foreach { compressionType =>
+      withTempDir { dir =>
+        logInfo(s"Local disk address is ${dir.toString}.")
 
-      WholeTextFileRecordReaderSuite.files.foreach { case (filename, contents) =>
-        createNativeFile(dir, filename, contents, false)
-      }
+        WholeTextFileRecordReaderSuite.files.foreach { case (filename, contents) =>
+          createNativeFile(dir, filename, contents, compressionType)
+        }
 
-      val res = sc.wholeTextFiles(dir.toString, 3).collect()
+        val res = sc.wholeTextFiles(dir.toString, 3).collect()
 
-      assert(res.length === WholeTextFileRecordReaderSuite.fileNames.length,
-        "Number of files read out does not fit with the actual value.")
+        assert(res.length === WholeTextFileRecordReaderSuite.fileNames.length,
+          "Number of files read out does not fit with the actual value.")
 
-      for ((filename, contents) <- res) {
-        val shortName = filename.split('/').last
-        assert(WholeTextFileRecordReaderSuite.fileNames.contains(shortName),
-          s"Missing file name $filename.")
-        assert(contents === new Text(WholeTextFileRecordReaderSuite.files(shortName)).toString,
-          s"file $filename contents can not match.")
-      }
-    }
-  }
-
-  test("Correctness of WholeTextFileRecordReader with GzipCodec.") {
-    withTempDir { dir =>
-      logInfo(s"Local disk address is ${dir.toString}.")
-
-      WholeTextFileRecordReaderSuite.files.foreach { case (filename, contents) =>
-        createNativeFile(dir, filename, contents, true)
-      }
-
-      val res = sc.wholeTextFiles(dir.toString, 3).collect()
-
-      assert(res.length === WholeTextFileRecordReaderSuite.fileNames.length,
-        "Number of files read out does not fit with the actual value.")
-
-      for ((filename, contents) <- res) {
-        val shortName = filename.split('/').last.split('.')(0)
-
-        assert(WholeTextFileRecordReaderSuite.fileNames.contains(shortName),
-          s"Missing file name $filename.")
-        assert(contents === new Text(WholeTextFileRecordReaderSuite.files(shortName)).toString,
-          s"file $filename contents can not match.")
+        for ((filename, contents) <- res) {
+          val shortName = compressionType match {
+            case CompressionType.NONE => filename.split('/').last
+            case _ => filename.split('/').last.split('.').head
+          }
+          assert(WholeTextFileRecordReaderSuite.fileNames.contains(shortName),
+            s"Missing file name $filename.")
+          assert(contents === new Text(WholeTextFileRecordReaderSuite.files(shortName)).toString,
+            s"file $filename contents can not match.")
+        }
       }
     }
   }
@@ -145,4 +143,9 @@ object WholeTextFileRecordReaderSuite {
   private val files = fileLengths.zip(fileNames).map { case (upperBound, filename) =>
     filename -> LazyList.continually(testWords.toList.to(LazyList)).flatten.take(upperBound).toArray
   }.toMap
+
+  object CompressionType extends Enumeration {
+    type CompressionType = Value
+    val NONE, GZ, GZIP, ZST, ZSTD = Value
+  }
 }
