@@ -17,7 +17,7 @@
 
 package org.apache.spark.storage
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 
 import org.apache.spark.SparkFunSuite
 
