@@ -17,16 +17,16 @@
 
 package org.apache.spark.ui
 
-import java.net.URL
+import java.net.{URI, URL}
 import java.util.Locale
-import javax.servlet.http.{HttpServletRequest, HttpServletResponse}
 
 import scala.io.Source
 import scala.xml.Node
 
-import com.gargoylesoftware.css.parser.CSSParseException
-import com.gargoylesoftware.htmlunit.DefaultCssErrorHandler
+import jakarta.servlet.http.{HttpServletRequest, HttpServletResponse}
 import org.glassfish.jersey.internal.util.collection.MultivaluedStringMap
+import org.htmlunit.DefaultCssErrorHandler
+import org.htmlunit.cssparser.parser.CSSParseException
 import org.json4s._
 import org.json4s.jackson.JsonMethods
 import org.openqa.selenium.{By, WebDriver}
@@ -46,6 +46,7 @@ import org.apache.spark.internal.config.Status._
 import org.apache.spark.internal.config.UI._
 import org.apache.spark.shuffle.FetchFailedException
 import org.apache.spark.status.api.v1.{JacksonMessageWriter, RDDDataDistribution, StageStatus}
+import org.apache.spark.tags.WebBrowserTest
 import org.apache.spark.util.Utils
 
 private[spark] class SparkUICssErrorHandler extends DefaultCssErrorHandler {
@@ -80,10 +81,11 @@ private[spark] class SparkUICssErrorHandler extends DefaultCssErrorHandler {
 /**
  * Selenium tests for the Spark Web UI.
  */
+@WebBrowserTest
 class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
 
   implicit var webDriver: WebDriver = _
-  implicit val formats = DefaultFormats
+  implicit val formats: Formats = DefaultFormats
 
 
   override def beforeAll(): Unit = {
@@ -340,7 +342,7 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
         data.dependencies.head.asInstanceOf[ShuffleDependency[_, _, _]].shuffleHandle
       // Simulate fetch failures:
       val mappedData = data.map { x =>
-        val taskContext = TaskContext.get
+        val taskContext = TaskContext.get()
         if (taskContext.taskAttemptId() == 1) {
           // Cause the post-shuffle stage to fail on its first attempt with a single task failure
           val env = SparkEnv.get
@@ -542,8 +544,8 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
     withSpark(newSparkContext(killEnabled = true)) { sc =>
       sc.parallelize(1 to 10).map{x => Thread.sleep(10000); x}.countAsync()
       eventually(timeout(5.seconds), interval(50.milliseconds)) {
-        val url = new URL(
-          sc.ui.get.webUrl.stripSuffix("/") + "/stages/stage/kill/?id=0")
+        val url = new URI(
+          sc.ui.get.webUrl.stripSuffix("/") + "/stages/stage/kill/?id=0").toURL
         // SPARK-6846: should be POST only but YARN AM doesn't proxy POST
         TestUtils.httpResponseCode(url, "GET") should be (200)
         TestUtils.httpResponseCode(url, "POST") should be (200)
@@ -555,8 +557,8 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
     withSpark(newSparkContext(killEnabled = true)) { sc =>
       sc.parallelize(1 to 10).map{x => Thread.sleep(10000); x}.countAsync()
       eventually(timeout(5.seconds), interval(50.milliseconds)) {
-        val url = new URL(
-          sc.ui.get.webUrl.stripSuffix("/") + "/jobs/job/kill/?id=0")
+        val url = new URI(
+          sc.ui.get.webUrl.stripSuffix("/") + "/jobs/job/kill/?id=0").toURL
         // SPARK-6846: should be POST only but YARN AM doesn't proxy POST
         TestUtils.httpResponseCode(url, "GET") should be (200)
         TestUtils.httpResponseCode(url, "POST") should be (200)
@@ -690,8 +692,8 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
 
   test("live UI json application list") {
     withSpark(newSparkContext()) { sc =>
-      val appListRawJson = HistoryServerSuite.getUrl(new URL(
-        sc.ui.get.webUrl + "/api/v1/applications"))
+      val appListRawJson = HistoryServerSuite.getUrl(new URI(
+        sc.ui.get.webUrl + "/api/v1/applications").toURL)
       val appListJsonAst = JsonMethods.parse(appListRawJson)
       appListJsonAst.children.length should be (1)
       val attempts = (appListJsonAst.children.head \ "attempts").children
@@ -721,32 +723,57 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
       eventually(timeout(5.seconds), interval(100.milliseconds)) {
         val stage0 = Utils.tryWithResource(Source.fromURL(sc.ui.get.webUrl +
           "/stages/stage/?id=0&attempt=0&expandDagViz=true"))(_.mkString)
-        assert(stage0.contains("digraph G {\n  subgraph clusterstage_0 {\n    " +
-          "label=&quot;Stage 0&quot;;\n    subgraph "))
-        assert(stage0.contains("{\n      label=&quot;parallelize&quot;;\n      " +
-          "0 [labelType=&quot;html&quot; label=&quot;ParallelCollectionRDD [0]"))
-        assert(stage0.contains("{\n      label=&quot;map&quot;;\n      " +
-          "1 [labelType=&quot;html&quot; label=&quot;MapPartitionsRDD [1]"))
-        assert(stage0.contains("{\n      label=&quot;groupBy&quot;;\n      " +
-          "2 [labelType=&quot;html&quot; label=&quot;MapPartitionsRDD [2]"))
+        assert(stage0.contains("""digraph G {
+                                 |  id=&quot;graph_0&quot;;
+                                 |  subgraph graph_stage_0 {
+                                 |    id=&quot;graph_stage_0&quot;;
+                                 |    isCluster=&quot;true&quot;;
+                                 |    label=&quot;Stage 0&quot;;""".stripMargin))
+        assert(stage0.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;parallelize&quot;;
+                                 |      0 [id=&quot;node_0&quot;""".stripMargin))
+        assert(stage0.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;map&quot;;
+                                 |      1 [id=&quot;node_1&quot;""".stripMargin))
+        assert(stage0.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;groupBy&quot;;
+                                 |      2 [id=&quot;node_2&quot;""".stripMargin))
 
         val stage1 = Utils.tryWithResource(Source.fromURL(sc.ui.get.webUrl +
           "/stages/stage/?id=1&attempt=0&expandDagViz=true"))(_.mkString)
-        assert(stage1.contains("digraph G {\n  subgraph clusterstage_1 {\n    " +
-          "label=&quot;Stage 1&quot;;\n    subgraph "))
-        assert(stage1.contains("{\n      label=&quot;groupBy&quot;;\n      " +
-          "3 [labelType=&quot;html&quot; label=&quot;ShuffledRDD [3]"))
-        assert(stage1.contains("{\n      label=&quot;map&quot;;\n      " +
-          "4 [labelType=&quot;html&quot; label=&quot;MapPartitionsRDD [4]"))
-        assert(stage1.contains("{\n      label=&quot;groupBy&quot;;\n      " +
-          "5 [labelType=&quot;html&quot; label=&quot;MapPartitionsRDD [5]"))
+        assert(stage1.contains("""digraph G {
+                                 |  id=&quot;graph_1&quot;;
+                                 |  subgraph graph_stage_1 {
+                                 |    id=&quot;graph_stage_1&quot;;
+                                 |    isCluster=&quot;true&quot;;
+                                 |    label=&quot;Stage 1&quot;;""".stripMargin))
+        assert(stage1.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;groupBy&quot;;""".stripMargin))
+        assert(stage1.contains(
+          "3 [id=&quot;node_3&quot; labelType=&quot;html&quot; label=&quot;ShuffledRDD"))
+        assert(stage1.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;map&quot;;""".stripMargin))
+        assert(stage1.contains(
+          "4 [id=&quot;node_4&quot; labelType=&quot;html&quot; label=&quot;MapPartitionsRDD [4]"))
 
         val stage2 = Utils.tryWithResource(Source.fromURL(sc.ui.get.webUrl +
           "/stages/stage/?id=2&attempt=0&expandDagViz=true"))(_.mkString)
-        assert(stage2.contains("digraph G {\n  subgraph clusterstage_2 {\n    " +
-          "label=&quot;Stage 2&quot;;\n    subgraph "))
-        assert(stage2.contains("{\n      label=&quot;groupBy&quot;;\n      " +
-          "6 [labelType=&quot;html&quot; label=&quot;ShuffledRDD [6]"))
+        assert(stage2.contains("""digraph G {
+                                 |  id=&quot;graph_2&quot;;
+                                 |  subgraph graph_stage_2 {
+                                 |    id=&quot;graph_stage_2&quot;;
+                                 |    isCluster=&quot;true&quot;;
+                                 |    label=&quot;Stage 2&quot;;""".stripMargin))
+        assert(stage2.contains("""
+                                 |      isCluster=&quot;true&quot;;
+                                 |      label=&quot;groupBy&quot;;""".stripMargin))
+        assert(stage2.contains(
+          "6 [id=&quot;node_6&quot; labelType=&quot;html&quot; label=&quot;ShuffledRDD [6]"))
       }
     }
   }
@@ -795,10 +822,10 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
 
   test("description for empty jobs") {
     withSpark(newSparkContext()) { sc =>
-      sc.emptyRDD[Int].collect
+      sc.emptyRDD[Int].collect()
       val description = "This is my job"
       sc.setJobDescription(description)
-      sc.emptyRDD[Int].collect
+      sc.emptyRDD[Int].collect()
 
       eventually(timeout(10.seconds), interval(50.milliseconds)) {
         goToUi(sc, "/jobs")
@@ -860,6 +887,20 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
     }
   }
 
+  test("SPARK-44895: Add 'daemon', 'priority' for ThreadStackTrace") {
+    withSpark(newSparkContext()) { sc =>
+      val uiThreads = getJson(sc.ui.get, "executors/driver/threads")
+        .children
+        .filter(v => (v \ "threadName").extract[String].matches("SparkUI-\\d+"))
+      val priority = Thread.currentThread().getPriority
+
+      uiThreads.foreach { v =>
+        assert((v \ "isDaemon").extract[Boolean])
+        assert((v \ "priority").extract[Int] === priority)
+      }
+    }
+  }
+
   def goToUi(sc: SparkContext, path: String): Unit = {
     goToUi(sc.ui.get, path)
   }
@@ -877,6 +918,6 @@ class UISeleniumSuite extends SparkFunSuite with WebBrowser with Matchers {
   }
 
   def apiUrl(ui: SparkUI, path: String): URL = {
-    new URL(ui.webUrl + "/api/v1/applications/" + ui.sc.get.applicationId + "/" + path)
+    new URI(ui.webUrl + "/api/v1/applications/" + ui.sc.get.applicationId + "/" + path).toURL
   }
 }
