@@ -118,16 +118,15 @@ class UninterruptibleThreadSuite extends SparkFunSuite {
 
   test("no runUninterruptibly") {
     @volatile var hasInterruptedException = false
-    val latch = new CountDownLatch(1)
     val t = new UninterruptibleThread("test") {
       override def run(): Unit = {
-        latch.countDown()
-        hasInterruptedException = sleep(1)
+        if (sleep(0)) {
+          hasInterruptedException = true
+        }
       }
     }
-    t.start()
-    latch.await(10, TimeUnit.SECONDS)
     t.interrupt()
+    t.start()
     t.join()
     assert(hasInterruptedException === true)
   }
