@@ -35,5 +35,5 @@ ERROR_CODES = {
     "BLOCK_BUILD_TESTS": 22,
     "BLOCK_PYSPARK_PIP_TESTS": 23,
     "BLOCK_SCALA_VERSION": 24,
-    "BLOCK_TIMEOUT": 124
+    "BLOCK_TIMEOUT": 124,
 }
