@@ -20,7 +20,7 @@ package org.apache.spark.graphx
 import scala.reflect.ClassTag
 
 import org.apache.spark.graphx.util.PeriodicGraphCheckpointer
-import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.Logging
 import org.apache.spark.internal.LogKeys.NUM_ITERATIONS
 import org.apache.spark.rdd.RDD
 import org.apache.spark.rdd.util.PeriodicRDDCheckpointer
