@@ -17,29 +17,37 @@
 
 package org.apache.spark.ml.linalg
 
-import com.github.fommil.netlib.{BLAS => NetlibBLAS, F2jBLAS}
-import com.github.fommil.netlib.BLAS.{getInstance => NativeBLAS}
+import dev.ludovic.netlib.blas.{BLAS => NetlibBLAS, JavaBLAS => NetlibJavaBLAS, NativeBLAS => NetlibNativeBLAS}
 
 /**
  * BLAS routines for MLlib's vectors and matrices.
  */
 private[spark] object BLAS extends Serializable {
 
-  @transient private var _f2jBLAS: NetlibBLAS = _
+  @transient private var _javaBLAS: NetlibBLAS = _
   @transient private var _nativeBLAS: NetlibBLAS = _
   private val nativeL1Threshold: Int = 256
 
-  // For level-1 function dspmv, use f2jBLAS for better performance.
-  private[ml] def f2jBLAS: NetlibBLAS = {
-    if (_f2jBLAS == null) {
-      _f2jBLAS = new F2jBLAS
+  // For level-1 function dspmv, use javaBLAS for better performance.
+  private[spark] def javaBLAS: NetlibBLAS = {
+    if (_javaBLAS == null) {
+      _javaBLAS = NetlibJavaBLAS.getInstance
     }
-    _f2jBLAS
+    _javaBLAS
   }
 
-  private[ml] def getBLAS(vectorSize: Int): NetlibBLAS = {
+  // For level-3 routines, we use the native BLAS.
+  private[spark] def nativeBLAS: NetlibBLAS = {
+    if (_nativeBLAS == null) {
+      _nativeBLAS =
+        try { NetlibNativeBLAS.getInstance } catch { case _: Throwable => javaBLAS }
+    }
+    _nativeBLAS
+  }
+
+  private[spark] def getBLAS(vectorSize: Int): NetlibBLAS = {
     if (vectorSize < nativeL1Threshold) {
-      f2jBLAS
+      javaBLAS
     } else {
       nativeBLAS
     }
@@ -235,14 +243,6 @@ private[spark] object BLAS extends Serializable {
     }
   }
 
-  // For level-3 routines, we use the native BLAS.
-  private[ml] def nativeBLAS: NetlibBLAS = {
-    if (_nativeBLAS == null) {
-      _nativeBLAS = NativeBLAS
-    }
-    _nativeBLAS
-  }
-
   /**
    * Adds alpha * x * x.t to a matrix in-place. This is the same as BLAS's ?SPR.
    *
@@ -267,11 +267,11 @@ private[spark] object BLAS extends Serializable {
       x: DenseVector,
       beta: Double,
       y: DenseVector): Unit = {
-    f2jBLAS.dspmv("U", n, alpha, A.values, x.values, 1, beta, y.values, 1)
+    javaBLAS.dspmv("U", n, alpha, A.values, x.values, 1, beta, y.values, 1)
   }
 
   /**
-   * Adds alpha * x * x.t to a matrix in-place. This is the same as BLAS's ?SPR.
+   * Adds alpha * v * v.t to a matrix in-place. This is the same as BLAS's ?SPR.
    *
    * @param U the upper triangular part of the matrix packed in an array (column major)
    */
@@ -279,7 +279,7 @@ private[spark] object BLAS extends Serializable {
     val n = v.size
     v match {
       case DenseVector(values) =>
-        NativeBLAS.dspr("U", n, alpha, values, 1, U)
+        nativeBLAS.dspr("U", n, alpha, values, 1, U)
       case SparseVector(size, indices, values) =>
         val nnz = indices.length
         var colStartIdx = 0
@@ -302,6 +302,8 @@ private[spark] object BLAS extends Serializable {
           j += 1
           prevCol = col
         }
+      case _ =>
+        throw new IllegalArgumentException(s"spr doesn't support vector type ${v.getClass}.")
     }
   }
 
@@ -381,7 +383,6 @@ private[spark] object BLAS extends Serializable {
       "The matrix C cannot be the product of a transpose() call. C.isTransposed must be false.")
     if (alpha == 0.0 && beta == 1.0) {
       // gemm: alpha is equal to 0 and beta is equal to 1. Returning C.
-      return
     } else if (alpha == 0.0) {
       getBLAS(C.values.length).dscal(C.values.length, beta, C.values, 1)
     } else {
@@ -394,6 +395,37 @@ private[spark] object BLAS extends Serializable {
     }
   }
 
+  /**
+   * CValues[0: A.numRows * B.numCols] := alpha * A * B + beta * CValues[0: A.numRows * B.numCols]
+   * @param alpha a scalar to scale the multiplication A * B.
+   * @param A the matrix A that will be left multiplied to B. Size of m x k.
+   * @param B the matrix B that will be left multiplied by A. Size of k x n.
+   * @param beta a scalar that can be used to scale matrix C.
+   * @param CValues the values of matrix C. C.isTransposed is supposed to be false.
+   */
+  def gemm(
+      alpha: Double,
+      A: Matrix,
+      B: DenseMatrix,
+      beta: Double,
+      CValues: Array[Double]): Unit = {
+    require(A.numRows * B.numCols <= CValues.length,
+      s"the length of CValues must be no less than ${A.numRows} X ${B.numCols}")
+    if (alpha == 0.0 && beta == 1.0) {
+      // gemm: alpha is equal to 0 and beta is equal to 1. Returning C.
+    } else if (alpha == 0.0) {
+      val n = A.numRows * B.numCols
+      getBLAS(n).dscal(n, beta, CValues, 1)
+    } else {
+      A match {
+        case sparse: SparseMatrix => gemmImpl(alpha, sparse, B, beta, CValues)
+        case dense: DenseMatrix => gemmImpl(alpha, dense, B, beta, CValues)
+        case _ =>
+          throw new IllegalArgumentException(s"gemm doesn't support matrix type ${A.getClass}.")
+      }
+    }
+  }
+
   /**
    * C := alpha * A * B + beta * C
    * For `DenseMatrix` A.
@@ -404,6 +436,25 @@ private[spark] object BLAS extends Serializable {
       B: DenseMatrix,
       beta: Double,
       C: DenseMatrix): Unit = {
+    require(A.numCols == B.numRows,
+      s"The columns of A don't match the rows of B. A: ${A.numCols}, B: ${B.numRows}")
+    require(A.numRows == C.numRows,
+      s"The rows of C don't match the rows of A. C: ${C.numRows}, A: ${A.numRows}")
+    require(B.numCols == C.numCols,
+      s"The columns of C don't match the columns of B. C: ${C.numCols}, A: ${B.numCols}")
+    gemmImpl(alpha, A, B, beta, C.values)
+  }
+
+  /**
+   * CValues[0: A.numRows * B.numCols] := alpha * A * B + beta * CValues[0: A.numRows * B.numCols]
+   * For `DenseMatrix` A.
+   */
+  private def gemmImpl(
+      alpha: Double,
+      A: DenseMatrix,
+      B: DenseMatrix,
+      beta: Double,
+      CValues: Array[Double]): Unit = {
     val tAstr = if (A.isTransposed) "T" else "N"
     val tBstr = if (B.isTransposed) "T" else "N"
     val lda = if (!A.isTransposed) A.numRows else A.numCols
@@ -411,12 +462,9 @@ private[spark] object BLAS extends Serializable {
 
     require(A.numCols == B.numRows,
       s"The columns of A don't match the rows of B. A: ${A.numCols}, B: ${B.numRows}")
-    require(A.numRows == C.numRows,
-      s"The rows of C don't match the rows of A. C: ${C.numRows}, A: ${A.numRows}")
-    require(B.numCols == C.numCols,
-      s"The columns of C don't match the columns of B. C: ${C.numCols}, A: ${B.numCols}")
+
     nativeBLAS.dgemm(tAstr, tBstr, A.numRows, B.numCols, A.numCols, alpha, A.values, lda,
-      B.values, ldb, beta, C.values, C.numRows)
+      B.values, ldb, beta, CValues, A.numRows)
   }
 
   /**
@@ -439,9 +487,29 @@ private[spark] object BLAS extends Serializable {
     require(nB == C.numCols,
       s"The columns of C don't match the columns of B. C: ${C.numCols}, A: $nB")
 
+    gemmImpl(alpha, A, B, beta, C.values)
+  }
+
+  /**
+   * CValues[0: A.numRows * B.numCols] := alpha * A * B + beta * CValues[0: A.numRows * B.numCols]
+   * For `SparseMatrix` A.
+   */
+  private def gemmImpl(
+      alpha: Double,
+      A: SparseMatrix,
+      B: DenseMatrix,
+      beta: Double,
+      CValues: Array[Double]): Unit = {
+    val mA: Int = A.numRows
+    val nB: Int = B.numCols
+    val kA: Int = A.numCols
+    val kB: Int = B.numRows
+
+    require(kA == kB, s"The columns of A don't match the rows of B. A: $kA, B: $kB")
+
     val Avals = A.values
     val Bvals = B.values
-    val Cvals = C.values
+    val Cvals = CValues
     val ArowIndices = A.rowIndices
     val AcolPtrs = A.colPtrs
 
@@ -476,7 +544,7 @@ private[spark] object BLAS extends Serializable {
             val indEnd = AcolPtrs(rowCounterForA + 1)
             var sum = 0.0
             while (i < indEnd) {
-              sum += Avals(i) * B(ArowIndices(i), colCounterForB)
+              sum += Avals(i) * Bvals(colCounterForB + nB * ArowIndices(i))
               i += 1
             }
             val Cindex = Cstart + rowCounterForA
@@ -489,7 +557,8 @@ private[spark] object BLAS extends Serializable {
     } else {
       // Scale matrix first if `beta` is not equal to 1.0
       if (beta != 1.0) {
-        getBLAS(C.values.length).dscal(C.values.length, beta, C.values, 1)
+        val n = A.numRows * B.numCols
+        getBLAS(n).dscal(n, beta, Cvals, 1)
       }
       // Perform matrix multiplication and add to C. The rows of A are multiplied by the columns of
       // B, and added to C.
@@ -518,7 +587,7 @@ private[spark] object BLAS extends Serializable {
           while (colCounterForA < kA) {
             var i = AcolPtrs(colCounterForA)
             val indEnd = AcolPtrs(colCounterForA + 1)
-            val Bval = B(colCounterForA, colCounterForB) * alpha
+            val Bval = Bvals(colCounterForB + nB * colCounterForA) * alpha
             while (i < indEnd) {
               Cvals(Cstart + ArowIndices(i)) += Avals(i) * Bval
               i += 1
@@ -531,6 +600,31 @@ private[spark] object BLAS extends Serializable {
     }
   }
 
+  /**
+   * y[0: A.numRows] := alpha * A * x[0: A.numCols] + beta * y[0: A.numRows]
+   */
+  def gemv(
+      alpha: Double,
+      A: Matrix,
+      x: Array[Double],
+      beta: Double,
+      y: Array[Double]): Unit = {
+    require(A.numCols <= x.length,
+      s"The columns of A don't match the number of elements of x. A: ${A.numCols}, x: ${x.length}")
+    require(A.numRows <= y.length,
+      s"The rows of A don't match the number of elements of y. A: ${A.numRows}, y:${y.length}")
+    if (alpha == 0.0 && beta == 1.0) {
+      // gemv: alpha is equal to 0 and beta is equal to 1. Returning y.
+    } else if (alpha == 0.0) {
+      getBLAS(A.numRows).dscal(A.numRows, beta, y, 1)
+    } else {
+      A match {
+        case smA: SparseMatrix => gemvImpl(alpha, smA, x, beta, y)
+        case dmA: DenseMatrix => gemvImpl(alpha, dmA, x, beta, y)
+      }
+    }
+  }
+
   /**
    * y := alpha * A * x + beta * y
    * @param alpha a scalar to scale the multiplication A * x.
@@ -551,7 +645,6 @@ private[spark] object BLAS extends Serializable {
       s"The rows of A don't match the number of elements of y. A: ${A.numRows}, y:${y.size}")
     if (alpha == 0.0 && beta == 1.0) {
       // gemv: alpha is equal to 0 and beta is equal to 1. Returning y.
-      return
     } else if (alpha == 0.0) {
       scal(beta, y)
     } else {
@@ -581,11 +674,24 @@ private[spark] object BLAS extends Serializable {
       x: DenseVector,
       beta: Double,
       y: DenseVector): Unit = {
+    gemvImpl(alpha, A, x.values, beta, y.values)
+  }
+
+  /**
+   * y[0: A.numRows] := alpha * A * x[0: A.numCols] + beta * y[0: A.numRows]
+   * For `DenseMatrix` A.
+   */
+  private def gemvImpl(
+      alpha: Double,
+      A: DenseMatrix,
+      xValues: Array[Double],
+      beta: Double,
+      yValues: Array[Double]): Unit = {
     val tStrA = if (A.isTransposed) "T" else "N"
     val mA = if (!A.isTransposed) A.numRows else A.numCols
     val nA = if (!A.isTransposed) A.numCols else A.numRows
-    nativeBLAS.dgemv(tStrA, mA, nA, alpha, A.values, mA, x.values, 1, beta,
-      y.values, 1)
+    nativeBLAS.dgemv(tStrA, mA, nA, alpha, A.values, mA, xValues, 1, beta,
+      yValues, 1)
   }
 
   /**
@@ -691,7 +797,6 @@ private[spark] object BLAS extends Serializable {
 
           val xTemp = xValues(k) * alpha
           while (i < indEnd) {
-            val rowIndex = Arows(i)
             yValues(Arows(i)) += Avals(i) * xTemp
             i += 1
           }
@@ -712,8 +817,19 @@ private[spark] object BLAS extends Serializable {
       x: DenseVector,
       beta: Double,
       y: DenseVector): Unit = {
-    val xValues = x.values
-    val yValues = y.values
+    gemvImpl(alpha, A, x.values, beta, y.values)
+  }
+
+  /**
+   * y[0: A.numRows] := alpha * A * x[0: A.numCols] + beta * y[0: A.numRows]
+   * For `SparseMatrix` A.
+   */
+  private def gemvImpl(
+      alpha: Double,
+      A: SparseMatrix,
+      xValues: Array[Double],
+      beta: Double,
+      yValues: Array[Double]): Unit = {
     val mA: Int = A.numRows
     val nA: Int = A.numCols
 
@@ -735,7 +851,7 @@ private[spark] object BLAS extends Serializable {
         rowCounter += 1
       }
     } else {
-      if (beta != 1.0) scal(beta, y)
+      if (beta != 1.0) getBLAS(mA).dscal(mA, beta, yValues, 1)
       // Perform matrix-vector multiplication and add to y
       var colCounterForA = 0
       while (colCounterForA < nA) {
@@ -743,8 +859,7 @@ private[spark] object BLAS extends Serializable {
         val indEnd = Acols(colCounterForA + 1)
         val xVal = xValues(colCounterForA) * alpha
         while (i < indEnd) {
-          val rowIndex = Arows(i)
-          yValues(rowIndex) += Avals(i) * xVal
+          yValues(Arows(i)) += Avals(i) * xVal
           i += 1
         }
         colCounterForA += 1
