@@ -30,6 +30,7 @@ import org.apache.spark.mllib.tree.{DecisionTreeSuite => OldDTSuite, EnsembleTes
 import org.apache.spark.mllib.tree.configuration.{Algo => OldAlgo, QuantileStrategy, Strategy => OldStrategy}
 import org.apache.spark.mllib.tree.impurity.{Entropy, Gini, GiniCalculator, Variance}
 import org.apache.spark.mllib.util.MLlibTestSparkContext
+import org.apache.spark.util.ArrayImplicits._
 import org.apache.spark.util.collection.OpenHashMap
 
 /**
@@ -46,7 +47,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("Binary classification with continuous features: split calculation") {
     val arr = OldDTSuite.generateOrderedLabeledPointsWithLabel1().map(_.asML.toInstance)
     assert(arr.length === 1000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     val strategy = new OldStrategy(OldAlgo.Classification, Gini, 3, 2, 100)
     val metadata = DecisionTreeMetadata.buildMetadata(rdd, strategy)
     assert(!metadata.isUnordered(featureIndex = 0))
@@ -58,7 +59,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("Binary classification with binary (ordered) categorical features: split calculation") {
     val arr = OldDTSuite.generateCategoricalDataPoints().map(_.asML.toInstance)
     assert(arr.length === 1000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     val strategy = new OldStrategy(OldAlgo.Classification, Gini, maxDepth = 2, numClasses = 2,
       maxBins = 100, categoricalFeaturesInfo = Map(0 -> 2, 1 -> 2))
 
@@ -75,7 +76,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
     " with no samples for one category: split calculation") {
     val arr = OldDTSuite.generateCategoricalDataPoints().map(_.asML.toInstance)
     assert(arr.length === 1000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     val strategy = new OldStrategy(OldAlgo.Classification, Gini, maxDepth = 2, numClasses = 2,
       maxBins = 100, categoricalFeaturesInfo = Map(0 -> 3, 1 -> 3))
 
@@ -96,7 +97,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
         Array(6), Gini, QuantileStrategy.Sort,
         0, 0, 0.0, 0.0, 0, 0
       )
-      val featureSamples = Array.fill(10000)((1.0, math.random)).filter(_._2 != 0.0)
+      val featureSamples = Array.fill(10000)((1.0, math.random())).filter(_._2 != 0.0)
       val splits = RandomForest.findSplitsForContinuousFeature(featureSamples, fakeMetadata, 0)
       assert(splits.length === 5)
       assert(fakeMetadata.numSplits(0) === 5)
@@ -183,7 +184,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
         Array(6), Gini, QuantileStrategy.Sort,
         0, 0, 0.0, 0.0, 0, 0
       )
-      val featureSamplesUnitWeight = Array.fill(10)((1.0, math.random))
+      val featureSamplesUnitWeight = Array.fill(10)((1.0, math.random()))
       val featureSamplesSmallWeight = featureSamplesUnitWeight.map { case (w, x) => (w * 0.001, x)}
       val featureSamplesLargeWeight = featureSamplesUnitWeight.map { case (w, x) => (w * 1000, x)}
       val splitsUnitWeight = RandomForest
@@ -214,7 +215,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("train with empty arrays") {
     val lp = LabeledPoint(1.0, Vectors.dense(Array.empty[Double])).toInstance
     val data = Array.fill(5)(lp)
-    val rdd = sc.parallelize(data)
+    val rdd = sc.parallelize(data.toImmutableArraySeq)
 
     val strategy = new OldStrategy(OldAlgo.Regression, Gini, maxDepth = 2,
       maxBins = 5)
@@ -229,7 +230,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("train with constant features") {
     val instance = LabeledPoint(1.0, Vectors.dense(0.0, 0.0, 0.0)).toInstance
     val data = Array.fill(5)(instance)
-    val rdd = sc.parallelize(data)
+    val rdd = sc.parallelize(data.toImmutableArraySeq)
     val strategy = new OldStrategy(
           OldAlgo.Classification,
           Gini,
@@ -257,7 +258,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("Multiclass classification with unordered categorical features: split calculations") {
     val arr = OldDTSuite.generateCategoricalDataPoints().map(_.asML.toInstance)
     assert(arr.length === 1000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     val strategy = new OldStrategy(
       OldAlgo.Classification,
       Gini,
@@ -299,7 +300,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
     val arr = OldDTSuite.generateCategoricalDataPointsForMulticlassForOrderedFeatures()
       .map(_.asML.toInstance)
     assert(arr.length === 3000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     val strategy = new OldStrategy(OldAlgo.Classification, Gini, maxDepth = 2, numClasses = 100,
       maxBins = 100, categoricalFeaturesInfo = Map(0 -> 10, 1 -> 10))
     // 2^(10-1) - 1 > 100, so categorical features will be ordered
@@ -329,7 +330,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       LabeledPoint(1.0, Vectors.dense(0.0, 1.0, 1.0)),
       LabeledPoint(0.0, Vectors.dense(2.0, 0.0, 0.0)),
       LabeledPoint(1.0, Vectors.dense(0.0, 2.0, 1.0)))
-    val input = sc.parallelize(arr.map(_.toInstance))
+    val input = sc.parallelize(arr.map(_.toInstance).toImmutableArraySeq)
 
     val strategy = new OldStrategy(algo = OldAlgo.Classification, impurity = Gini, maxDepth = 1,
       numClasses = 2, categoricalFeaturesInfo = Map(0 -> 3))
@@ -373,7 +374,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       LabeledPoint(1.0, Vectors.dense(0.0, 1.0, 1.0)),
       LabeledPoint(0.0, Vectors.dense(2.0, 0.0, 0.0)),
       LabeledPoint(1.0, Vectors.dense(0.0, 2.0, 1.0)))
-    val input = sc.parallelize(arr.map(_.toInstance))
+    val input = sc.parallelize(arr.map(_.toInstance).toImmutableArraySeq)
 
     val strategy = new OldStrategy(algo = OldAlgo.Classification, impurity = Gini, maxDepth = 5,
       numClasses = 2, categoricalFeaturesInfo = Map(0 -> 3))
@@ -427,7 +428,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       LabeledPoint(0.0, Vectors.dense(2.0)),
       LabeledPoint(0.0, Vectors.dense(2.0)),
       LabeledPoint(1.0, Vectors.dense(2.0)))
-    val input = sc.parallelize(arr.map(_.toInstance))
+    val input = sc.parallelize(arr.map(_.toInstance).toImmutableArraySeq)
 
     // Must set maxBins s.t. the feature will be treated as an ordered categorical feature.
     val strategy = new OldStrategy(algo = OldAlgo.Classification, impurity = Gini, maxDepth = 1,
@@ -449,7 +450,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
   test("Second level node building with vs. without groups") {
     val arr = OldDTSuite.generateOrderedLabeledPoints().map(_.asML.toInstance)
     assert(arr.length === 1000)
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
     // For tree with 1 group
     val strategy1 =
       new OldStrategy(OldAlgo.Classification, Entropy, 3, 2, 100, maxMemoryInMB = 1000)
@@ -492,7 +493,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       strategy: OldStrategy): Unit = {
     val numFeatures = 50
     val arr = EnsembleTestHelper.generateOrderedLabeledPoints(numFeatures, 1000)
-    val rdd = sc.parallelize(arr).map(_.asML.toInstance)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq).map(_.asML.toInstance)
 
     // Select feature subset for top nodes.  Return true if OK.
     def checkFeatureSubsetStrategy(
@@ -678,7 +679,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       Instance(0.0, 1.0, Vectors.dense(1.0, 0.0)),
       Instance(1.0, 1.0, Vectors.dense(1.0, 1.0))
     )
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
 
     val numClasses = 2
     val strategy = new OldStrategy(algo = OldAlgo.Classification, impurity = Gini, maxDepth = 4,
@@ -693,7 +694,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
     assert(prunedTree.numNodes === 5)
     assert(unprunedTree.numNodes === 7)
 
-    assert(RandomForestSuite.getSumLeafCounters(List(prunedTree.rootNode)) === arr.size)
+    assert(RandomForestSuite.getSumLeafCounters(List(prunedTree.rootNode)) === arr.length)
   }
 
   test("SPARK-3159 tree model redundancy - regression") {
@@ -709,7 +710,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       Instance(0.0, 1.0, Vectors.dense(1.0, 1.0)),
       Instance(0.5, 1.0, Vectors.dense(1.0, 1.0))
     )
-    val rdd = sc.parallelize(arr)
+    val rdd = sc.parallelize(arr.toImmutableArraySeq)
 
     val strategy = new OldStrategy(algo = OldAlgo.Regression, impurity = Variance, maxDepth = 4,
       numClasses = 0, maxBins = 32)
@@ -722,12 +723,12 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
 
     assert(prunedTree.numNodes === 3)
     assert(unprunedTree.numNodes === 5)
-    assert(RandomForestSuite.getSumLeafCounters(List(prunedTree.rootNode)) === arr.size)
+    assert(RandomForestSuite.getSumLeafCounters(List(prunedTree.rootNode)) === arr.length)
   }
 
   test("weights at arbitrary scale") {
     val arr = EnsembleTestHelper.generateOrderedLabeledPoints(3, 10)
-    val rddWithUnitWeights = sc.parallelize(arr.map(_.asML.toInstance))
+    val rddWithUnitWeights = sc.parallelize(arr.map(_.asML.toInstance).toImmutableArraySeq)
     val rddWithSmallWeights = rddWithUnitWeights.map { inst =>
       Instance(inst.label, 0.001, inst.features)
     }
@@ -756,7 +757,7 @@ class RandomForestSuite extends SparkFunSuite with MLlibTestSparkContext {
       Instance(0.0, 1.0, Vectors.dense(0.0)),
       Instance(1.0, 0.1, Vectors.dense(1.0))
     )
-    val rdd = sc.parallelize(data)
+    val rdd = sc.parallelize(data.toImmutableArraySeq)
     val strategy = new OldStrategy(OldAlgo.Classification, Gini, 3, 2,
       minWeightFractionPerNode = 0.5)
     val Array(tree1) = RandomForest.run(rdd, strategy, 1, "all", 42L, None)
