@@ -16,25 +16,23 @@
 #
 
 """Spark Connect Python Client - ML module"""
-
 from pyspark.ml.connect.base import (
     Estimator,
     Transformer,
     Model,
 )
-
 from pyspark.ml.connect import (
     feature,
     evaluation,
     tuning,
 )
-
+from pyspark.ml.connect.evaluation import Evaluator
 from pyspark.ml.connect.pipeline import Pipeline, PipelineModel
 
 __all__ = [
     "Estimator",
     "Transformer",
-    "Estimator",
+    "Evaluator",
     "Model",
     "feature",
     "evaluation",
