@@ -14,12 +14,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from typing import overload, Optional, Dict
-
-from py4j.java_gateway import JavaObject, JVMView
+from typing import overload, Optional, Dict, TYPE_CHECKING
 
 from pyspark.util import _parse_memory
 
+if TYPE_CHECKING:
+    from py4j.java_gateway import JavaObject, JVMView
+
 
 class ExecutorResourceRequest:
     """
@@ -147,7 +148,7 @@ class ExecutorResourceRequests:
     _OFFHEAP_MEM = "offHeap"
 
     @overload
-    def __init__(self, _jvm: JVMView):
+    def __init__(self, _jvm: "JVMView"):
         ...
 
     @overload
@@ -160,16 +161,21 @@ class ExecutorResourceRequests:
 
     def __init__(
         self,
-        _jvm: Optional[JVMView] = None,
+        _jvm: Optional["JVMView"] = None,
         _requests: Optional[Dict[str, ExecutorResourceRequest]] = None,
     ):
-        from pyspark import SparkContext
+        from pyspark.sql import is_remote
 
-        _jvm = _jvm or SparkContext._jvm
-        if _jvm is not None:
-            self._java_executor_resource_requests = (
-                _jvm.org.apache.spark.resource.ExecutorResourceRequests()
-            )
+        jvm = None
+        if not is_remote():
+            from pyspark.core.context import SparkContext
+
+            jvm = _jvm or SparkContext._jvm
+
+        if jvm is not None:
+            self._java_executor_resource_requests = getattr(
+                jvm, "org.apache.spark.resource.ExecutorResourceRequests"
+            )()
             if _requests is not None:
                 for k, v in _requests.items():
                     if k == self._MEMORY:
@@ -443,7 +449,7 @@ class TaskResourceRequests:
     _CPUS = "cpus"
 
     @overload
-    def __init__(self, _jvm: JVMView):
+    def __init__(self, _jvm: "JVMView"):
         ...
 
     @overload
@@ -456,16 +462,21 @@ class TaskResourceRequests:
 
     def __init__(
         self,
-        _jvm: Optional[JVMView] = None,
+        _jvm: Optional["JVMView"] = None,
         _requests: Optional[Dict[str, TaskResourceRequest]] = None,
     ):
-        from pyspark import SparkContext
+        from pyspark.sql import is_remote
 
-        _jvm = _jvm or SparkContext._jvm
-        if _jvm is not None:
-            self._java_task_resource_requests: Optional[
-                JavaObject
-            ] = _jvm.org.apache.spark.resource.TaskResourceRequests()
+        jvm = None
+        if not is_remote():
+            from pyspark.core.context import SparkContext
+
+            jvm = _jvm or SparkContext._jvm
+
+        if jvm is not None:
+            self._java_task_resource_requests: Optional["JavaObject"] = getattr(
+                jvm, "org.apache.spark.resource.TaskResourceRequests"
+            )()
             if _requests is not None:
                 for k, v in _requests.items():
                     if k == self._CPUS:
