@@ -14,28 +14,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import itertools
 import sys
-from typing import Any, Iterator, List, Optional, Union, TYPE_CHECKING, cast
+from typing import List, Optional, Union, TYPE_CHECKING, cast, Any
 import warnings
 
 from pyspark.errors import PySparkTypeError
 from pyspark.util import PythonEvalType
 from pyspark.sql.column import Column
 from pyspark.sql.dataframe import DataFrame
+from pyspark.sql.pandas.typehints import infer_group_arrow_eval_type_from_func
 from pyspark.sql.streaming.state import GroupStateTimeout
-from pyspark.sql.streaming.stateful_processor_api_client import (
-    StatefulProcessorApiClient,
-    StatefulProcessorHandleState,
-)
-from pyspark.sql.streaming.stateful_processor import (
-    ExpiredTimerInfo,
-    StatefulProcessor,
-    StatefulProcessorHandle,
-    TimerValues,
-)
-from pyspark.sql.streaming.stateful_processor import StatefulProcessor, StatefulProcessorHandle
-from pyspark.sql.streaming.stateful_processor_util import TransformWithStateInPandasFuncMode
+from pyspark.sql.streaming.stateful_processor import StatefulProcessor
 from pyspark.sql.types import StructType
 
 if TYPE_CHECKING:
@@ -46,7 +35,6 @@ if TYPE_CHECKING:
         PandasCogroupedMapFunction,
         ArrowGroupedMapFunction,
         ArrowCogroupedMapFunction,
-        DataFrameLike as PandasDataFrameLike,
     )
     from pyspark.sql.group import GroupedData
 
@@ -135,12 +123,13 @@ class PandasGroupedOpsMixin:
         Maps each group of the current :class:`DataFrame` using a pandas udf and returns the result
         as a `DataFrame`.
 
-        The function should take a `pandas.DataFrame` and return another
-        `pandas.DataFrame`. Alternatively, the user can pass a function that takes
-        a tuple of the grouping key(s) and a `pandas.DataFrame`.
-        For each group, all columns are passed together as a `pandas.DataFrame`
-        to the user-function and the returned `pandas.DataFrame` are combined as a
-        :class:`DataFrame`.
+        The function can take one of two forms: It can take a `pandas.DataFrame` and return a
+        `pandas.DataFrame`, or it can take an iterator of `pandas.DataFrame` and yield
+        `pandas.DataFrame`. Alternatively each form can take a tuple of grouping keys
+        as the first argument in addition to the input type above.
+        For each group, all columns are passed together as a `pandas.DataFrame` or iterator of
+        `pandas.DataFrame`, and the returned `pandas.DataFrame` or iterator of `pandas.DataFrame`
+        are combined as a :class:`DataFrame`.
 
         The `schema` should be a :class:`StructType` describing the schema of the returned
         `pandas.DataFrame`. The column labels of the returned `pandas.DataFrame` must either match
@@ -153,12 +142,17 @@ class PandasGroupedOpsMixin:
         .. versionchanged:: 3.4.0
             Support Spark Connect.
 
+        .. versionchanged:: 4.1.0
+            Added support for an iterator of `pandas.DataFrame` API.
+
         Parameters
         ----------
         func : function
-            a Python native function that takes a `pandas.DataFrame` and outputs a
-            `pandas.DataFrame`, or that takes one tuple (grouping keys) and a
-            `pandas.DataFrame` and outputs a `pandas.DataFrame`.
+            a Python native function that either takes a `pandas.DataFrame` and outputs a
+            `pandas.DataFrame` or takes an iterator of `pandas.DataFrame` and yields
+            `pandas.DataFrame`. Additionally, each form can take a tuple of grouping keys
+            as the first argument, with the `pandas.DataFrame` or iterator of `pandas.DataFrame`
+            as the second argument.
         schema : :class:`pyspark.sql.types.DataType` or str
             the return type of the `func` in PySpark. The value can be either a
             :class:`pyspark.sql.types.DataType` object or a DDL-formatted type string.
@@ -226,22 +220,84 @@ class PandasGroupedOpsMixin:
         |  2|          2| 3.0|
         +---+-----------+----+
 
+        The function can also take and return an iterator of `pandas.DataFrame` using type
+        hints.
+
+        >>> from typing import Iterator  # doctest: +SKIP
+        >>> df = spark.createDataFrame(
+        ...     [(1, 1.0), (1, 2.0), (2, 3.0), (2, 5.0), (2, 10.0)],
+        ...     ("id", "v"))  # doctest: +SKIP
+        >>> def filter_func(
+        ...     batches: Iterator[pd.DataFrame]
+        ... ) -> Iterator[pd.DataFrame]:  # doctest: +SKIP
+        ...     for batch in batches:
+        ...         # Process and yield each batch independently
+        ...         filtered = batch[batch['v'] > 2.0]
+        ...         if not filtered.empty:
+        ...             yield filtered[['v']]
+        >>> df.groupby("id").applyInPandas(
+        ...     filter_func, schema="v double").show()  # doctest: +SKIP
+        +----+
+        |   v|
+        +----+
+        | 3.0|
+        | 5.0|
+        |10.0|
+        +----+
+
+        Alternatively, the user can pass a function that takes two arguments.
+        In this case, the grouping key(s) will be passed as the first argument and the data will
+        be passed as the second argument. The grouping key(s) will be passed as a tuple of numpy
+        data types. The data will still be passed in as an iterator of `pandas.DataFrame`.
+
+        >>> from typing import Iterator, Tuple, Any  # doctest: +SKIP
+        >>> def transform_func(
+        ...     key: Tuple[Any, ...], batches: Iterator[pd.DataFrame]
+        ... ) -> Iterator[pd.DataFrame]:  # doctest: +SKIP
+        ...     for batch in batches:
+        ...         # Yield transformed results for each batch
+        ...         result = batch.assign(id=key[0], v_doubled=batch['v'] * 2)
+        ...         yield result[['id', 'v_doubled']]
+        >>> df.groupby("id").applyInPandas(
+        ...     transform_func, schema="id long, v_doubled double").show()  # doctest: +SKIP
+        +---+----------+
+        | id|v_doubled |
+        +---+----------+
+        |  1|       2.0|
+        |  1|       4.0|
+        |  2|       6.0|
+        |  2|      10.0|
+        |  2|      20.0|
+        +---+----------+
+
         Notes
         -----
-        This function requires a full shuffle. All the data of a group will be loaded
-        into memory, so the user should be aware of the potential OOM risk if data is skewed
-        and certain groups are too large to fit in memory.
+        This function requires a full shuffle. If using the `pandas.DataFrame` API, all data of a
+        group will be loaded into memory, so the user should be aware of the potential OOM risk if
+        data is skewed and certain groups are too large to fit in memory, and can use the
+        iterator of `pandas.DataFrame` API to mitigate this.
 
         See Also
         --------
         pyspark.sql.functions.pandas_udf
         """
         from pyspark.sql import GroupedData
-        from pyspark.sql.functions import pandas_udf, PandasUDFType
+        from pyspark.sql.functions import pandas_udf
+        from pyspark.sql.pandas.typehints import infer_group_pandas_eval_type_from_func
 
         assert isinstance(self, GroupedData)
 
-        udf = pandas_udf(func, returnType=schema, functionType=PandasUDFType.GROUPED_MAP)
+        # Try to infer the eval type from type hints
+        eval_type = None
+        try:
+            eval_type = infer_group_pandas_eval_type_from_func(func)
+        except Exception as e:
+            warnings.warn(f"Cannot infer the eval type from type hints: {e}", UserWarning)
+
+        if eval_type is None:
+            eval_type = PythonEvalType.SQL_GROUPED_MAP_PANDAS_UDF
+
+        udf = pandas_udf(func, returnType=schema, functionType=eval_type)
         df = self._df
         udf_column = udf(*[df[col] for col in df.columns])
         jdf = self._jgd.flatMapGroupsInPandas(udf_column._jc)
@@ -376,7 +432,7 @@ class PandasGroupedOpsMixin:
         timeMode: str,
         initialState: Optional["GroupedData"] = None,
         eventTimeColumnName: str = "",
-    ) -> DataFrame:
+    ) -> "DataFrame":
         """
         Invokes methods defined in the stateful processor used in arbitrary state API v2. It
         requires protobuf, pandas and pyarrow as dependencies to process input/state data. We
@@ -494,9 +550,162 @@ class PandasGroupedOpsMixin:
         -----
         This function requires a full shuffle.
         """
+        return self.__transformWithState(
+            statefulProcessor,
+            outputStructType,
+            outputMode,
+            timeMode,
+            True,
+            initialState,
+            eventTimeColumnName,
+        )
 
+    def transformWithState(
+        self,
+        statefulProcessor: StatefulProcessor,
+        outputStructType: Union[StructType, str],
+        outputMode: str,
+        timeMode: str,
+        initialState: Optional["GroupedData"] = None,
+        eventTimeColumnName: str = "",
+    ) -> "DataFrame":
+        """
+        Invokes methods defined in the stateful processor used in arbitrary state API v2. It
+        requires protobuf and pyarrow as dependencies to process input/state data. We allow
+        the user to act on per-group set of input rows along with keyed state and the user
+        can choose to output/return 0 or more rows.
+
+        For a streaming dataframe, we will repeatedly invoke the interface methods for new rows
+        in each trigger and the user's state/state variables will be stored persistently across
+        invocations.
+
+        The `statefulProcessor` should be a Python class that implements the interface defined in
+        :class:`StatefulProcessor`.
+
+        The `outputStructType` should be a :class:`StructType` describing the schema of all
+        elements in the returned value, `Row`. The column labels of all elements in
+        returned `Row` must either match the field names in the defined schema.
+
+        The number of `Row`s in the iterator in both the input and output can be arbitrary.
+
+        .. versionadded:: 4.1.0
+
+        Parameters
+        ----------
+        statefulProcessor : :class:`pyspark.sql.streaming.stateful_processor.StatefulProcessor`
+            Instance of StatefulProcessor whose functions will be invoked by the operator.
+        outputStructType : :class:`pyspark.sql.types.DataType` or str
+            The type of the output records. The value can be either a
+            :class:`pyspark.sql.types.DataType` object or a DDL-formatted type string.
+        outputMode : str
+            The output mode of the stateful processor.
+        timeMode : str
+            The time mode semantics of the stateful processor for timers and TTL.
+        initialState : :class:`pyspark.sql.GroupedData`
+            Optional. The grouped dataframe as initial states used for initialization
+            of state variables in the first batch.
+
+        Examples
+        --------
+        >>> from typing import Iterator
+        ...
+        >>> from pyspark.sql import Row
+        >>> from pyspark.sql.functions import col, split
+        >>> from pyspark.sql.streaming import StatefulProcessor, StatefulProcessorHandle
+        >>> from pyspark.sql.types import IntegerType, LongType, StringType, StructField, StructType
+        ...
+        >>> spark.conf.set("spark.sql.streaming.stateStore.providerClass",
+        ...     "org.apache.spark.sql.execution.streaming.state.RocksDBStateStoreProvider")
+        ... # Below is a simple example to find erroneous sensors from temperature sensor data. The
+        ... # processor returns a count of total readings, while keeping erroneous reading counts
+        ... # in streaming state. A violation is defined when the temperature is above 100.
+        ... # The input data is a DataFrame with the following schema:
+        ... #    `id: string, temperature: long`.
+        ... # The output schema and state schema are defined as below.
+        >>> output_schema = StructType([
+        ...     StructField("id", StringType(), True),
+        ...     StructField("count", IntegerType(), True)
+        ... ])
+        >>> state_schema = StructType([
+        ...     StructField("value", IntegerType(), True)
+        ... ])
+        >>> class SimpleStatefulProcessor(StatefulProcessor):
+        ...     def init(self, handle: StatefulProcessorHandle):
+        ...         self.num_violations_state = handle.getValueState("numViolations", state_schema)
+        ...
+        ...     def handleInputRows(self, key, rows):
+        ...         new_violations = 0
+        ...         count = 0
+        ...         exists = self.num_violations_state.exists()
+        ...         if exists:
+        ...             existing_violations_row = self.num_violations_state.get()
+        ...             existing_violations = existing_violations_row[0]
+        ...         else:
+        ...             existing_violations = 0
+        ...         for row in rows:
+        ...             if row.temperature is not None:
+        ...                  count += 1
+        ...                  if row.temperature > 100:
+        ...                      new_violations += 1
+        ...         updated_violations = new_violations + existing_violations
+        ...         self.num_violations_state.update((updated_violations,))
+        ...         yield Row(id=key, count=count)
+        ...
+        ...     def close(self) -> None:
+        ...         pass
+
+        Input DataFrame:
+        +---+-----------+
+        | id|temperature|
+        +---+-----------+
+        |  0|        123|
+        |  0|         23|
+        |  1|         33|
+        |  1|        188|
+        |  1|         88|
+        +---+-----------+
+
+        >>> df.groupBy("value").transformWithState(statefulProcessor =
+        ...     SimpleStatefulProcessor(), outputStructType=output_schema, outputMode="Update",
+        ...     timeMode="None") # doctest: +SKIP
+
+        Output DataFrame:
+        +---+-----+
+        | id|count|
+        +---+-----+
+        |  0|    2|
+        |  1|    3|
+        +---+-----+
+
+        Notes
+        -----
+        This function requires a full shuffle.
+        """
+        return self.__transformWithState(
+            statefulProcessor,
+            outputStructType,
+            outputMode,
+            timeMode,
+            False,
+            initialState,
+            eventTimeColumnName,
+        )
+
+    def __transformWithState(
+        self,
+        statefulProcessor: StatefulProcessor,
+        outputStructType: Union[StructType, str],
+        outputMode: str,
+        timeMode: str,
+        usePandas: bool,
+        initialState: Optional["GroupedData"],
+        eventTimeColumnName: str,
+    ) -> "DataFrame":
         from pyspark.sql import GroupedData
         from pyspark.sql.functions import pandas_udf
+        from pyspark.sql.streaming.stateful_processor_util import (
+            TransformWithStateInPandasUdfUtils,
+        )
 
         assert isinstance(self, GroupedData)
         if initialState is not None:
@@ -504,212 +713,56 @@ class PandasGroupedOpsMixin:
         if isinstance(outputStructType, str):
             outputStructType = cast(StructType, self._df._session._parse_ddl(outputStructType))
 
-        def handle_pre_init(
-            statefulProcessorApiClient: StatefulProcessorApiClient,
-        ) -> Iterator["PandasDataFrameLike"]:
-            # Driver handle is different from the handle used on executors;
-            # On JVM side, we will use `DriverStatefulProcessorHandleImpl` for driver handle which
-            # will only be used for handling init() and get the state schema on the driver.
-            driver_handle = StatefulProcessorHandle(statefulProcessorApiClient)
-            statefulProcessorApiClient.set_handle_state(StatefulProcessorHandleState.PRE_INIT)
-            statefulProcessor.init(driver_handle)
-
-            # This method is used for the driver-side stateful processor after we have collected
-            # all the necessary schemas. This instance of the DriverStatefulProcessorHandleImpl
-            # won't be used again on JVM.
-            statefulProcessor.close()
-
-            # return a dummy results, no return value is needed for pre init
-            return iter([])
-
-        def handle_data_rows(
-            statefulProcessorApiClient: StatefulProcessorApiClient,
-            key: Any,
-            inputRows: Optional[Iterator["PandasDataFrameLike"]] = None,
-        ) -> Iterator["PandasDataFrameLike"]:
-            statefulProcessorApiClient.set_implicit_key(key)
-
-            batch_timestamp, watermark_timestamp = statefulProcessorApiClient.get_timestamps(
-                timeMode
-            )
-
-            # process with data rows
-            if inputRows is not None:
-                data_iter = statefulProcessor.handleInputRows(
-                    key, inputRows, TimerValues(batch_timestamp, watermark_timestamp)
-                )
-                return data_iter
-            else:
-                return iter([])
-
-        def handle_expired_timers(
-            statefulProcessorApiClient: StatefulProcessorApiClient,
-        ) -> Iterator["PandasDataFrameLike"]:
-            batch_timestamp, watermark_timestamp = statefulProcessorApiClient.get_timestamps(
-                timeMode
-            )
-
-            if timeMode.lower() == "processingtime":
-                expiry_list_iter = statefulProcessorApiClient.get_expiry_timers_iterator(
-                    batch_timestamp
-                )
-            elif timeMode.lower() == "eventtime":
-                expiry_list_iter = statefulProcessorApiClient.get_expiry_timers_iterator(
-                    watermark_timestamp
-                )
-            else:
-                expiry_list_iter = iter([[]])
-
-            # process with expiry timers, only timer related rows will be emitted
-            for expiry_list in expiry_list_iter:
-                for key_obj, expiry_timestamp in expiry_list:
-                    statefulProcessorApiClient.set_implicit_key(key_obj)
-                    for pd in statefulProcessor.handleExpiredTimer(
-                        key=key_obj,
-                        timerValues=TimerValues(batch_timestamp, watermark_timestamp),
-                        expiredTimerInfo=ExpiredTimerInfo(expiry_timestamp),
-                    ):
-                        yield pd
-                    statefulProcessorApiClient.delete_timer(expiry_timestamp)
-
-        def transformWithStateUDF(
-            statefulProcessorApiClient: StatefulProcessorApiClient,
-            mode: TransformWithStateInPandasFuncMode,
-            key: Any,
-            inputRows: Iterator["PandasDataFrameLike"],
-        ) -> Iterator["PandasDataFrameLike"]:
-            if mode == TransformWithStateInPandasFuncMode.PRE_INIT:
-                return handle_pre_init(statefulProcessorApiClient)
-
-            handle = StatefulProcessorHandle(statefulProcessorApiClient)
-
-            if statefulProcessorApiClient.handle_state == StatefulProcessorHandleState.CREATED:
-                statefulProcessor.init(handle)
-                statefulProcessorApiClient.set_handle_state(
-                    StatefulProcessorHandleState.INITIALIZED
-                )
-
-            if mode == TransformWithStateInPandasFuncMode.PROCESS_TIMER:
-                statefulProcessorApiClient.set_handle_state(
-                    StatefulProcessorHandleState.DATA_PROCESSED
-                )
-                result = handle_expired_timers(statefulProcessorApiClient)
-                return result
-            elif mode == TransformWithStateInPandasFuncMode.COMPLETE:
-                statefulProcessorApiClient.set_handle_state(
-                    StatefulProcessorHandleState.TIMER_PROCESSED
-                )
-                statefulProcessorApiClient.remove_implicit_key()
-                statefulProcessor.close()
-                statefulProcessorApiClient.set_handle_state(StatefulProcessorHandleState.CLOSED)
-                return iter([])
-            else:
-                # mode == TransformWithStateInPandasFuncMode.PROCESS_DATA
-                result = handle_data_rows(statefulProcessorApiClient, key, inputRows)
-                return result
-
-        def transformWithStateWithInitStateUDF(
-            statefulProcessorApiClient: StatefulProcessorApiClient,
-            mode: TransformWithStateInPandasFuncMode,
-            key: Any,
-            inputRows: Iterator["PandasDataFrameLike"],
-            initialStates: Optional[Iterator["PandasDataFrameLike"]] = None,
-        ) -> Iterator["PandasDataFrameLike"]:
-            """
-            UDF for TWS operator with non-empty initial states. Possible input combinations
-            of inputRows and initialStates iterator:
-            - Both `inputRows` and `initialStates` are non-empty. Both input rows and initial
-             states contains the grouping key and data.
-            - `InitialStates` is non-empty, while `inputRows` is empty. Only initial states
-             contains the grouping key and data, and it is first batch.
-            - `initialStates` is empty, while `inputRows` is non-empty. Only inputRows contains the
-             grouping key and data, and it is first batch.
-            - `initialStates` is None, while `inputRows` is not empty. This is not first batch.
-             `initialStates` is initialized to the positional value as None.
-            """
-            if mode == TransformWithStateInPandasFuncMode.PRE_INIT:
-                return handle_pre_init(statefulProcessorApiClient)
-
-            handle = StatefulProcessorHandle(statefulProcessorApiClient)
-
-            if statefulProcessorApiClient.handle_state == StatefulProcessorHandleState.CREATED:
-                statefulProcessor.init(handle)
-                statefulProcessorApiClient.set_handle_state(
-                    StatefulProcessorHandleState.INITIALIZED
-                )
-
-            if mode == TransformWithStateInPandasFuncMode.PROCESS_TIMER:
-                statefulProcessorApiClient.set_handle_state(
-                    StatefulProcessorHandleState.DATA_PROCESSED
-                )
-                result = handle_expired_timers(statefulProcessorApiClient)
-                return result
-            elif mode == TransformWithStateInPandasFuncMode.COMPLETE:
-                statefulProcessorApiClient.remove_implicit_key()
-                statefulProcessor.close()
-                statefulProcessorApiClient.set_handle_state(StatefulProcessorHandleState.CLOSED)
-                return iter([])
-            else:
-                # mode == TransformWithStateInPandasFuncMode.PROCESS_DATA
-                batch_timestamp, watermark_timestamp = statefulProcessorApiClient.get_timestamps(
-                    timeMode
-                )
-
-            # only process initial state if first batch and initial state is not None
-            if initialStates is not None:
-                for cur_initial_state in initialStates:
-                    statefulProcessorApiClient.set_implicit_key(key)
-                    statefulProcessor.handleInitialState(
-                        key, cur_initial_state, TimerValues(batch_timestamp, watermark_timestamp)
-                    )
-
-            # if we don't have input rows for the given key but only have initial state
-            # for the grouping key, the inputRows iterator could be empty
-            input_rows_empty = False
-            try:
-                first = next(inputRows)
-            except StopIteration:
-                input_rows_empty = True
-            else:
-                inputRows = itertools.chain([first], inputRows)
-
-            if not input_rows_empty:
-                result = handle_data_rows(statefulProcessorApiClient, key, inputRows)
-            else:
-                result = iter([])
-
-            return result
-
-        if isinstance(outputStructType, str):
-            outputStructType = cast(StructType, self._df._session._parse_ddl(outputStructType))
-
         df = self._df
+        udf_util = TransformWithStateInPandasUdfUtils(statefulProcessor, timeMode)
+
+        # explicitly set the type to Any since it could match to various types (literals)
+        functionType: Any = None
+        if usePandas and initialState is None:
+            functionType = PythonEvalType.SQL_TRANSFORM_WITH_STATE_PANDAS_UDF
+        elif usePandas and initialState is not None:
+            functionType = PythonEvalType.SQL_TRANSFORM_WITH_STATE_PANDAS_INIT_STATE_UDF
+        elif not usePandas and initialState is None:
+            functionType = PythonEvalType.SQL_TRANSFORM_WITH_STATE_PYTHON_ROW_UDF
+        else:
+            # not usePandas and initialState is not None
+            functionType = PythonEvalType.SQL_TRANSFORM_WITH_STATE_PYTHON_ROW_INIT_STATE_UDF
 
         if initialState is None:
             initial_state_java_obj = None
             udf = pandas_udf(
-                transformWithStateUDF,  # type: ignore
+                udf_util.transformWithStateUDF,  # type: ignore
                 returnType=outputStructType,
-                functionType=PythonEvalType.SQL_TRANSFORM_WITH_STATE_PANDAS_UDF,
+                functionType=functionType,
             )
         else:
             initial_state_java_obj = initialState._jgd
             udf = pandas_udf(
-                transformWithStateWithInitStateUDF,  # type: ignore
+                udf_util.transformWithStateWithInitStateUDF,  # type: ignore
                 returnType=outputStructType,
-                functionType=PythonEvalType.SQL_TRANSFORM_WITH_STATE_PANDAS_INIT_STATE_UDF,
+                functionType=functionType,
             )
 
         udf_column = udf(*[df[col] for col in df.columns])
 
-        jdf = self._jgd.transformWithStateInPandas(
-            udf_column._jc,
-            self.session._jsparkSession.parseDataType(outputStructType.json()),
-            outputMode,
-            timeMode,
-            initial_state_java_obj,
-            eventTimeColumnName,
-        )
+        if usePandas:
+            jdf = self._jgd.transformWithStateInPandas(
+                udf_column._jc,
+                self.session._jsparkSession.parseDataType(outputStructType.json()),
+                outputMode,
+                timeMode,
+                initial_state_java_obj,
+                eventTimeColumnName,
+            )
+        else:
+            jdf = self._jgd.transformWithStateInPySpark(
+                udf_column._jc,
+                self.session._jsparkSession.parseDataType(outputStructType.json()),
+                outputMode,
+                timeMode,
+                initial_state_java_obj,
+                eventTimeColumnName,
+            )
         return DataFrame(jdf, self.session)
 
     def applyInArrow(
@@ -719,27 +772,33 @@ class PandasGroupedOpsMixin:
         Maps each group of the current :class:`DataFrame` using an Arrow udf and returns the result
         as a `DataFrame`.
 
-        The function should take a `pyarrow.Table` and return another
-        `pyarrow.Table`. Alternatively, the user can pass a function that takes
-        a tuple of `pyarrow.Scalar` grouping key(s) and a `pyarrow.Table`.
-        For each group, all columns are passed together as a `pyarrow.Table`
-        to the user-function and the returned `pyarrow.Table` are combined as a
-        :class:`DataFrame`.
+        The function can take one of two forms: It can take a `pyarrow.Table` and return a
+        `pyarrow.Table`, or it can take an iterator of `pyarrow.RecordBatch` and yield
+        `pyarrow.RecordBatch`. Alternatively each form can take a tuple of `pyarrow.Scalar`
+        as the first argument in addition to the input type above. For each group, all columns
+        are passed together in the `pyarrow.Table` or `pyarrow.RecordBatch`, and the returned
+        `pyarrow.Table` or iterator of `pyarrow.RecordBatch` are combined as a :class:`DataFrame`.
 
         The `schema` should be a :class:`StructType` describing the schema of the returned
-        `pyarrow.Table`. The column labels of the returned `pyarrow.Table` must either match
-        the field names in the defined schema if specified as strings, or match the
-        field data types by position if not strings, e.g. integer indices.
-        The length of the returned `pyarrow.Table` can be arbitrary.
+        `pyarrow.Table` or `pyarrow.RecordBatch`. The column labels of the returned `pyarrow.Table`
+        or `pyarrow.RecordBatch` must either match the field names in the defined schema if
+        specified as strings, or match the field data types by position if not strings, e.g.
+        integer indices. The length of the returned `pyarrow.Table` or iterator of
+        `pyarrow.RecordBatch` can be arbitrary.
 
         .. versionadded:: 4.0.0
 
+        .. versionchanged:: 4.1.0
+           Added support for an iterator of `pyarrow.RecordBatch` API.
+
         Parameters
         ----------
         func : function
-            a Python native function that takes a `pyarrow.Table` and outputs a
-            `pyarrow.Table`, or that takes one tuple (grouping keys) and a
-            `pyarrow.Table` and outputs a `pyarrow.Table`.
+            a Python native function that either takes a `pyarrow.Table` and outputs a
+            `pyarrow.Table` or takes an iterator of `pyarrow.RecordBatch` and yields
+            `pyarrow.RecordBatch`. Additionally, each form can take a tuple of grouping keys
+            as the first argument, with the `pyarrow.Table` or iterator of `pyarrow.RecordBatch`
+            as the second argument.
         schema : :class:`pyspark.sql.types.DataType` or str
             the return type of the `func` in PySpark. The value can be either a
             :class:`pyspark.sql.types.DataType` object or a DDL-formatted type string.
@@ -768,6 +827,28 @@ class PandasGroupedOpsMixin:
         |  2| 1.1094003924504583|
         +---+-------------------+
 
+        The function can also take and return an iterator of `pyarrow.RecordBatch` using type
+        hints.
+
+        >>> df = spark.createDataFrame(
+        ...     [(1, 1.0), (1, 2.0), (2, 3.0), (2, 5.0), (2, 10.0)],
+        ...     ("id", "v"))  # doctest: +SKIP
+        >>> def sum_func(
+        ...     batches: Iterator[pyarrow.RecordBatch]
+        ... ) -> Iterator[pyarrow.RecordBatch]:  # doctest: +SKIP
+        ...     total = 0
+        ...     for batch in batches:
+        ...         total += pc.sum(batch.column("v")).as_py()
+        ...     yield pyarrow.RecordBatch.from_pydict({"v": [total]})
+        >>> df.groupby("id").applyInArrow(
+        ...     sum_func, schema="v double").show()  # doctest: +SKIP
+        +----+
+        |   v|
+        +----+
+        | 3.0|
+        |18.0|
+        +----+
+
         Alternatively, the user can pass a function that takes two arguments.
         In this case, the grouping key(s) will be passed as the first argument and the data will
         be passed as the second argument. The grouping key(s) will be passed as a tuple of Arrow
@@ -812,11 +893,28 @@ class PandasGroupedOpsMixin:
         |  2|          2| 3.0|
         +---+-----------+----+
 
+        >>> def sum_func(
+        ...     key: Tuple[pyarrow.Scalar, ...], batches: Iterator[pyarrow.RecordBatch]
+        ... ) -> Iterator[pyarrow.RecordBatch]:  # doctest: +SKIP
+        ...     total = 0
+        ...     for batch in batches:
+        ...         total += pc.sum(batch.column("v")).as_py()
+        ...     yield pyarrow.RecordBatch.from_pydict({"id": [key[0].as_py()], "v": [total]})
+        >>> df.groupby("id").applyInArrow(
+        ...     sum_func, schema="id long, v double").show()  # doctest: +SKIP
+        +---+----+
+        | id|   v|
+        +---+----+
+        |  1| 3.0|
+        |  2|18.0|
+        +---+----+
+
         Notes
         -----
-        This function requires a full shuffle. All the data of a group will be loaded
-        into memory, so the user should be aware of the potential OOM risk if data is skewed
-        and certain groups are too large to fit in memory.
+        This function requires a full shuffle. If using the `pyarrow.Table` API, all data of a
+        group will be loaded into memory, so the user should be aware of the potential OOM risk
+        if data is skewed and certain groups are too large to fit in memory, and can use the
+        iterator of `pyarrow.RecordBatch` API to mitigate this.
 
         This API is unstable, and for developers.
 
@@ -829,9 +927,18 @@ class PandasGroupedOpsMixin:
 
         assert isinstance(self, GroupedData)
 
+        try:
+            # Try to infer the eval type from type hints
+            eval_type = infer_group_arrow_eval_type_from_func(func)
+        except Exception:
+            warnings.warn("Cannot infer the eval type from type hints. ", UserWarning)
+
+        if eval_type is None:
+            eval_type = PythonEvalType.SQL_GROUPED_MAP_ARROW_UDF
+
         # The usage of the pandas_udf is internal so type checking is disabled.
         udf = pandas_udf(
-            func, returnType=schema, functionType=PythonEvalType.SQL_GROUPED_MAP_ARROW_UDF
+            func, returnType=schema, functionType=eval_type
         )  # type: ignore[call-overload]
         df = self._df
         udf_column = udf(*[df[col] for col in df.columns])
