@@ -21,10 +21,12 @@ import scala.jdk.CollectionConverters._
 
 import org.apache.spark.SparkException
 import org.apache.spark.sql.AnalysisException
-import org.apache.spark.sql.catalyst.SqlScriptingLocalVariableManager
+import org.apache.spark.sql.catalyst.SqlScriptingContextManager
+import org.apache.spark.sql.catalyst.expressions.AttributeReference
 import org.apache.spark.sql.catalyst.plans.logical._
 import org.apache.spark.sql.catalyst.rules.Rule
-import org.apache.spark.sql.connector.catalog.{CatalogManager, CatalogPlugin, Identifier, LookupCatalog, SupportsNamespaces}
+import org.apache.spark.sql.catalyst.util.SparkCharVarcharUtils.replaceCharVarcharWithString
+import org.apache.spark.sql.connector.catalog._
 import org.apache.spark.sql.errors.DataTypeErrors.toSQLId
 import org.apache.spark.sql.errors.QueryCompilationErrors
 import org.apache.spark.util.ArrayImplicits._
@@ -38,35 +40,40 @@ class ResolveCatalogs(val catalogManager: CatalogManager)
   override def apply(plan: LogicalPlan): LogicalPlan = plan resolveOperatorsDown {
     // We only support temp variables for now and the system catalog is not properly implemented
     // yet. We need to resolve `UnresolvedIdentifier` for variable commands specially.
-    case c @ CreateVariable(UnresolvedIdentifier(nameParts, _), _, _) =>
-      // From scripts we can only create local variables, which must be unqualified,
-      // and must not be DECLARE OR REPLACE.
-      val resolved = if (withinSqlScript) {
-        if (c.replace) {
-          throw new AnalysisException(
-            "INVALID_VARIABLE_DECLARATION.REPLACE_LOCAL_VARIABLE",
-            Map("varName" -> toSQLId(nameParts))
-          )
-        }
+    case c @ CreateVariable(identifiers, _, _) =>
+      // We resolve only UnresolvedIdentifiers, and pass on the other nodes
+      val resolved = identifiers.map {
+        case UnresolvedIdentifier(nameParts, _) =>
+          // From scripts we can only create local variables, which must be unqualified,
+          // and must not be DECLARE OR REPLACE.
+          if (withinSqlScript) {
+            if (c.replace) {
+              throw new AnalysisException(
+                "INVALID_VARIABLE_DECLARATION.REPLACE_LOCAL_VARIABLE",
+                Map("varName" -> toSQLId(nameParts))
+              )
+            }
 
-        if (nameParts.length != 1) {
-          throw new AnalysisException(
-            "INVALID_VARIABLE_DECLARATION.QUALIFIED_LOCAL_VARIABLE",
-            Map("varName" -> toSQLId(nameParts)))
-        }
+            if (nameParts.length != 1) {
+              throw new AnalysisException(
+                "INVALID_VARIABLE_DECLARATION.QUALIFIED_LOCAL_VARIABLE",
+                Map("varName" -> toSQLId(nameParts)))
+            }
 
-        SqlScriptingLocalVariableManager.get()
-          .getOrElse(throw SparkException.internalError(
-              "Scripting local variable manager should be present in SQL script."))
-          .qualify(nameParts.last)
-      } else {
-        val resolvedIdentifier = catalogManager.tempVariableManager.qualify(nameParts.last)
+            SqlScriptingContextManager.get().map(_.getVariableManager)
+              .getOrElse(throw SparkException.internalError(
+                "Scripting local variable manager should be present in SQL script."))
+              .qualify(nameParts.last)
+          } else {
+            val resolvedIdentifier
+            = catalogManager.tempVariableManager.qualify(nameParts.last)
 
-        assertValidSessionVariableNameParts(nameParts, resolvedIdentifier)
-        resolvedIdentifier
+            assertValidSessionVariableNameParts(nameParts, resolvedIdentifier)
+            resolvedIdentifier
+          }
+        case plan => plan
       }
-
-      c.copy(name = resolved)
+      c.copy(names = resolved)
     case d @ DropVariable(UnresolvedIdentifier(nameParts, _), _) =>
       if (withinSqlScript) {
         throw new AnalysisException(
@@ -76,14 +83,19 @@ class ResolveCatalogs(val catalogManager: CatalogManager)
       assertValidSessionVariableNameParts(nameParts, resolved)
       d.copy(name = resolved)
 
+    // For CREATE TABLE and REPLACE TABLE statements, resolve the table identifier and include
+    // the table columns as output. This allows expressions (e.g., constraints) referencing these
+    // columns to be resolved correctly.
+    case c @ CreateTable(UnresolvedIdentifier(nameParts, allowTemp), columns, _, _, _) =>
+      val resolvedIdentifier = resolveIdentifier(nameParts, allowTemp, columns)
+      c.copy(name = resolvedIdentifier)
+
+    case r @ ReplaceTable(UnresolvedIdentifier(nameParts, allowTemp), columns, _, _, _) =>
+      val resolvedIdentifier = resolveIdentifier(nameParts, allowTemp, columns)
+      r.copy(name = resolvedIdentifier)
+
     case UnresolvedIdentifier(nameParts, allowTemp) =>
-      if (allowTemp && catalogManager.v1SessionCatalog.isTempView(nameParts)) {
-        val ident = Identifier.of(nameParts.dropRight(1).toArray, nameParts.last)
-        ResolvedIdentifier(FakeSystemCatalog, ident)
-      } else {
-        val CatalogAndIdentifier(catalog, identifier) = nameParts
-        ResolvedIdentifier(catalog, identifier)
-      }
+      resolveIdentifier(nameParts, allowTemp, Nil)
 
     case CurrentNamespace =>
       ResolvedNamespace(currentCatalog, catalogManager.currentNamespace.toImmutableArraySeq)
@@ -93,6 +105,27 @@ class ResolveCatalogs(val catalogManager: CatalogManager)
       resolveNamespace(catalog, ns, fetchMetadata)
   }
 
+  private def resolveIdentifier(
+      nameParts: Seq[String],
+      allowTemp: Boolean,
+      columns: Seq[ColumnDefinition]): ResolvedIdentifier = {
+    val columnOutput = columns.map { col =>
+      val dataType = if (conf.preserveCharVarcharTypeInfo) {
+        col.dataType
+      } else {
+        replaceCharVarcharWithString(col.dataType)
+      }
+      AttributeReference(col.name, dataType, col.nullable, col.metadata)()
+    }
+    if (allowTemp && catalogManager.v1SessionCatalog.isTempView(nameParts)) {
+      val ident = Identifier.of(nameParts.dropRight(1).toArray, nameParts.last)
+      ResolvedIdentifier(FakeSystemCatalog, ident, columnOutput)
+    } else {
+      val CatalogAndIdentifier(catalog, identifier) = nameParts
+      ResolvedIdentifier(catalog, identifier, columnOutput)
+    }
+  }
+
   private def resolveNamespace(
       catalog: CatalogPlugin,
       ns: Seq[String],
@@ -109,7 +142,7 @@ class ResolveCatalogs(val catalogManager: CatalogManager)
   }
 
   private def withinSqlScript: Boolean =
-    SqlScriptingLocalVariableManager.get().isDefined && !AnalysisContext.get.isExecuteImmediate
+    SqlScriptingContextManager.get().map(_.getVariableManager).isDefined
 
   private def assertValidSessionVariableNameParts(
       nameParts: Seq[String],
