@@ -18,11 +18,11 @@
 package org.apache.spark.sql.catalyst.analysis
 
 import org.apache.spark.SparkException
-import org.apache.spark.sql.catalyst.expressions.{Alias, CreateArray, CreateMap, CreateNamedStruct, Expression, LeafExpression, Literal, MapFromArrays, MapFromEntries, SubqueryExpression, Unevaluable, VariableReference}
+import org.apache.spark.sql.catalyst.expressions.{Expression, LeafExpression, SubqueryExpression, Unevaluable}
 import org.apache.spark.sql.catalyst.plans.logical.{LogicalPlan, SupervisingCommand}
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.catalyst.trees.TreePattern.{COMMAND, PARAMETER, PARAMETERIZED_QUERY, TreePattern, UNRESOLVED_WITH}
-import org.apache.spark.sql.errors.QueryErrorsBase
+import org.apache.spark.sql.errors.{QueryCompilationErrors, QueryErrorsBase}
 import org.apache.spark.sql.types.DataType
 
 sealed trait Parameter extends LeafExpression with Unevaluable {
@@ -104,6 +104,31 @@ case class PosParameterizedQuery(child: LogicalPlan, args: Seq[Expression])
     copy(child = newChild)
 }
 
+/**
+ * The logical plan representing a parameterized query with general parameter support.
+ * This allows the query to use either positional or named parameters based on the
+ * parameter markers found in the query, with optional parameter names provided.
+ *
+ * @param child The parameterized logical plan.
+ * @param args The literal values or collection constructor functions such as `map()`,
+ *             `array()`, `struct()` of parameters.
+ * @param paramNames Optional parameter names corresponding to args. If provided for an argument,
+ *                   that argument can be used for named parameter binding. If not provided
+ *                   parameters are treated as positional.
+ */
+case class GeneralParameterizedQuery(
+    child: LogicalPlan,
+    args: Seq[Expression],
+    paramNames: Seq[String])
+  extends ParameterizedQuery(child) {
+  assert(args.nonEmpty)
+  assert(paramNames.length == args.length,
+    s"paramNames must be same length as args. " +
+    s"paramNames.length=${paramNames.length}, args.length=${args.length}")
+  override protected def withNewChildInternal(newChild: LogicalPlan): LogicalPlan =
+    copy(child = newChild)
+}
+
 /**
  * Moves `ParameterizedQuery` inside `SupervisingCommand` for their supervised plans to be
  * resolved later by the analyzer.
@@ -143,24 +168,11 @@ object MoveParameterizedQueriesDown extends Rule[LogicalPlan] {
 }
 
 /**
- * Finds all named parameters in `ParameterizedQuery` and substitutes them by literals or
+ * Binds all named parameters in `ParameterizedQuery` and substitutes them by literals or
  * by collection constructor functions such as `map()`, `array()`, `struct()`
  * from the user-specified arguments.
  */
 object BindParameters extends Rule[LogicalPlan] with QueryErrorsBase {
-  private def checkArgs(args: Iterable[(String, Expression)]): Unit = {
-    def isNotAllowed(expr: Expression): Boolean = expr.exists {
-      case _: Literal | _: CreateArray | _: CreateNamedStruct |
-        _: CreateMap | _: MapFromArrays |  _: MapFromEntries | _: VariableReference => false
-      case a: Alias => isNotAllowed(a.child)
-      case _ => true
-    }
-    args.find(arg => isNotAllowed(arg._2)).foreach { case (name, expr) =>
-      expr.failAnalysis(
-        errorClass = "INVALID_SQL_ARG",
-        messageParameters = Map("name" -> name))
-    }
-  }
 
   private def bind(p0: LogicalPlan)(f: PartialFunction[Expression, Expression]): LogicalPlan = {
     var stop = false
@@ -185,14 +197,15 @@ object BindParameters extends Rule[LogicalPlan] with QueryErrorsBase {
             s"must be equal to the number of argument values ${argValues.length}.")
         }
         val args = argNames.zip(argValues).toMap
-        checkArgs(args)
+        ParameterizedQueryArgumentsValidator(args)
         bind(child) { case NamedParameter(name) if args.contains(name) => args(name) }
 
       case PosParameterizedQuery(child, args)
         if !child.containsPattern(UNRESOLVED_WITH) &&
           args.forall(_.resolved) =>
+
         val indexedArgs = args.zipWithIndex
-        checkArgs(indexedArgs.map(arg => (s"_${arg._2}", arg._1)))
+        ParameterizedQueryArgumentsValidator(indexedArgs.map(arg => (s"_${arg._2}", arg._1)))
 
         val positions = scala.collection.mutable.Set.empty[Int]
         bind(child) { case p @ PosParameter(pos) => positions.add(pos); p }
@@ -203,6 +216,63 @@ object BindParameters extends Rule[LogicalPlan] with QueryErrorsBase {
             args(posToIndex(pos))
         }
 
+      case GeneralParameterizedQuery(child, args, paramNames)
+        if !child.containsPattern(UNRESOLVED_WITH) &&
+          args.forall(_.resolved) =>
+
+        // Check all arguments for validity (args are already evaluated expressions/literals)
+        val allArgs = args.zip(paramNames).zipWithIndex.map { case ((arg, name), index) =>
+          val finalName = if (name.isEmpty) s"_$index" else name
+          finalName -> arg
+        }
+        ParameterizedQueryArgumentsValidator(allArgs)
+
+        // Collect parameter types used in the query to enforce invariants
+        var hasNamedParam = false
+        val positionalParams = scala.collection.mutable.Set.empty[Int]
+        bind(child) {
+          case p @ NamedParameter(_) => hasNamedParam = true; p
+          case p @ PosParameter(pos) => positionalParams.add(pos); p
+        }
+
+        // Validate: no mixing of positional and named parameters
+        if (hasNamedParam && positionalParams.nonEmpty) {
+          throw QueryCompilationErrors.invalidQueryMixedQueryParameters()
+        }
+
+        // Validate: if query uses named parameters, all USING expressions must have names
+        if (hasNamedParam && positionalParams.isEmpty) {
+          if (paramNames.isEmpty) {
+            // Query uses named parameters but no USING expressions provided
+            throw QueryCompilationErrors.invalidQueryAllParametersMustBeNamed(Seq.empty)
+          } else {
+            // Check that all USING expressions have names
+            val unnamedExpressions = paramNames.zipWithIndex.collect {
+              case ("", index) => index // empty strings are unnamed
+            }
+            if (unnamedExpressions.nonEmpty) {
+              val unnamedExprs = unnamedExpressions.map(args(_))
+              throw QueryCompilationErrors.invalidQueryAllParametersMustBeNamed(unnamedExprs)
+            }
+          }
+        }
+
+        // Now we can do simple binding based on which type we determined
+        if (hasNamedParam) {
+          // Named parameter binding - paramNames guaranteed to have no nulls at this point
+          val namedArgsMap = paramNames.zip(args).toMap
+          bind(child) {
+            case NamedParameter(name) => namedArgsMap.getOrElse(name, NamedParameter(name))
+          }
+        } else {
+          // Positional parameter binding (same logic as PosParameterizedQuery)
+          val posToIndex = positionalParams.toSeq.sorted.zipWithIndex.toMap
+          bind(child) {
+            case PosParameter(pos) if posToIndex.contains(pos) && args.size > posToIndex(pos) =>
+              args(posToIndex(pos))
+          }
+        }
+
       case other => other
     }
   }
