@@ -17,18 +17,19 @@
 
 package org.apache.spark.sql.catalyst.analysis.resolver
 
-import scala.collection.mutable.ArrayBuffer
+import java.util.HashSet
 
 import org.apache.spark.SparkException
 import org.apache.spark.sql.AnalysisException
-import org.apache.spark.sql.catalyst.EvaluateUnresolvedInlineTable
 import org.apache.spark.sql.catalyst.analysis.{
   withPosition,
   AnalysisErrorAt,
   FunctionResolution,
   MultiInstanceRelation,
+  RelationCache,
   RelationResolution,
   ResolvedInlineTable,
+  UnresolvedHaving,
   UnresolvedInlineTable,
   UnresolvedRelation,
   UnresolvedSubqueryColumnAliases
@@ -38,31 +39,13 @@ import org.apache.spark.sql.catalyst.expressions.{
   Attribute,
   AttributeSet,
   Expression,
-  NamedExpression
+  ExprId
 }
-import org.apache.spark.sql.catalyst.plans.logical.{
-  AnalysisHelper,
-  CTERelationDef,
-  CTERelationRef,
-  Distinct,
-  Filter,
-  GlobalLimit,
-  LeafNode,
-  LocalLimit,
-  LocalRelation,
-  LogicalPlan,
-  OneRowRelation,
-  Project,
-  SubqueryAlias,
-  Union,
-  UnresolvedWith,
-  View,
-  WithCTE
-}
-import org.apache.spark.sql.catalyst.trees.CurrentOrigin.withOrigin
+import org.apache.spark.sql.catalyst.plans.logical._
+import org.apache.spark.sql.catalyst.trees.CurrentOrigin
+import org.apache.spark.sql.catalyst.util.EvaluateUnresolvedInlineTable
 import org.apache.spark.sql.connector.catalog.CatalogManager
 import org.apache.spark.sql.errors.QueryCompilationErrors
-import org.apache.spark.sql.types.BooleanType
 
 /**
  * The Resolver implements a single-pass bottom-up analysis algorithm in the Catalyst.
@@ -84,24 +67,37 @@ import org.apache.spark.sql.types.BooleanType
  *   operators.
  * @param metadataResolverExtensions A list of [[ResolverExtension]]s that will be used to resolve
  *   relation operators in [[MetadataResolver]].
+ * @param externalRelationResolution An optional [[RelationResolution]] with to override the default
+ *   one. The default is constructed using [[Resolver.createRelationResolution]].
  */
 class Resolver(
     catalogManager: CatalogManager,
+    sharedRelationCache: RelationCache = RelationCache.empty,
     override val extensions: Seq[ResolverExtension] = Seq.empty,
-    metadataResolverExtensions: Seq[ResolverExtension] = Seq.empty)
+    metadataResolverExtensions: Seq[ResolverExtension] = Seq.empty,
+    externalRelationResolution: Option[RelationResolution] = None)
     extends LogicalPlanResolver
-    with ResolvesOperatorChildren
     with DelegatesResolutionToExtensions {
-  private val scopes = new NameScopeStack
-  private val cteRegistry = new CteRegistry
   private val planLogger = new PlanLogger
-  private val relationResolution = Resolver.createRelationResolution(catalogManager)
+  private val scopes = new NameScopeStack(planLogger)
+  private val cteRegistry = new CteRegistry
+  private val subqueryRegistry = new SubqueryRegistry
+  private val identifierAndCteSubstitutor = new IdentifierAndCteSubstitutor
+  private val relationResolution = externalRelationResolution.getOrElse {
+    Resolver.createRelationResolution(catalogManager, sharedRelationCache)
+  }
   private val functionResolution = new FunctionResolution(catalogManager, relationResolution)
   private val expressionResolver = new ExpressionResolver(this, functionResolution, planLogger)
+  private val aggregateResolver = new AggregateResolver(this, expressionResolver)
   private val expressionIdAssigner = expressionResolver.getExpressionIdAssigner
   private val projectResolver = new ProjectResolver(this, expressionResolver)
   private val viewResolver = new ViewResolver(resolver = this, catalogManager = catalogManager)
-  private val unionResolver = new UnionResolver(this, expressionResolver)
+  private val setOperationLikeResolver =
+    new SetOperationLikeResolver(this, expressionResolver)
+  private val filterResolver = new FilterResolver(this, expressionResolver)
+  private val sortResolver = new SortResolver(this, expressionResolver)
+  private val joinResolver = new JoinResolver(this, expressionResolver)
+  private val havingResolver = new HavingResolver(this, expressionResolver)
 
   /**
    * [[relationMetadataProvider]] is used to resolve metadata for relations. It's initialized with
@@ -122,35 +118,65 @@ class Resolver(
   )
 
   /**
-   * Get the [[CteRegistry]] which is a single instance per query resolution.
+   * Get [[NameScopeStack]] bound to the used [[Resolver]].
    */
-  def getCteRegistry: CteRegistry = {
-    cteRegistry
-  }
+  def getNameScopes: NameScopeStack = scopes
 
   /**
-   * This method is an analysis entry point. It resolves the metadata and invokes [[resolve]],
-   * which does most of the analysis work.
+   * Get the [[CteRegistry]] which is a single instance per query resolution.
+   */
+  def getCteRegistry: CteRegistry = cteRegistry
+
+  /**
+   * Get the [[SubqueryRegistry]] which is a single instance per query resolution.
+   */
+  def getSubqueryRegistry: SubqueryRegistry = subqueryRegistry
+
+  /**
+   * Get the [[ViewResolver]] bound to the used [[Resolver]].
+   */
+  def getViewResolver: ViewResolver = viewResolver
+
+  /**
+   * This method is a top-level analysis entry point:
+   * 1. Substitute IDENTIFIERs and CTEs in the `unresolvedPlan` using
+   *    [[IdentifierAndCteSubstitutor]];
+   * 2. Resolve the metadata for the plan using [[MetadataResolver]]. When
+   *    [[ANALYZER_SINGLE_PASS_RESOLVER_RELATION_BRIDGING_ENABLED]] is enabled, we need to
+   *    re-instantiate the [[RelationMetadataProvider]] as [[View]] resolution context might have
+   *    changed in the meantime;
+   * 3. Resolve the plan using [[resolve]].
+   *
+   * This method is called for the top-level query and each unresolved [[View]].
    */
   def lookupMetadataAndResolve(
       unresolvedPlan: LogicalPlan,
       analyzerBridgeState: Option[AnalyzerBridgeState] = None): LogicalPlan = {
-    planLogger.logPlanResolutionEvent(unresolvedPlan, "Lookup metadata and resolve")
+    planLogger.logPlanResolutionEvent(unresolvedPlan, "IDENTIFIER and CTE substitution")
+
+    val planAfterSubstitution = identifierAndCteSubstitutor.substitutePlan(unresolvedPlan)
+
+    planLogger.logPlanResolutionEvent(planAfterSubstitution, "Metadata lookup")
 
     relationMetadataProvider = analyzerBridgeState match {
       case Some(analyzerBridgeState) =>
         new BridgedRelationMetadataProvider(
           catalogManager,
           relationResolution,
-          analyzerBridgeState
+          analyzerBridgeState,
+          viewResolver
         )
       case None =>
         relationMetadataProvider
     }
 
-    relationMetadataProvider.resolve(unresolvedPlan)
+    relationMetadataProvider.resolve(planAfterSubstitution)
 
-    resolve(unresolvedPlan)
+    planLogger.logPlanResolutionEvent(planAfterSubstitution, "Main resolution")
+
+    planAfterSubstitution.setTagValue(ResolverTag.TOP_LEVEL_OPERATOR, ())
+
+    resolve(planAfterSubstitution)
   }
 
   /**
@@ -165,18 +191,29 @@ class Resolver(
    * [[resolve]] will be called recursively during the unresolved plan traversal eventually
    * producing a fully resolved plan or a descriptive error message.
    */
-  override def resolve(unresolvedPlan: LogicalPlan): LogicalPlan =
-    withOrigin(unresolvedPlan.origin) {
+  override def resolve(unresolvedPlan: LogicalPlan): LogicalPlan = {
+    val previousOrigin = CurrentOrigin.get
+    CurrentOrigin.set(unresolvedPlan.origin)
+
+    try {
       planLogger.logPlanResolutionEvent(unresolvedPlan, "Unresolved plan")
 
       val resolvedPlan =
         unresolvedPlan match {
+          case unresolvedJoin: Join =>
+            joinResolver.resolve(unresolvedJoin)
           case unresolvedWith: UnresolvedWith =>
             resolveWith(unresolvedWith)
+          case withCte: WithCTE =>
+            handleResolvedWithCte(withCte)
           case unresolvedProject: Project =>
             projectResolver.resolve(unresolvedProject)
+          case unresolvedAggregate: Aggregate =>
+            aggregateResolver.resolve(unresolvedAggregate)
           case unresolvedFilter: Filter =>
-            resolveFilter(unresolvedFilter)
+            filterResolver.resolve(unresolvedFilter)
+          case unresolvedHaving: UnresolvedHaving =>
+            havingResolver.resolve(unresolvedHaving)
           case unresolvedSubqueryColumnAliases: UnresolvedSubqueryColumnAliases =>
             resolveSubqueryColumnAliases(unresolvedSubqueryColumnAliases)
           case unresolvedSubqueryAlias: SubqueryAlias =>
@@ -187,16 +224,26 @@ class Resolver(
             resolveGlobalLimit(unresolvedGlobalLimit)
           case unresolvedLocalLimit: LocalLimit =>
             resolveLocalLimit(unresolvedLocalLimit)
+          case unresolvedOffset: Offset =>
+            resolveOffset(unresolvedOffset)
+          case unresolvedTail: Tail =>
+            resolveTail(unresolvedTail)
           case unresolvedDistinct: Distinct =>
             resolveDistinct(unresolvedDistinct)
           case unresolvedRelation: UnresolvedRelation =>
             resolveRelation(unresolvedRelation)
-          case unresolvedCteRelationDef: CTERelationDef =>
-            resolveCteRelationDef(unresolvedCteRelationDef)
+          case unresolvedCteRelationRef: UnresolvedCteRelationRef =>
+            resolveCteRelationRef(unresolvedCteRelationRef)
+          case cteRelationDef: CTERelationDef =>
+            handleResolvedCteRelationDef(cteRelationDef)
+          case cteRelationRef: CTERelationRef =>
+            handleLeafOperator(cteRelationRef)
           case unresolvedInlineTable: UnresolvedInlineTable =>
             resolveInlineTable(unresolvedInlineTable)
-          case unresolvedUnion: Union =>
-            unionResolver.resolve(unresolvedUnion)
+          case unresolvedSetOperationLike @ (_: Union | _: SetOperation) =>
+            setOperationLikeResolver.resolve(unresolvedSetOperationLike)
+          case unresolvedSort: Sort =>
+            sortResolver.resolve(unresolvedSort)
           // See the reason why we have to match both [[LocalRelation]] and [[ResolvedInlineTable]]
           // in the [[resolveInlineTable]] scaladoc
           case resolvedInlineTable: ResolvedInlineTable =>
@@ -205,34 +252,33 @@ class Resolver(
             handleLeafOperator(localRelation)
           case unresolvedOneRowRelation: OneRowRelation =>
             handleLeafOperator(unresolvedOneRowRelation)
+          case unresolvedRange: Range =>
+            handleLeafOperator(unresolvedRange)
+          case supervisingCommand: SupervisingCommand =>
+            resolveSupervisingCommand(supervisingCommand)
+          case repartition: Repartition =>
+            resolveRepartition(repartition)
+          case sample: Sample =>
+            resolveSample(sample)
           case _ =>
             tryDelegateResolutionToExtension(unresolvedPlan).getOrElse {
               handleUnmatchedOperator(unresolvedPlan)
             }
         }
 
-      if (resolvedPlan.children.nonEmpty) {
-        val missingInput = resolvedPlan.missingInput
-        if (missingInput.nonEmpty) {
-          withPosition(unresolvedPlan) {
-            throwMissingAttributesError(resolvedPlan, missingInput)
-          }
-        }
-      }
-
-      if (!resolvedPlan.resolved) {
-        throwSinglePassFailedToResolveOperator(resolvedPlan)
+      withPosition(unresolvedPlan) {
+        validateResolvedOperatorGenerically(resolvedPlan)
       }
 
       planLogger.logPlanResolution(unresolvedPlan, resolvedPlan)
 
-      preservePlanIdTag(unresolvedPlan, resolvedPlan)
-    }
+      resolvedPlan.copyTagsFrom(unresolvedPlan)
 
-  /**
-   * Get [[NameScopeStack]] bound to the used [[Resolver]].
-   */
-  def getNameScopes: NameScopeStack = scopes
+      resolvedPlan
+    } finally {
+      CurrentOrigin.set(previousOrigin)
+    }
+  }
 
   /**
    * [[UnresolvedWith]] contains a list of unresolved CTE definitions, which are represented by
@@ -247,59 +293,61 @@ class Resolver(
    * See [[CteScope]] scaladoc for all the details on how CTEs are resolved.
    */
   private def resolveWith(unresolvedWith: UnresolvedWith): LogicalPlan = {
-    val childOutputs = new ArrayBuffer[Seq[Attribute]]
-
     for (cteRelation <- unresolvedWith.cteRelations) {
-      val (cteName, ctePlan) = cteRelation
+      val (cteName, ctePlan, _) = cteRelation
 
-      val resolvedCtePlan = scopes.withNewScope {
-        expressionIdAssigner.withNewMapping() {
-          cteRegistry.withNewScope() {
-            val resolvedCtePlan = resolve(ctePlan)
+      expressionIdAssigner.pushMapping()
+      scopes.pushScope()
+      cteRegistry.pushScope()
 
-            childOutputs.append(scopes.top.output)
-
-            resolvedCtePlan
-          }
-        }
+      val resolvedCtePlan = try {
+        resolve(ctePlan)
+      } finally {
+        cteRegistry.popScope()
+        scopes.popScope()
+        expressionIdAssigner.popMapping()
       }
 
       cteRegistry.currentScope.registerCte(cteName, CTERelationDef(resolvedCtePlan))
     }
 
-    val resolvedChild = cteRegistry.withNewScope() {
+    cteRegistry.pushScope()
+
+    val resolvedChild = try {
       resolve(unresolvedWith.child)
+    } finally {
+      cteRegistry.popScope()
     }
 
-    childOutputs.append(scopes.top.output)
-
-    ExpressionIdAssigner.assertOutputsHaveNoConflictingExpressionIds(childOutputs.toSeq)
-
-    if (cteRegistry.currentScope.isRoot) {
-      WithCTE(resolvedChild, cteRegistry.currentScope.getKnownCtes)
-    } else {
-      resolvedChild
-    }
+    cteRegistry.currentScope.tryPutWithCTE(
+      unresolvedOperator = unresolvedWith,
+      resolvedOperator = resolvedChild
+    )
   }
 
   /**
-   * [[Filter]] has a single child and a single condition and we resolve them in this respective
-   * order.
+   * We may meet resolved [[WithCTE]] while traversing partially resolved trees in DataFrame
+   * programs. In that case we simply recurse into the CTE definitions and the main plan under
+   * new scopes and mappings.
    */
-  private def resolveFilter(unresolvedFilter: Filter): LogicalPlan = {
-    val resolvedChild = resolve(unresolvedFilter.child)
-    val resolvedCondition =
-      expressionResolver
-        .resolveExpressionTreeInOperator(unresolvedFilter.condition, unresolvedFilter)
+  private def handleResolvedWithCte(withCte: WithCTE): LogicalPlan = {
+    val resolvedCteDefs = withCte.cteDefs.map { cteDef =>
+      expressionIdAssigner.pushMapping()
+      scopes.pushScope()
+      cteRegistry.pushScope()
 
-    val resolvedFilter = Filter(resolvedCondition, resolvedChild)
-    if (resolvedFilter.condition.dataType != BooleanType) {
-      withPosition(unresolvedFilter) {
-        throwDatatypeMismatchFilterNotBoolean(resolvedFilter)
+      try {
+        resolve(cteDef).asInstanceOf[CTERelationDef]
+      } finally {
+        cteRegistry.popScope()
+        scopes.popScope()
+        expressionIdAssigner.popMapping()
       }
     }
 
-    resolvedFilter
+    val resolvedPlan = resolve(withCte.plan)
+
+    WithCTE(plan = resolvedPlan, cteDefs = resolvedCteDefs)
   }
 
   /**
@@ -315,21 +363,20 @@ class Resolver(
       unresolvedSubqueryColumnAliases: UnresolvedSubqueryColumnAliases): LogicalPlan = {
     val resolvedChild = resolve(unresolvedSubqueryColumnAliases.child)
 
-    if (unresolvedSubqueryColumnAliases.outputColumnNames.size != scopes.top.output.size) {
-      withPosition(unresolvedSubqueryColumnAliases) {
-        throw QueryCompilationErrors.aliasNumberNotMatchColumnNumberError(
-          unresolvedSubqueryColumnAliases.outputColumnNames.size,
-          scopes.top.output.size,
-          unresolvedSubqueryColumnAliases
-        )
+    if (unresolvedSubqueryColumnAliases.outputColumnNames.size != scopes.current.output.size) {
+      throw QueryCompilationErrors.aliasNumberNotMatchColumnNumberError(
+        unresolvedSubqueryColumnAliases.outputColumnNames.size,
+        scopes.current.output.size,
+        unresolvedSubqueryColumnAliases
+      )
+    }
+
+    val projectList =
+      scopes.current.output.zip(unresolvedSubqueryColumnAliases.outputColumnNames).map {
+        case (attr, columnName) => expressionIdAssigner.mapExpression(Alias(attr, columnName)())
       }
-    }
 
-    val projectList = scopes.top.output.zip(unresolvedSubqueryColumnAliases.outputColumnNames).map {
-      case (attr, columnName) => expressionIdAssigner.mapExpression(Alias(attr, columnName)())
-    }
-
-    overwriteTopScope(unresolvedSubqueryColumnAliases, projectList.map(_.toAttribute))
+    scopes.overwriteCurrent(output = Some(projectList.map(_.toAttribute)))
 
     Project(projectList = projectList, child = resolvedChild)
   }
@@ -338,15 +385,33 @@ class Resolver(
    * [[SubqueryAlias]] has a single child and an identifier. We need to resolve the child and update
    * the scope with the output, since upper expressions can reference [[SubqueryAlias]]es output by
    * its identifier.
+   *
+   * Hidden output is reset when [[SubqueryAlias]] is reached during tree traversal. This has to be
+   * done because upper SQL projection (or DataFrame) cannot look down into the previous
+   * [[SubqueryAlias]]'s hidden output.
+   * Examples (both will throw `UNRESOLVED_COLUMN` exception):
+   *
+   *  1. SQL
+   *  {{{
+   *  -- `hiddenOutput` and `availableAliases` will be reset at [[SubqueryAlias]]. Therefore both
+   *  -- `output` and `hiddenOutput` will be [`col2`]. Because of that, `UNRESOLVED_COLUMN` is
+   *  -- thrown when resolving `col1`
+   *  SELECT col1 FROM (SELECT col2 FROM VALUES (1, 2));
+   *  }}}
+   *
+   *  2. DataFrame
+   *  {{{ spark.sql("SELECT * FROM VALUES (1, 2)").select("col1").as("q1").select("col2"); }}}
    */
   private def resolveSubqueryAlias(unresolvedSubqueryAlias: SubqueryAlias): LogicalPlan = {
     val resolvedSubqueryAlias =
       unresolvedSubqueryAlias.copy(child = resolve(unresolvedSubqueryAlias.child))
 
     val qualifier = resolvedSubqueryAlias.identifier.qualifier :+ resolvedSubqueryAlias.alias
-    overwriteTopScope(
-      unresolvedSubqueryAlias,
-      scopes.top.output.map(attribute => attribute.withQualifier(qualifier))
+    val output = scopes.current.output.map(attribute => attribute.withQualifier(qualifier))
+    scopes.overwriteCurrent(
+      output = Some(output),
+      hiddenOutput = Some(output),
+      availableAliases = Some(new HashSet[ExprId])
     )
 
     resolvedSubqueryAlias
@@ -359,12 +424,10 @@ class Resolver(
   private def resolveGlobalLimit(unresolvedGlobalLimit: GlobalLimit): LogicalPlan = {
     val resolvedChild = resolve(unresolvedGlobalLimit.child)
 
-    val resolvedLimitExpr = withPosition(unresolvedGlobalLimit) {
-      expressionResolver.resolveLimitExpression(
-        unresolvedGlobalLimit.limitExpr,
-        unresolvedGlobalLimit
-      )
-    }
+    val resolvedLimitExpr = expressionResolver.resolveLimitLikeExpression(
+      unresolvedGlobalLimit.limitExpr,
+      unresolvedGlobalLimit.copy(child = resolvedChild)
+    )
 
     GlobalLimit(resolvedLimitExpr, resolvedChild)
   }
@@ -376,21 +439,63 @@ class Resolver(
   private def resolveLocalLimit(unresolvedLocalLimit: LocalLimit): LogicalPlan = {
     val resolvedChild = resolve(unresolvedLocalLimit.child)
 
-    val resolvedLimitExpr = withPosition(unresolvedLocalLimit) {
-      expressionResolver.resolveLimitExpression(
-        unresolvedLocalLimit.limitExpr,
-        unresolvedLocalLimit
-      )
-    }
+    val resolvedLimitExpr = expressionResolver.resolveLimitLikeExpression(
+      unresolvedLocalLimit.limitExpr,
+      unresolvedLocalLimit.copy(child = resolvedChild)
+    )
 
     LocalLimit(resolvedLimitExpr, resolvedChild)
   }
 
+  /**
+   * Resolve [[Offset]]. We have to resolve its child and resolve and validate its offset
+   * expression.
+   */
+  private def resolveOffset(unresolvedOffset: Offset): LogicalPlan = {
+    val resolvedChild = resolve(unresolvedOffset.child)
+
+    val resolvedOffsetExpr = expressionResolver.resolveLimitLikeExpression(
+      unresolvedOffset.offsetExpr,
+      unresolvedOffset.copy(child = resolvedChild)
+    )
+
+    Offset(resolvedOffsetExpr, resolvedChild)
+  }
+
+  /**
+   * Resolve [[Tail]]. We have to resolve its child and resolve and validate its limit
+   * expression.
+   */
+  private def resolveTail(unresolvedTail: Tail): LogicalPlan = {
+    val resolvedChild = resolve(unresolvedTail.child)
+
+    val resolvedTailExpr = expressionResolver.resolveLimitLikeExpression(
+      unresolvedTail.limitExpr,
+      unresolvedTail.copy(child = resolvedChild)
+    )
+
+    Tail(resolvedTailExpr, resolvedChild)
+  }
+
   /**
    * [[Distinct]] operator doesn't require any special resolution.
+   * We validate results of the resolution using the [[OperatorWithUncomparableTypeValidator]]
+   * ([[MapType]], [[VariantType]], [[GeometryType]] and [[GeographyType]] are not supported
+   * under [[Distinct]] operator).
+   *
+   * `hiddenOutput` and `availableAliases` are reset when [[Distinct]] is reached during tree
+   * traversal.
    */
   private def resolveDistinct(unresolvedDistinct: Distinct): LogicalPlan = {
-    withResolvedChildren(unresolvedDistinct, resolve)
+    val resolvedDistinct = unresolvedDistinct.copy(child = resolve(unresolvedDistinct.child))
+
+    OperatorWithUncomparableTypeValidator.validate(resolvedDistinct, scopes.current.output)
+
+    scopes.overwriteCurrent(
+      hiddenOutput = Some(scopes.current.output),
+      availableAliases = Some(new HashSet[ExprId])
+    )
+    resolvedDistinct
   }
 
   /**
@@ -400,49 +505,53 @@ class Resolver(
    * - Resolve it further, usually using extensions, like [[DataSourceResolver]]
    */
   private def resolveRelation(unresolvedRelation: UnresolvedRelation): LogicalPlan = {
-    withPosition(unresolvedRelation) {
-      viewResolver.withSourceUnresolvedRelation(unresolvedRelation) {
-        val maybeResolvedRelation = cteRegistry.resolveCteName(unresolvedRelation.name).orElse {
-          relationMetadataProvider.getRelationWithResolvedMetadata(unresolvedRelation)
-        }
+    viewResolver.withSourceUnresolvedRelation(unresolvedRelation) {
+      val maybeResolvedRelation =
+        relationMetadataProvider.getRelationWithResolvedMetadata(unresolvedRelation)
 
-        val resolvedRelation = maybeResolvedRelation match {
-          case Some(cteRelationDef: CTERelationDef) =>
-            planLogger.logPlanResolutionEvent(cteRelationDef, "CTE definition resolved")
+      val resolvedRelation = maybeResolvedRelation match {
+        case Some(relationsWithResolvedMetadata) =>
+          planLogger.logPlanResolutionEvent(
+            relationsWithResolvedMetadata,
+            "Relation metadata retrieved"
+          )
 
-            SubqueryAlias(identifier = unresolvedRelation.name, child = cteRelationDef)
-          case Some(relationsWithResolvedMetadata) =>
-            planLogger.logPlanResolutionEvent(
-              relationsWithResolvedMetadata,
-              "Relation metadata retrieved"
-            )
-
-            relationsWithResolvedMetadata
-          case None =>
-            unresolvedRelation.tableNotFound(unresolvedRelation.multipartIdentifier)
-        }
-
-        resolve(resolvedRelation)
+          relationsWithResolvedMetadata
+        case None =>
+          unresolvedRelation.tableNotFound(unresolvedRelation.multipartIdentifier)
       }
+
+      resolve(resolvedRelation)
     }
   }
 
   /**
-   * Resolve [[CTERelationDef]] by replacing it with [[CTERelationRef]] with the same ID so that
-   * the Optimizer can make a decision whether to inline the definition or not.
-   *
-   * [[CTERelationDef.statsOpt]] is filled by the Optimizer.
+   * Resolve the [[UnresolvedCteRelationRef]] which was previously introduved by the
+   * [[IdentifierAndCteSubstitutor]].
    */
-  private def resolveCteRelationDef(unresolvedCteRelationDef: CTERelationDef): LogicalPlan = {
-    val cteRelationRef = CTERelationRef(
-      cteId = unresolvedCteRelationDef.id,
-      _resolved = true,
-      isStreaming = unresolvedCteRelationDef.isStreaming,
-      output = unresolvedCteRelationDef.output,
-      recursive = false
-    )
+  private def resolveCteRelationRef(
+      unresolvedCteRelationRef: UnresolvedCteRelationRef): LogicalPlan = {
+    val cteRelationRef = cteRegistry.resolveCteName(unresolvedCteRelationRef.name) match {
+      case Some(cteRelationDef) =>
+        planLogger.logPlanResolutionEvent(cteRelationDef, "CTE definition resolved")
 
-    handleLeafOperator(cteRelationRef)
+        createCteRelationRef(
+          name = unresolvedCteRelationRef.name,
+          cteRelationDef = cteRelationDef
+        )
+      case None =>
+        unresolvedCteRelationRef.tableNotFound(Seq(unresolvedCteRelationRef.name))
+    }
+
+    resolve(cteRelationRef)
+  }
+
+  /**
+   * We may meet resolved [[CTERelationRef]] while traversing partially resolved trees in DataFrame
+   * programs. In that case we simply recurse into the child plan.
+   */
+  private def handleResolvedCteRelationDef(cteRelationDef: CTERelationDef): LogicalPlan = {
+    cteRelationDef.copy(child = resolve(cteRelationDef.child))
   }
 
   /**
@@ -469,28 +578,12 @@ class Resolver(
     val resolvedRelation = EvaluateUnresolvedInlineTable
       .evaluateUnresolvedInlineTable(withResolvedExpressions)
 
-    withPosition(unresolvedInlineTable) {
-      resolve(resolvedRelation)
-    }
-  }
-
-  /**
-   * Preserve `PLAN_ID_TAG` which is used for DataFrame column resolution in Spark Connect.
-   */
-  private def preservePlanIdTag(
-      unresolvedOperator: LogicalPlan,
-      resolvedOperator: LogicalPlan): LogicalPlan = {
-    unresolvedOperator.getTagValue(LogicalPlan.PLAN_ID_TAG) match {
-      case Some(planIdTag) =>
-        resolvedOperator.setTagValue(LogicalPlan.PLAN_ID_TAG, planIdTag)
-      case None =>
-    }
-    resolvedOperator
+    resolve(resolvedRelation)
   }
 
   private def tryDelegateResolutionToExtension(
       unresolvedOperator: LogicalPlan): Option[LogicalPlan] = {
-    val resolutionResult = super.tryDelegateResolutionToExtension(unresolvedOperator, this)
+    val resolutionResult = runExtensions(unresolvedOperator)
     resolutionResult match {
       case Some(leafOperator: LeafNode) =>
         Some(handleLeafOperator(leafOperator))
@@ -499,6 +592,10 @@ class Resolver(
     }
   }
 
+  private def runExtensions(unresolvedOperator: LogicalPlan): Option[LogicalPlan] = {
+    super.tryDelegateResolutionToExtension(unresolvedOperator, this)
+  }
+
   /**
    * Leaf operators introduce original attributes to this operator subtree and need to be handled in
    * a special way:
@@ -506,57 +603,94 @@ class Resolver(
    *    `leafOperator`'s output attribute IDs. We don't reassign expression IDs in the leftmost
    *    branch, see [[ExpressionIdAssigner]] class doc for more details.
    *    [[CTERelationRef]]'s output can always be reassigned.
-   *  - Overwrite the current [[NameScope]] with remapped output attributes. It's OK to call
+   *  - Overwrite the current [[NameScope]] with remapped output attributes (both
+   *    [[NameScope.output]] and [[NameScope.hiddenOutput]] are updated). It's OK to call
    *    `output` on a [[LeafNode]], because it's not recursive (this call fits the single-pass
    *    framework).
    */
   private def handleLeafOperator(leafOperator: LeafNode): LogicalPlan = {
     val leafOperatorWithAssignedExpressionIds = leafOperator match {
-      case leafOperator
-          if expressionIdAssigner.isLeftmostBranch && !leafOperator.isInstanceOf[CTERelationRef] =>
-        expressionIdAssigner.createMapping(newOutput = leafOperator.output)
+      case leafOperator if expressionIdAssigner.shouldPreserveLeafOperatorIds(leafOperator) =>
+        expressionIdAssigner.createMappingForLeafOperator(newOperator = leafOperator)
+
         leafOperator
 
-      /**
-       * [[InMemoryRelation.statsOfPlanToCache]] is mutable and does not get copied during normal
-       * [[transformExpressionsUp]]. The easiest way to correctly copy it is via [[newInstance]]
-       * call.
-       *
-       * We match [[MultiInstanceRelation]] to avoid a cyclic import between [[catalyst]] and
-       * [[execution]].
-       */
-      case originalRelation: MultiInstanceRelation =>
-        val newRelation = originalRelation.newInstance()
+      case originalLeafOperator =>
+        val newLeafOperator = originalLeafOperator match {
 
-        expressionIdAssigner.createMapping(
-          newOutput = newRelation.output,
-          oldOutput = Some(originalRelation.output)
+          /**
+           * [[InMemoryRelation.statsOfPlanToCache]] is mutable and does not get copied during
+           * [[transformExpressionsUp]]. The easiest way to correctly copy it is via
+           * [[newInstance]] call.
+           *
+           * We match [[MultiInstanceRelation]] to avoid a cyclic import between [[catalyst]] and
+           * [[execution]].
+           */
+          case originalRelation: MultiInstanceRelation =>
+            originalRelation.newInstance().asInstanceOf[LeafNode]
+
+          case _ =>
+            AnalysisHelper
+              .allowInvokingTransformsInAnalyzer {
+                leafOperator.transformExpressions {
+                  case attribute: Attribute => attribute.newInstance()
+                }
+              }
+              .asInstanceOf[LeafNode]
+        }
+
+        expressionIdAssigner.createMappingForLeafOperator(
+          newOperator = newLeafOperator,
+          oldOperator = Some(originalLeafOperator)
         )
 
-        newRelation
-      case _ =>
-        expressionIdAssigner.createMapping()
-
-        AnalysisHelper.allowInvokingTransformsInAnalyzer {
-          leafOperator.transformExpressionsUp {
-            case expression: NamedExpression =>
-              val newExpression = expressionIdAssigner.mapExpression(expression)
-              if (newExpression.eq(expression)) {
-                throw SparkException.internalError(
-                  s"Leaf operator expression ID was not reassigned. Expression: $expression, " +
-                  s"leaf operator: $leafOperator"
-                )
-              }
-              newExpression
-          }
-        }
+        newLeafOperator
     }
 
-    overwriteTopScope(leafOperator, leafOperatorWithAssignedExpressionIds.output)
+    val output = leafOperatorWithAssignedExpressionIds.output
+    scopes.overwriteCurrent(output = Some(output), hiddenOutput = Some(output))
 
     leafOperatorWithAssignedExpressionIds
   }
 
+  /**
+   * [[SupervisingCommand]] is an [[ExplainCommand]] or [[DescribeQueryCommand]] that doesn't
+   * require any resolution.
+   */
+  private def resolveSupervisingCommand(supervisingCommand: SupervisingCommand): LogicalPlan = {
+    supervisingCommand
+  }
+
+  /**
+   * Resolve [[Repartition]] operator. Its resolution doesn't require any specific logic (besides
+   * child resolution).
+   */
+  private def resolveRepartition(repartition: Repartition): LogicalPlan = {
+    repartition.copy(child = resolve(repartition.child))
+  }
+
+  /**
+   * Resolve [[Sample]] operator. Its resolution doesn't require any specific logic (besides
+   * child resolution).
+   */
+  private def resolveSample(sample: Sample): LogicalPlan = {
+    sample.copy(child = resolve(sample.child))
+  }
+
+  private def createCteRelationRef(name: String, cteRelationDef: CTERelationDef): LogicalPlan = {
+    SubqueryAlias(
+      identifier = name,
+      child = CTERelationRef(
+        cteId = cteRelationDef.id,
+        _resolved = true,
+        isStreaming = cteRelationDef.isStreaming,
+        output = cteRelationDef.output,
+        recursive = false,
+        maxRows = cteRelationDef.maxRows
+      )
+    )
+  }
+
   /**
    * Check if the unresolved operator is explicitly unsupported and throw
    * [[ExplicitlyUnsupportedResolverFeature]] in that case. Otherwise, throw
@@ -577,15 +711,23 @@ class Resolver(
       .withPosition(unresolvedOperator.origin)
   }
 
-  private def throwDatatypeMismatchFilterNotBoolean(filter: Filter): Nothing =
-    throw new AnalysisException(
-      errorClass = "DATATYPE_MISMATCH.FILTER_NOT_BOOLEAN",
-      messageParameters = Map(
-        "sqlExpr" -> makeCommaSeparatedExpressionString(filter.expressions),
-        "filter" -> toSQLExpr(filter.condition),
-        "type" -> toSQLType(filter.condition.dataType)
-      )
-    )
+  private def validateResolvedOperatorGenerically(resolvedOperator: LogicalPlan): Unit = {
+    if (!resolvedOperator.resolved) {
+      throwSinglePassFailedToResolveOperator(resolvedOperator)
+    }
+
+    if (resolvedOperator.children.nonEmpty) {
+      val missingInput = resolvedOperator.missingInput
+      if (missingInput.nonEmpty) {
+        throwMissingAttributesError(resolvedOperator, missingInput)
+      }
+    }
+
+    val invalidExpressions = expressionResolver.getLastInvalidExpressionsInTheContextOfOperator
+    if (invalidExpressions.nonEmpty) {
+      throwUnsupportedExprForOperator(invalidExpressions)
+    }
+  }
 
   private def throwMissingAttributesError(
       operator: LogicalPlan,
@@ -630,16 +772,17 @@ class Resolver(
       summary = operator.origin.context.summary()
     )
 
-  private def makeCommaSeparatedExpressionString(expressions: Seq[Expression]): String = {
-    expressions.map(toSQLExpr).mkString(", ")
+  private def throwUnsupportedExprForOperator(invalidExpressions: Seq[Expression]): Nothing = {
+    throw new AnalysisException(
+      errorClass = "UNSUPPORTED_EXPR_FOR_OPERATOR",
+      messageParameters = Map(
+        "invalidExprSqls" -> makeCommaSeparatedExpressionString(invalidExpressions)
+      )
+    )
   }
 
-  private def overwriteTopScope(
-      sourceUnresolvedOperator: LogicalPlan,
-      output: Seq[Attribute]): Unit = {
-    withPosition(sourceUnresolvedOperator) {
-      scopes.overwriteTop(output)
-    }
+  private def makeCommaSeparatedExpressionString(expressions: Seq[Expression]): String = {
+    expressions.map(toSQLExpr).mkString(", ")
   }
 }
 
@@ -648,7 +791,9 @@ object Resolver {
   /**
    * Create a new instance of the [[RelationResolution]].
    */
-  def createRelationResolution(catalogManager: CatalogManager): RelationResolution = {
-    new RelationResolution(catalogManager)
+  def createRelationResolution(
+      catalogManager: CatalogManager,
+      sharedRelationCache: RelationCache = RelationCache.empty): RelationResolution = {
+    new RelationResolution(catalogManager, sharedRelationCache)
   }
 }
