@@ -63,7 +63,10 @@ case class CountMinSketchAgg(
   // Mark as lazy so that they are not evaluated during tree transformation.
   private lazy val eps: Double = epsExpression.eval().asInstanceOf[Double]
   private lazy val confidence: Double = confidenceExpression.eval().asInstanceOf[Double]
-  private lazy val seed: Int = seedExpression.eval().asInstanceOf[Int]
+  private lazy val seed: Int = seedExpression.eval() match {
+    case i: Int => i
+    case l: Long => l.toInt
+  }
 
   override def checkInputDataTypes(): TypeCheckResult = {
     val defaultCheck = super.checkInputDataTypes()
@@ -73,7 +76,7 @@ case class CountMinSketchAgg(
       DataTypeMismatch(
         errorSubClass = "NON_FOLDABLE_INPUT",
         messageParameters = Map(
-          "inputName" -> "eps",
+          "inputName" -> toSQLId("eps"),
           "inputType" -> toSQLType(epsExpression.dataType),
           "inputExpr" -> toSQLExpr(epsExpression))
       )
@@ -81,7 +84,7 @@ case class CountMinSketchAgg(
       DataTypeMismatch(
         errorSubClass = "NON_FOLDABLE_INPUT",
         messageParameters = Map(
-          "inputName" -> "confidence",
+          "inputName" -> toSQLId("confidence"),
           "inputType" -> toSQLType(confidenceExpression.dataType),
           "inputExpr" -> toSQLExpr(confidenceExpression))
       )
@@ -89,7 +92,7 @@ case class CountMinSketchAgg(
       DataTypeMismatch(
         errorSubClass = "NON_FOLDABLE_INPUT",
         messageParameters = Map(
-          "inputName" -> "seed",
+          "inputName" -> toSQLId("seed"),
           "inputType" -> toSQLType(seedExpression.dataType),
           "inputExpr" -> toSQLExpr(seedExpression))
       )
@@ -168,7 +171,8 @@ case class CountMinSketchAgg(
     copy(inputAggBufferOffset = newInputAggBufferOffset)
 
   override def inputTypes: Seq[AbstractDataType] = {
-    Seq(TypeCollection(IntegralType, StringType, BinaryType), DoubleType, DoubleType, IntegerType)
+    Seq(TypeCollection(IntegralType, StringType, BinaryType), DoubleType, DoubleType,
+      TypeCollection(IntegerType, LongType))
   }
 
   override def nullable: Boolean = false
