@@ -18,6 +18,7 @@
 package org.apache.spark.sql.catalyst.analysis
 
 import java.io.File
+import java.util
 
 import scala.jdk.CollectionConverters._
 
@@ -30,6 +31,7 @@ import org.apache.spark.sql.catalyst.TableIdentifier
 import org.apache.spark.sql.catalyst.catalog.{CatalogDatabase, CatalogStorageFormat, CatalogTable, CatalogTableType, ExternalCatalog, InMemoryCatalog, SessionCatalog}
 import org.apache.spark.sql.catalyst.dsl.plans._
 import org.apache.spark.sql.connector.catalog.{CatalogManager, Identifier, InMemoryTable, InMemoryTableCatalog, Table}
+import org.apache.spark.sql.connector.catalog.TableWritePrivilege
 import org.apache.spark.sql.errors.QueryExecutionErrors
 import org.apache.spark.sql.types._
 
@@ -55,6 +57,11 @@ class TableLookupCacheSuite extends AnalysisTest with Matchers {
           Array.empty,
           Map.empty[String, String].asJava)
       }
+      override def loadTable(
+          ident: Identifier,
+          writePrivileges: util.Set[TableWritePrivilege]): Table = {
+        loadTable(ident)
+      }
       override def name: String = CatalogManager.SESSION_CATALOG_NAME
     }
     val catalogManager = mock(classOf[CatalogManager])
