@@ -18,7 +18,7 @@
 package org.apache.spark.sql.catalyst.expressions
 
 import java.sql.{Date, Timestamp}
-import java.time.{Duration, LocalDate, LocalDateTime, Period}
+import java.time.{Duration, LocalDate, LocalDateTime, LocalTime, Period}
 import java.time.temporal.ChronoUnit
 import java.util.{Calendar, Locale, TimeZone}
 
@@ -82,7 +82,7 @@ abstract class CastSuiteBase extends SparkFunSuite with ExpressionEvalHelper {
     }
 
     atomicTypes.foreach(dt => checkNullCast(NullType, dt))
-    atomicTypes.foreach(dt => checkNullCast(dt, StringType))
+    (atomicTypes ++ timeTypes).foreach(dt => checkNullCast(dt, StringType))
     checkNullCast(StringType, BinaryType)
     checkNullCast(StringType, BooleanType)
     numericTypes.foreach(dt => checkNullCast(dt, BooleanType))
@@ -1438,4 +1438,458 @@ abstract class CastSuiteBase extends SparkFunSuite with ExpressionEvalHelper {
       }
     }
   }
+
+  test("cast time to string") {
+    Seq(
+      LocalTime.MIDNIGHT -> "00:00:00",
+      LocalTime.NOON -> "12:00:00",
+      LocalTime.of(23, 59, 59) -> "23:59:59",
+      LocalTime.of(23, 59, 59, 1000000) -> "23:59:59.001",
+      LocalTime.of(23, 59, 59, 999999000) -> "23:59:59.999999"
+    ).foreach { case (time, expectedStr) =>
+      checkEvaluation(Cast(Literal(time), StringType), expectedStr)
+    }
+
+    checkConsistencyBetweenInterpretedAndCodegen(
+      (child: Expression) => Cast(child, StringType), TimeType())
+  }
+
+  test("Casting geospatial data types to/from other data types") {
+    val geoTypes: Seq[DataType] = Seq(
+      GeographyType(4326),
+      GeographyType("ANY"),
+      GeometryType(0),
+      GeometryType(3857),
+      GeometryType(4326),
+      GeometryType("ANY")
+    )
+    val otherTypes: Seq[DataType] = Seq(
+      BinaryType,
+      BooleanType,
+      ByteType,
+      StringType,
+      StringType("UTF8_LCASE"),
+      StringType("UNICODE_CI"),
+      ShortType,
+      IntegerType,
+      LongType,
+      FloatType,
+      DoubleType
+    )
+    // Iterate over the test cases and verify casting.
+    geoTypes.foreach { geoType =>
+      otherTypes.foreach { otherType =>
+        // Cast cannot be performed from `geoType` to `otherType`.
+        assert(!Cast.canCast(geoType, otherType))
+        assert(!Cast.canAnsiCast(geoType, otherType))
+        // Cast cannot be performed from `otherType` to `geoType`.
+        assert(!Cast.canCast(otherType, geoType))
+        assert(!Cast.canAnsiCast(otherType, geoType))
+      }
+    }
+  }
+
+  // The following tests are confirming the behavior of casting between geospatial types.
+
+  test("Casting GeographyType to GeographyType") {
+    // Casting from fixed SRID GEOGRAPHY(<srid>) to mixed SRID GEOGRAPHY(ANY) is always allowed.
+    // Type casting is always safe in this direction, so no additional constraints are imposed.
+    // Casting from mixed SRID GEOGRAPHY(ANY) to fixed SRID GEOGRAPHY(<srid>) is not allowed.
+    // Type casting can be unsafe in this direction, because per-row SRID values may be different.
+
+    // Valid cast test cases.
+    val canCastTestCases: Seq[(DataType, DataType)] = Seq(
+      (GeographyType(4326), GeographyType("ANY"))
+    )
+    // Iterate over the test cases and verify casting.
+    canCastTestCases.foreach { case (fromType, toType) =>
+      // Cast can be performed from `fromType` to `toType`.
+      assert(Cast.canCast(fromType, toType))
+      assert(Cast.canAnsiCast(fromType, toType))
+      // Cast cannot be performed from `toType` to `fromType`.
+      assert(!Cast.canCast(toType, fromType))
+      assert(!Cast.canAnsiCast(toType, fromType))
+    }
+  }
+
+  test("Casting GeographyType to GeometryType") {
+    // Casting from GEOGRAPHY to GEOMETRY is only allowed if the SRIDs are the same.
+
+    // Valid cast test cases.
+    val canAnsiCastTestCases: Seq[(DataType, DataType)] = Seq(
+      (GeographyType(4326), GeometryType(4326)),
+      (GeographyType("ANY"), GeometryType("ANY"))
+    )
+    // Iterate over the test cases and verify casting.
+    canAnsiCastTestCases.foreach { case (fromType, toType) =>
+      // Cast can be performed from `fromType` to `toType`.
+      assert(Cast.canCast(fromType, toType))
+      assert(Cast.canAnsiCast(fromType, toType))
+    }
+
+    // Invalid cast test cases.
+    val cannotAnsiCastTestCases: Seq[(DataType, DataType)] = Seq(
+      (GeographyType(4326), GeometryType(0)),
+      (GeographyType(4326), GeometryType(3857)),
+      (GeographyType(4326), GeometryType("ANY")),
+      (GeographyType("ANY"), GeometryType(0)),
+      (GeographyType("ANY"), GeometryType(3857)),
+      (GeographyType("ANY"), GeometryType(4326))
+    )
+    // Iterate over the test cases and verify casting.
+    cannotAnsiCastTestCases.foreach { case (fromType, toType) =>
+      // Cast cannot be performed from `fromType` to `toType`.
+      assert(!Cast.canCast(fromType, toType))
+      assert(!Cast.canAnsiCast(fromType, toType))
+    }
+  }
+
+  test("Casting GeometryType to GeometryType") {
+    // Casting from fixed SRID GEOMETRY(<srid>) to mixed SRID GEOMETRY(ANY) is always allowed.
+    // Type casting is always safe in this direction, so no additional constraints are imposed.
+    // Casting from mixed SRID GEOMETRY(ANY) to fixed SRID GEOMETRY(<srid>) is not allowed.
+    // Type casting can be unsafe in this direction, because per-row SRID values may be different.
+
+    // Valid cast test cases.
+    val canCastTestCases: Seq[(DataType, DataType)] = Seq(
+      (GeometryType(0), GeometryType("ANY")),
+      (GeometryType(3857), GeometryType("ANY")),
+      (GeometryType(4326), GeometryType("ANY"))
+    )
+    // Iterate over the test cases and verify casting.
+    canCastTestCases.foreach { case (fromType, toType) =>
+      // Cast can be performed from `fromType` to `toType`.
+      assert(Cast.canCast(fromType, toType))
+      assert(Cast.canAnsiCast(fromType, toType))
+      // Cast cannot be performed from `toType` to `fromType`.
+      assert(!Cast.canCast(toType, fromType))
+      assert(!Cast.canAnsiCast(toType, fromType))
+    }
+  }
+
+  test("cast string to time") {
+    checkEvaluation(cast(Literal.create("0:0:0"), TimeType()), 0L)
+    checkEvaluation(cast(Literal.create(" 01:2:3.01   "), TimeType(2)), localTime(1, 2, 3, 10000))
+    checkEvaluation(cast(Literal.create(" 12:13:14.999"),
+      TimeType(3)), localTime(12, 13, 14, 999 * 1000))
+    checkEvaluation(cast(Literal.create("23:0:59.0001 "), TimeType(4)), localTime(23, 0, 59, 100))
+    checkEvaluation(cast(Literal.create("23:59:0.99999"),
+      TimeType(5)), localTime(23, 59, 0, 999990))
+    checkEvaluation(cast(Literal.create("23:59:59.000001     "),
+      TimeType(6)), localTime(23, 59, 59, 1))
+  }
+
+  test("context independent foldable") {
+    val array = Literal.create(Seq(1, 2, 3), ArrayType(IntegerType))
+    val targetArrayType = ArrayType(StringType, containsNull = true)
+    val map = Literal.create(
+      Map("a" -> "123", "b" -> "true", "c" -> "f"),
+      MapType(StringType, StringType, valueContainsNull = true))
+    val targetMapType = MapType(StringType, StringType, valueContainsNull = false)
+    val struct = Literal.create(
+      Row(1, "2", true, null),
+      StructType(Seq(
+        StructField("a", IntegerType),
+        StructField("b", StringType),
+        StructField("c", BooleanType),
+        StructField("d", NullType))))
+    val targetStructType = StructType(Seq(
+      StructField("a", StringType),
+      StructField("b", IntegerType),
+      StructField("c", BooleanType),
+      StructField("d", BooleanType)))
+    Seq(
+      cast(Literal(1), StringType),
+      cast(Literal(1), DateType),
+      cast(Literal(1L), TimestampNTZType),
+      cast(Literal("1"), IntegerType),
+      cast(Literal("true"), BooleanType),
+      cast(Literal("false"), BooleanType),
+      cast(Literal(1.5), IntegerType),
+      cast(Literal(1.5), LongType),
+      cast(Literal(100), ByteType),
+      cast(Literal(1000), ShortType),
+      cast(Literal(1), DoubleType),
+      cast(Literal(2.5), FloatType),
+      cast(Literal(123), DecimalType(10, 2)),
+      cast(Literal("1.5"), DoubleType),
+      cast(Literal("2023-01-01"), DateType),
+      cast(Literal(true), StringType),
+      cast(Literal(false), IntegerType),
+      cast(Literal(Duration.ofDays(1)), StringType),
+      cast(Literal(Period.ofMonths(1)), StringType),
+      cast(Literal(Decimal(10.5)), IntegerType),
+      cast(Literal(null, NullType), BooleanType),
+      cast(array, targetArrayType),
+      cast(map, targetMapType),
+      cast(struct, targetStructType)
+    ).foreach { expr =>
+      assert(expr.foldable, s"Expression $expr should be foldable")
+      assert(expr.contextIndependentFoldable,
+        s"Expression $expr should be context independent foldable")
+    }
+  }
+
+  test("context dependent foldable") {
+    val array = Literal.create(Seq("1", "2", "3"), ArrayType(StringType))
+    val dateArray = Literal.create(Seq(
+      Date.valueOf("2023-01-01"),
+      Date.valueOf("2023-01-02"),
+      Date.valueOf("2023-01-03")), ArrayType(DateType))
+    val targetArrayType = ArrayType(TimestampType, containsNull = true)
+
+    val map = Literal.create(
+      Map("a" -> "123", "b" -> "true", "c" -> "f"),
+      MapType(StringType, StringType, valueContainsNull = true))
+    val dateMap = Literal.create(
+      Map("a" -> Date.valueOf("2023-01-01"),
+        "b" -> Date.valueOf("2023-01-02"),
+        "c" -> Date.valueOf("2023-01-03")),
+      MapType(StringType, DateType, valueContainsNull = true))
+    val targetMapType = MapType(StringType, TimestampType, valueContainsNull = true)
+    val struct = Literal.create(
+      Row(1, "2", true, null),
+      StructType(Seq(
+        StructField("a", IntegerType),
+        StructField("b", StringType),
+        StructField("c", BooleanType),
+        StructField("d", NullType))))
+    val dateStruct = Literal.create(
+      Row(1, Date.valueOf("2023-01-02"), true, null),
+      StructType(Seq(
+        StructField("a", IntegerType),
+        StructField("b", DateType),
+        StructField("c", BooleanType),
+        StructField("d", NullType))))
+    val targetStructType = StructType(Seq(
+      StructField("a", StringType),
+      StructField("b", TimestampType),
+      StructField("c", BooleanType),
+      StructField("d", BooleanType)))
+    Seq(
+      cast(Literal("abc"), TimestampType),
+      cast(Literal(Date.valueOf("2023-01-01")), TimestampType),
+      cast(array, targetArrayType),
+      cast(dateArray, targetArrayType),
+      cast(map, targetMapType),
+      cast(dateMap, targetMapType),
+      cast(struct, targetStructType),
+      cast(dateStruct, targetStructType)
+    ).foreach { expr =>
+      assert(expr.foldable, s"Expression $expr should be foldable")
+      assert(!expr.contextIndependentFoldable,
+        s"Expression $expr should not be context independent foldable")
+    }
+  }
+
+  test("SPARK-51562: cast alias - time function") {
+    import org.apache.spark.sql.catalyst.analysis.FunctionRegistry
+    import org.apache.spark.sql.catalyst.FunctionIdentifier
+    // Test that time() function is registered and works correctly.
+    val registry = FunctionRegistry.builtin
+    val timeFunction = registry.lookupFunctionBuilder(FunctionIdentifier("time"))
+    assert(timeFunction.isDefined, "time function should be registered in FunctionRegistry")
+    // Test that time() function creates a proper Cast expression.
+    val stringInput = Literal("12:34:56")
+    val timeExpr = timeFunction.get(Seq(stringInput))
+    assert(timeExpr.isInstanceOf[Cast])
+    // The return type of the cast expression should be TimeType().
+    val castExpr = timeExpr.asInstanceOf[Cast]
+    assert(castExpr.dataType === TimeType())
+
+    // Test basic string to time conversions using the alias.
+    checkEvaluation(timeExpr, localTime(12, 34, 56))
+    val timeExprWithMillis = timeFunction.get(Seq(Literal("12:34:56.789")))
+    checkEvaluation(timeExprWithMillis, localTime(12, 34, 56, 789000))
+    val timeExprWithMicros = timeFunction.get(Seq(Literal("12:34:56.789012")))
+    checkEvaluation(timeExprWithMicros, localTime(12, 34, 56, 789012))
+
+    // Test null inputs.
+    val timeExprNull = timeFunction.get(Seq(Literal.create(null, StringType)))
+    checkEvaluation(timeExprNull, null)
+  }
+
+  test("cast time to time") {
+    checkEvaluation(cast(Literal(localTime(), TimeType(0)), TimeType(0)), 0L)
+    checkEvaluation(cast(Literal(localTime(0, 0, 0, 1), TimeType(6)), TimeType(6)),
+      localTime(0, 0, 0, 1))
+    checkEvaluation(cast(Literal(localTime(0, 0, 0, 19), TimeType(6)), TimeType(5)),
+      localTime(0, 0, 0, 10))
+    checkEvaluation(cast(Literal(localTime(23, 59, 59, 999990), TimeType(5)), TimeType(6)),
+      localTime(23, 59, 59, 999990))
+    checkEvaluation(cast(Literal(localTime(23, 59, 59, 999999), TimeType(6)), TimeType(5)),
+      localTime(23, 59, 59, 999990))
+    checkEvaluation(cast(Literal(localTime(11, 58, 59, 123400), TimeType(4)), TimeType(5)),
+      localTime(11, 58, 59, 123400))
+    checkEvaluation(cast(Literal(localTime(19, 2, 3, 765000), TimeType(3)), TimeType(2)),
+      localTime(19, 2, 3, 760000))
+
+    for (sp <- TimeType.MIN_PRECISION to TimeType.MAX_PRECISION) {
+      for (tp <- TimeType.MIN_PRECISION to TimeType.MAX_PRECISION) {
+        checkConsistencyBetweenInterpretedAndCodegen(
+          (child: Expression) => Cast(child, TimeType(sp)), TimeType(tp))
+      }
+    }
+  }
+
+  test("SPARK-52620: cast time to decimal with sufficient precision and scale") {
+    // Test various TIME values converted to DecimalType(14, 9), which always has sufficient
+    // precision and scale to represent the number of (nano)seconds since midnight. Note that
+    // 5 decimal places are sufficient to represent the maximum number of seconds in any TIME
+    // value, while 9 decimal places are needed to account for nanosecond fractional values.
+    Seq(
+      // 00:00:00 -> 0
+      (LocalTime.MIDNIGHT, Decimal(0)),
+      // 01:00:00 -> 3600
+      (LocalTime.of(1, 0, 0), Decimal(3600)),
+      // 01:01:00 -> 3660
+      (LocalTime.of(1, 1, 0), Decimal(3660)),
+      // 01:01:01 -> 3661
+      (LocalTime.of(1, 1, 1), Decimal(3661)),
+      // 01:02:03 -> 3723
+      (LocalTime.of(1, 2, 3), Decimal(3723)),
+      // 12:00:00 -> 43200
+      (LocalTime.NOON, Decimal(43200)),
+      // 23:59:59 -> 86399
+      (LocalTime.of(23, 59, 59), Decimal(86399)),
+      // 23:59:59.1 -> 86399.1
+      (LocalTime.of(23, 59, 59, 100000000), Decimal(86399.1)),
+      // 23:59:59.01 -> 86399.01
+      (LocalTime.of(23, 59, 59, 10000000), Decimal(86399.01)),
+      // 23:59:59.001 -> 86399.001
+      (LocalTime.of(23, 59, 59, 1000000), Decimal(86399.001)),
+      // 23:59:59.0001 -> 86399.0001
+      (LocalTime.of(23, 59, 59, 100000), Decimal(86399.0001)),
+      // 23:59:59.00001 -> 86399.00001
+      (LocalTime.of(23, 59, 59, 10000), Decimal(86399.00001)),
+      // 23:59:59.000001 -> 86399.000001
+      (LocalTime.of(23, 59, 59, 1000), Decimal(86399.000001)),
+      // 23:59:59.0000001 -> 86399.0000001
+      (LocalTime.of(23, 59, 59, 100), Decimal(86399.0000001)),
+      // 23:59:59.00000001 -> 86399.00000001
+      (LocalTime.of(23, 59, 59, 10), Decimal(86399.00000001)),
+      // 23:59:59.000000001 -> 86399.000000001
+      (LocalTime.of(23, 59, 59, 1), Decimal(86399.000000001))
+    ).foreach { case (time, expectedDecimal) =>
+      checkEvaluation(Cast(Literal(time), DecimalType(14, 9)), expectedDecimal)
+    }
+
+    // Test with different decimal precision and scale. The precision and scale of the
+    // DecimalType should be sufficient to represent the number of seconds since midnight.
+    // However, if the scale of the DecimalType is larger than necessary, the fractional
+    // seconds will be padded with zeros - which is the correct and expected behavior.
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(5, 0)), Decimal(43200, 5, 0))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(6, 1)), Decimal(43200.0, 6, 1))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(7, 2)), Decimal(43200.00, 7, 2))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(8, 3)), Decimal(43200.000, 8, 3))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(9, 4)), Decimal(43200.0000, 9, 4))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(10, 5)), Decimal(43200.00000, 10, 5))
+    checkEvaluation(Cast(Literal(LocalTime.NOON), DecimalType(11, 6)), Decimal(43200.000000, 11, 6))
+  }
+
+  test("SPARK-52620: cast time to decimal with insufficient scale") {
+    // Decimal time precision loss happens if the scale of the target the DecimalType is not
+    // sufficient to represent the fractional seconds of the TIME value. The following tests
+    // check that the cast operation correctly rounds the fractional seconds to the nearest
+    // decimal value that can be represented by the DecimalType, as limited by its scale.
+
+    // The following test cases will test DecimalType(1,0). Here, the scale is 0, meaning
+    // that the fractional seconds will be ignored, and rounded to the nearest whole second.
+    var precision = 1
+    var scale = 0
+    Seq(
+      // 00:00:00.1 -> 0 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 100000000), 0),
+      // 00:00:00.4 -> 0 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 400000000), 0),
+      // 00:00:00.49 -> 0 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 490000000), 0),
+      // 00:00:00.499999999 -> 0 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 499999999), 0),
+      // 00:00:00.5 -> 0 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 500000000), 1),
+      // 00:00:00.500000001 -> 1 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 500000001), 1),
+      // 00:00:00.599999999 -> 1 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 599999999), 1),
+      // 00:00:00.999999999 -> 1 (when scale is: 0)
+      (LocalTime.of(0, 0, 0, 999999999), 1)
+    ).foreach { case (timeValue, decimalValue) =>
+      checkEvaluation(
+        expression = Cast(Literal(timeValue), DecimalType(precision, scale)),
+        expected = Decimal(decimalValue, precision, scale)
+      )
+    }
+
+    // The following test cases will test DecimalType(2,1). Here, the scale is 1, meaning
+    // that the fractional seconds will be rounded to the nearest tenth of a second.
+    precision = 2
+    scale = 1
+    Seq(
+      // 00:00:00.1 -> 0.1 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 100000000), 0.1),
+      // 00:00:00.4 -> 0.4 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 400000000), 0.4),
+      // 00:00:00.49 -> 0.5 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 490000000), 0.5),
+      // 00:00:00.499999999 -> 0.5 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 499999999), 0.5),
+      // 00:00:00.5 -> 0.5 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 500000000), 0.5),
+      // 00:00:00.500000001 -> 0.5 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 500000001), 0.5),
+      // 00:00:00.599999999 -> 0.6 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 599999999), 0.6),
+      // 00:00:00.999999999 -> 1.0 (when scale is: 1)
+      (LocalTime.of(0, 0, 0, 999999999), 1.0)
+    ).foreach { case (timeValue, decimalValue) =>
+      checkEvaluation(
+        expression = Cast(Literal(timeValue), DecimalType(precision, scale)),
+        expected = Decimal(decimalValue, precision, scale)
+      )
+    }
+  }
+
+  test("SPARK-52619: cast time to integral types") {
+    // Test normal cases that should work with a small number like 112 seconds after midnight
+    val smallTime = Literal.create(LocalTime.of(0, 1, 52), TimeType(6))
+    checkEvaluation(cast(smallTime, ByteType), 112.toByte)
+    checkEvaluation(cast(smallTime, ShortType), 112.toShort)
+    checkEvaluation(cast(smallTime, IntegerType), 112)
+    checkEvaluation(cast(smallTime, LongType), 112L)
+
+    // Test midnight to all integral types
+    val midnight = Literal.create(LocalTime.MIDNIGHT, TimeType(6))
+    checkEvaluation(cast(midnight, ByteType), 0.toByte)
+    checkEvaluation(cast(midnight, ShortType), 0.toShort)
+    checkEvaluation(cast(midnight, IntegerType), 0)
+    checkEvaluation(cast(midnight, LongType), 0L)
+
+    // Precision rounding/truncation tests with fractional seconds
+    val time0 = Literal.create(LocalTime.NOON, TimeType(0))
+    val time2 = Literal.create(LocalTime.of(12, 0, 0, 120000000), TimeType(2))
+    val time4 = Literal.create(LocalTime.of(12, 0, 0, 345600000), TimeType(4))
+    val oneTwoThreeTime5 = Literal.create(LocalTime.of(1, 2, 3, 555550000), TimeType(5))
+    val maxTime4 = Literal.create(LocalTime.of(23, 59, 59, 999900000), TimeType(4))
+    val fractional5 = Literal.create(LocalTime.of(0, 0, 17, 500000000), TimeType(1))
+    val fractional000001 = Literal.create(LocalTime.of(0, 0, 17, 1000), TimeType(6))
+    val fractional999999 = Literal.create(LocalTime.of(0, 0, 17, 999999000), TimeType(6))
+    val fractional6 = Literal.create(LocalTime.of(0, 0, 17, 600000000), TimeType(1))
+    val fractional4 = Literal.create(LocalTime.of(0, 0, 17, 400000000), TimeType(1))
+    val fractional555 = Literal.create(LocalTime.of(0, 0, 17, 555000000), TimeType(3))
+    checkEvaluation(cast(fractional5, IntegerType), 17)
+    checkEvaluation(cast(fractional5, LongType), 17L)
+    checkEvaluation(cast(fractional000001, IntegerType), 17)
+    checkEvaluation(cast(fractional999999, IntegerType), 17)
+    checkEvaluation(cast(fractional6, IntegerType), 17)
+    checkEvaluation(cast(fractional4, IntegerType), 17)
+    checkEvaluation(cast(fractional555, IntegerType), 17)
+    checkEvaluation(cast(time0, IntegerType), 43200)
+    checkEvaluation(cast(time2, IntegerType), 43200)
+    checkEvaluation(cast(time4, IntegerType), 43200)
+    checkEvaluation(cast(oneTwoThreeTime5, IntegerType), 3723)
+    checkEvaluation(cast(oneTwoThreeTime5, LongType), 3723L)
+    checkEvaluation(cast(maxTime4, IntegerType), 86399)
+    checkEvaluation(cast(maxTime4, LongType), 86399L)
+  }
 }
