@@ -39,11 +39,11 @@ class ConvertToLocalRelationSuite extends PlanTest {
 
   test("Project on LocalRelation should be turned into a single LocalRelation") {
     val testRelation = LocalRelation(
-      LocalRelation('a.int, 'b.int).output,
+      LocalRelation($"a".int, $"b".int).output,
       InternalRow(1, 2) :: InternalRow(4, 5) :: Nil)
 
     val correctAnswer = LocalRelation(
-      LocalRelation('a1.int, 'b1.int).output,
+      LocalRelation($"a1".int, $"b1".int).output,
       InternalRow(1, 3) :: InternalRow(4, 6) :: Nil)
 
     val projectOnLocal = testRelation.select(
@@ -57,11 +57,11 @@ class ConvertToLocalRelationSuite extends PlanTest {
 
   test("Filter on LocalRelation should be turned into a single LocalRelation") {
     val testRelation = LocalRelation(
-      LocalRelation('a.int, 'b.int).output,
+      LocalRelation($"a".int, $"b".int).output,
       InternalRow(1, 2) :: InternalRow(4, 5) :: Nil)
 
     val correctAnswer = LocalRelation(
-      LocalRelation('a1.int, 'b1.int).output,
+      LocalRelation($"a1".int, $"b1".int).output,
       InternalRow(1, 3) :: Nil)
 
     val filterAndProjectOnLocal = testRelation
@@ -75,11 +75,11 @@ class ConvertToLocalRelationSuite extends PlanTest {
 
   test("SPARK-27798: Expression reusing output shouldn't override values in local relation") {
     val testRelation = LocalRelation(
-      LocalRelation('a.int).output,
+      LocalRelation($"a".int).output,
       InternalRow(1) :: InternalRow(2) :: Nil)
 
     val correctAnswer = LocalRelation(
-      LocalRelation('a.struct('a1.int)).output,
+      LocalRelation($"a".struct($"a1".int)).output,
       InternalRow(InternalRow(1)) :: InternalRow(InternalRow(2)) :: Nil)
 
     val projected = testRelation.select(ExprReuseOutput(UnresolvedAttribute("a")).as("a"))
