@@ -626,16 +626,6 @@ class Dataset[T] private[sql] (
   def transpose(): DataFrame =
     buildTranspose(Seq.empty)
 
-  /** @inheritdoc */
-  def scalar(): Column = {
-    Column(SubqueryExpressionNode(plan.getRoot, SubqueryType.SCALAR))
-  }
-
-  /** @inheritdoc */
-  def exists(): Column = {
-    Column(SubqueryExpressionNode(plan.getRoot, SubqueryType.EXISTS))
-  }
-
   /** @inheritdoc */
   def limit(n: Int): Dataset[T] = sparkSession.newDataset(agnosticEncoder) { builder =>
     builder.getLimitBuilder
@@ -1025,6 +1015,21 @@ class Dataset[T] private[sql] (
     }
   }
 
+  private def buildRepartitionById(numPartitions: Int, partitionExpr: Column): Dataset[T] = {
+    val exprBuilder = proto.Expression.newBuilder()
+    val directShufflePartitionIdExpr = exprBuilder
+      .setDirectShufflePartitionId(
+        exprBuilder.getDirectShufflePartitionIdBuilder
+          .setChild(toExpr(partitionExpr)))
+      .build()
+    sparkSession.newDataset(agnosticEncoder, partitionExpr :: Nil) { builder =>
+      val repartitionBuilder = builder.getRepartitionByExpressionBuilder
+        .setInput(plan.getRoot)
+        .addAllPartitionExprs(Seq(directShufflePartitionIdExpr).asJava)
+      repartitionBuilder.setNumPartitions(numPartitions)
+    }
+  }
+
   /** @inheritdoc */
   def repartition(numPartitions: Int): Dataset[T] = {
     buildRepartition(numPartitions, shuffle = true)
@@ -1056,6 +1061,11 @@ class Dataset[T] private[sql] (
     buildRepartitionByExpression(numPartitions, sortExprs)
   }
 
+  /** @inheritdoc */
+  def repartitionById(numPartitions: Int, partitionIdExpr: Column): Dataset[T] = {
+    buildRepartitionById(numPartitions, partitionIdExpr)
+  }
+
   /** @inheritdoc */
   def coalesce(numPartitions: Int): Dataset[T] = {
     buildRepartition(numPartitions, shuffle = false)
