@@ -869,7 +869,8 @@ private[columnar] object ColumnType {
       case ByteType => BYTE
       case ShortType => SHORT
       case IntegerType | DateType | _: YearMonthIntervalType => INT
-      case LongType | TimestampType | TimestampNTZType | _: DayTimeIntervalType => LONG
+      case LongType | TimestampType | TimestampNTZType | _: DayTimeIntervalType | _: TimeType =>
+        LONG
       case FloatType => FLOAT
       case DoubleType => DOUBLE
       case s: StringType => STRING(s)
