@@ -22,14 +22,14 @@ import scala.collection.mutable.ArrayBuffer
 
 import org.apache.spark.SparkException
 import org.apache.spark.api.python.PythonEvalType
-import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.Logging
 import org.apache.spark.internal.LogKeys.REASON
 import org.apache.spark.sql.catalyst.expressions._
+import org.apache.spark.sql.catalyst.expressions.PythonUDF.{correctEvalType, isScalarPythonUDF}
 import org.apache.spark.sql.catalyst.expressions.aggregate.AggregateExpression
 import org.apache.spark.sql.catalyst.plans.logical._
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.catalyst.trees.TreePattern._
-import org.apache.spark.sql.types._
 
 
 /**
@@ -48,8 +48,7 @@ object ExtractPythonUDFFromAggregate extends Rule[LogicalPlan] {
 
   private def hasPythonUdfOverAggregate(expr: Expression, agg: Aggregate): Boolean = {
     expr.exists {
-      e => PythonUDF.isScalarPythonUDF(e) &&
-        (e.references.isEmpty || e.exists(belongAggregate(_, agg)))
+      e => isScalarPythonUDF(e) && (e.references.isEmpty || e.exists(belongAggregate(_, agg)))
     }
   }
 
@@ -92,7 +91,7 @@ object ExtractPythonUDFFromAggregate extends Rule[LogicalPlan] {
  */
 object ExtractGroupingPythonUDFFromAggregate extends Rule[LogicalPlan] {
   private def hasScalarPythonUDF(e: Expression): Boolean = {
-    e.exists(PythonUDF.isScalarPythonUDF)
+    e.exists(isScalarPythonUDF)
   }
 
   private def extract(agg: Aggregate): LogicalPlan = {
@@ -166,28 +165,83 @@ object ExtractPythonUDFs extends Rule[LogicalPlan] with Logging {
   private type EvalTypeChecker = EvalType => Boolean
 
   private def hasScalarPythonUDF(e: Expression): Boolean = {
-    e.exists(PythonUDF.isScalarPythonUDF)
+    e.exists(isScalarPythonUDF)
   }
 
+  /**
+   * Return true if we should extract the current expression, including all of its current
+   * children (including UDF expression, and all others), to a logical node.
+   * The children of the expression can be UDF expressions, this would be nested chaining.
+   * If child UDF expressions were already extracted before, then this will just extract
+   * the current UDF expression, so they will end up in separate logical nodes. The child
+   * expressions will have been transformed to Attribute expressions referencing the child plan
+   * node's output.
+   *
+   * Return false if there is no single continuous chain of UDFs that can be extracted:
+   * - if there are other expression in-between, return false. In
+   *   below example, the caller will have to extract bar(baz()) separately first:
+   *   Query: foo(1 + bar(baz()))
+   *   Plan:
+   *   - PythonUDF (foo)
+   *      - Project
+   *         - PythonUDF (bar)
+   *           - PythonUDF (baz)
+   * - if the eval types of the UDF expressions in the chain differ, return false.
+   * - if a UDF has more than one child, e.g. foo(bar(), baz()), return false
+   * If we return false here, the expectation is that the recursive calls of
+   * collectEvaluableUDFsFromExpressions will then visit the children and extract them first to
+   * separate nodes.
+   */
   @scala.annotation.tailrec
-  private def canEvaluateInPython(e: PythonUDF): Boolean = {
+  private def shouldExtractUDFExpressionTree(
+      e: PythonUDF,
+      pythonUDFArrowFallbackOnUDT: Boolean): Boolean = {
     e.children match {
-      // single PythonUDF child could be chained and evaluated in Python
-      case Seq(u: PythonUDF) => correctEvalType(e) == correctEvalType(u) && canEvaluateInPython(u)
+      case Seq(child: PythonUDF) =>
+        correctEvalType(e, pythonUDFArrowFallbackOnUDT) ==
+          correctEvalType(child, pythonUDFArrowFallbackOnUDT) &&
+          shouldExtractUDFExpressionTree(child, pythonUDFArrowFallbackOnUDT)
       // Python UDF can't be evaluated directly in JVM
       case children => !children.exists(hasScalarPythonUDF)
     }
   }
 
-  private def collectEvaluableUDFsFromExpressions(expressions: Seq[Expression]): Seq[PythonUDF] = {
-    // If first UDF is SQL_SCALAR_PANDAS_ITER_UDF, then only return this UDF,
+  /**
+   * We use the following terminology:
+   * - chaining is the act of combining multiple UDFs into a single logical node. This can be
+   *   accomplished in different cases, for example:
+   *   - parallel chaining: if the UDFs are siblings, e.g., foo(x), bar(x),
+   *     where multiple independent UDFs are evaluated together over the same input
+   *   - nested chaining: if the UDFs are nested, e.g., foo(bar(...)),
+   *     where the output of one UDF feeds into the next in a sequential pipeline
+   *
+   * collectEvaluableUDFsFromExpressions returns a list of UDF expressions that can be planned
+   * together into one plan node. collectEvaluableUDFsFromExpressions will be called multiple times
+   * by recursive calls of extract(plan), until no more evaluable UDFs are found.
+   *
+   * As an example, consider the following expression tree:
+   * udf1(udf2(udf3(x)), udf4(x))), where all UDFs are PythonUDFs of the same evaltype.
+   * We can only fuse UDFs of the same eval type, and never UDFs of SQL_SCALAR_PANDAS_ITER_UDF.
+   * The following udf expressions will be returned:
+   * - First, we will return Seq(udf3, udf4), as these two UDFs must be evaluated first.
+   *   We return both in one Seq, as it is possible to do parallel fusing for udf3 an udf4.
+   * - As we can only chain UDFs with exactly one child, we will not fuse udf2 with its children.
+   *   But we can chain udf1 and udf2, so a later call to collectEvaluableUDFsFromExpressions will
+   *   return Seq(udf1, udf2).
+   */
+  private def collectEvaluableUDFsFromExpressions(
+      expressions: Seq[Expression],
+      pythonUDFArrowFallbackOnUDT: Boolean): Seq[PythonUDF] = {
+    // If first UDF is SQL_SCALAR_PANDAS_ITER_UDF or SQL_SCALAR_ARROW_ITER_UDF,
+    // then only return this UDF,
     // otherwise check if subsequent UDFs are of the same type as the first UDF. (since we can only
     // extract UDFs of the same eval type)
 
     var firstVisitedScalarUDFEvalType: Option[Int] = None
 
-    def canChainUDF(evalType: Int): Boolean = {
-      if (evalType == PythonEvalType.SQL_SCALAR_PANDAS_ITER_UDF) {
+    def canChainWithParallelUDFs(evalType: Int): Boolean = {
+      if (evalType == PythonEvalType.SQL_SCALAR_PANDAS_ITER_UDF ||
+        evalType == PythonEvalType.SQL_SCALAR_ARROW_ITER_UDF) {
         false
       } else {
         evalType == firstVisitedScalarUDFEvalType.get
@@ -195,12 +249,14 @@ object ExtractPythonUDFs extends Rule[LogicalPlan] with Logging {
     }
 
     def collectEvaluableUDFs(expr: Expression): Seq[PythonUDF] = expr match {
-      case udf: PythonUDF if PythonUDF.isScalarPythonUDF(udf) && canEvaluateInPython(udf)
+      case udf: PythonUDF if isScalarPythonUDF(udf)
+        && shouldExtractUDFExpressionTree(udf, pythonUDFArrowFallbackOnUDT)
         && firstVisitedScalarUDFEvalType.isEmpty =>
-        firstVisitedScalarUDFEvalType = Some(correctEvalType(udf))
+        firstVisitedScalarUDFEvalType = Some(correctEvalType(udf, pythonUDFArrowFallbackOnUDT))
         Seq(udf)
-      case udf: PythonUDF if PythonUDF.isScalarPythonUDF(udf) && canEvaluateInPython(udf)
-        && canChainUDF(correctEvalType(udf)) =>
+      case udf: PythonUDF if isScalarPythonUDF(udf)
+        && shouldExtractUDFExpressionTree(udf, pythonUDFArrowFallbackOnUDT)
+        && canChainWithParallelUDFs(correctEvalType(udf, pythonUDFArrowFallbackOnUDT)) =>
         Seq(udf)
       case e => e.children.flatMap(collectEvaluableUDFs)
     }
@@ -235,31 +291,14 @@ object ExtractPythonUDFs extends Rule[LogicalPlan] with Logging {
     }
   }
 
-  private def correctEvalType(udf: PythonUDF): Int = {
-    if (udf.evalType == PythonEvalType.SQL_ARROW_BATCHED_UDF) {
-      if (containsUDT(udf.dataType) || udf.children.exists(expr => containsUDT(expr.dataType))) {
-        PythonEvalType.SQL_BATCHED_UDF
-      } else {
-        PythonEvalType.SQL_ARROW_BATCHED_UDF
-      }
-    } else {
-      udf.evalType
-    }
-  }
-
-  private def containsUDT(dataType: DataType): Boolean = dataType match {
-    case _: UserDefinedType[_] => true
-    case ArrayType(elementType, _) => containsUDT(elementType)
-    case StructType(fields) => fields.exists(field => containsUDT(field.dataType))
-    case MapType(keyType, valueType, _) => containsUDT(keyType) || containsUDT(valueType)
-    case _ => false
-  }
-
   /**
    * Extract all the PythonUDFs from the current operator and evaluate them before the operator.
    */
   private def extract(plan: LogicalPlan): LogicalPlan = {
-    val udfs = ExpressionSet(collectEvaluableUDFsFromExpressions(plan.expressions))
+    val pythonUDFArrowFallbackOnUDT = conf.pythonUDFArrowFallbackOnUDT
+
+    val udfs = ExpressionSet(
+      collectEvaluableUDFsFromExpressions(plan.expressions, pythonUDFArrowFallbackOnUDT))
       // ignore the PythonUDF that come from second/third aggregate, which is not used
       .filter(udf => udf.references.subsetOf(plan.inputSet))
       .toSeq.asInstanceOf[Seq[PythonUDF]]
@@ -277,14 +316,14 @@ object ExtractPythonUDFs extends Rule[LogicalPlan] with Logging {
         }
         if (validUdfs.nonEmpty) {
           require(
-            validUdfs.forall(PythonUDF.isScalarPythonUDF),
+            validUdfs.forall(isScalarPythonUDF),
             "Can only extract scalar vectorized udf or sql batch udf")
 
           val resultAttrs = validUdfs.zipWithIndex.map { case (u, i) =>
             AttributeReference(s"pythonUDF$i", u.dataType)()
           }
 
-          val evalTypes = validUdfs.map(correctEvalType).toSet
+          val evalTypes = validUdfs.map(correctEvalType(_, pythonUDFArrowFallbackOnUDT)).toSet
           if (evalTypes.size != 1) {
             throw SparkException.internalError(
               "Expected udfs have the same evalType but got different evalTypes: " +
@@ -300,8 +339,11 @@ object ExtractPythonUDFs extends Rule[LogicalPlan] with Logging {
                   log"Falling back to non-Arrow-optimized UDF execution.")
               }
               BatchEvalPython(validUdfs, resultAttrs, child)
-            case PythonEvalType.SQL_SCALAR_PANDAS_UDF | PythonEvalType.SQL_SCALAR_PANDAS_ITER_UDF
-                 | PythonEvalType.SQL_ARROW_BATCHED_UDF =>
+            case PythonEvalType.SQL_SCALAR_PANDAS_UDF
+                 | PythonEvalType.SQL_SCALAR_PANDAS_ITER_UDF
+                 | PythonEvalType.SQL_ARROW_BATCHED_UDF
+                 | PythonEvalType.SQL_SCALAR_ARROW_UDF
+                 | PythonEvalType.SQL_SCALAR_ARROW_ITER_UDF =>
               ArrowEvalPython(validUdfs, resultAttrs, child, evalType)
             case _ =>
               throw SparkException.internalError("Unexpected UDF evalType")
@@ -351,6 +393,10 @@ object ExtractPythonUDTFs extends Rule[LogicalPlan] {
           BatchEvalPythonUDTF(func, g.requiredChildOutput, g.generatorOutput, child)
         case PythonEvalType.SQL_ARROW_TABLE_UDF =>
           ArrowEvalPythonUDTF(func, g.requiredChildOutput, g.generatorOutput, child, func.evalType)
+        case PythonEvalType.SQL_ARROW_UDTF =>
+          ArrowEvalPythonUDTF(func, g.requiredChildOutput, g.generatorOutput, child, func.evalType)
+        case _ =>
+          throw SparkException.internalError(s"Unsupported UDTF eval type: ${func.evalType}")
       }
     }
   }
