@@ -1,6 +1,6 @@
 -- create a table with properties
 CREATE TABLE tbl (a INT, b STRING, c INT) USING parquet
-TBLPROPERTIES('p1'='v1', 'p2'='v2');
+TBLPROPERTIES('p1'='v1', 'p2'='v2', password = 'password');
 
 SHOW TBLPROPERTIES tbl;
 SHOW TBLPROPERTIES tbl("p1");
@@ -17,8 +17,8 @@ SHOW TBLPROPERTIES view("p3");
 
 DROP VIEW view;
 
--- create a temporary view with properties
-CREATE TEMPORARY VIEW tv TBLPROPERTIES('p1'='v1') AS SELECT 1 AS c1;
+-- create a temporary view
+CREATE TEMPORARY VIEW tv AS SELECT 1 AS c1;
 
 -- Properties for a temporary view should be empty
 SHOW TBLPROPERTIES tv;
