@@ -4,6 +4,7 @@
 --CONFIG_DIM1 spark.sql.optimizeNullAwareAntiJoin=true
 --CONFIG_DIM1 spark.sql.optimizeNullAwareAntiJoin=false
 
+--ONLY_IF spark
 create temporary view t1 as select * from values
   ("val1a", 6S, 8, 10L, float(15.0), 20D, 20E2BD, timestamp '2014-04-04 01:00:00.000', date '2014-04-04'),
   ("val1b", 8S, 16, 19L, float(17.0), 25D, 26E2BD, timestamp '2014-05-04 01:01:00.000', date '2014-05-04'),
@@ -60,6 +61,57 @@ WHERE  t1a IN (SELECT t2a
                WHERE  t1d = t2d)
 LIMIT  2;
 
+-- correlated IN subquery
+-- LIMIT on both parent and subquery sides
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d
+               LIMIT 10)
+LIMIT  2;
+
+-- correlated IN subquery
+-- LIMIT with OFFSET in parent side
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d)
+LIMIT  2
+OFFSET 1;
+
+-- correlated IN subquery
+-- LIMIT with OFFSET on both parent and subquery sides
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d
+               LIMIT 10
+               OFFSET 2)
+LIMIT  2
+OFFSET 1;
+
+-- correlated IN subquery
+-- OFFSET in parent side
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d)
+OFFSET 1;
+
+-- correlated IN subquery
+-- OFFSET on both parent and subquery sides
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d
+               OFFSET 2)
+OFFSET 1;
+
 -- TC 01.02
 SELECT *
 FROM   t1
@@ -81,6 +133,14 @@ GROUP  BY t1b
 ORDER  BY t1b DESC NULLS FIRST
 LIMIT  1;
 
+SELECT *
+FROM   t1
+WHERE  t1c IN (SELECT t2c
+               FROM   t2
+               WHERE  t2b <= t1d
+               LIMIT  2)
+LIMIT 4;
+
 -- LIMIT with NOT IN
 -- TC 01.04
 SELECT *
@@ -102,6 +162,18 @@ GROUP  BY t1b
 ORDER BY t1b NULLS last
 LIMIT  1;
 
+SELECT Count(DISTINCT( t1a )),
+       t1b
+FROM   t1
+WHERE  t1d IN (SELECT t2d
+               FROM   t2
+               WHERE t2b <= t1d
+               ORDER  BY t2c, t2d
+               LIMIT 2)
+GROUP  BY t1b
+ORDER  BY t1b DESC NULLS FIRST
+LIMIT  1;
+
 -- LIMIT and OFFSET in parent side
 -- TC 02.01
 SELECT *
@@ -112,6 +184,16 @@ WHERE  t1a IN (SELECT t2a
 LIMIT  2
 OFFSET 2;
 
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d > t2d
+               ORDER BY t2a DESC
+               LIMIT 3)
+LIMIT  2
+OFFSET 2;
+
 -- TC 02.02
 SELECT *
 FROM   t1
@@ -146,6 +228,21 @@ WHERE  t1b NOT IN (SELECT t2b
                    LIMIT  2
                    OFFSET 2);
 
+SELECT *
+FROM   t1
+WHERE  t1b NOT IN (SELECT t2b
+                   FROM   t2
+                   WHERE  t2b <= t1d
+                   LIMIT  2);
+
+SELECT *
+FROM   t1
+WHERE  t1b NOT IN (SELECT t2b
+                   FROM   t2
+                   WHERE  t2b = t1b
+                   LIMIT  2
+                   OFFSET 2);
+
 -- TC 02.05
 SELECT Count(DISTINCT( t1a )),
        t1b
@@ -160,6 +257,20 @@ ORDER BY t1b NULLS last
 LIMIT  1
 OFFSET 1;
 
+SELECT Count(DISTINCT( t1a )),
+       t1b
+FROM   t1
+WHERE  t1d NOT IN (SELECT t2d
+                   FROM   t2
+                   WHERE t2b > t1b
+                   ORDER  BY t2b DESC nulls first, t2d
+                   LIMIT 1
+                   OFFSET 1)
+GROUP  BY t1b
+ORDER BY t1b NULLS last
+LIMIT  1
+OFFSET 1;
+
 -- OFFSET in parent side
 -- TC 03.01
 SELECT *
@@ -169,6 +280,16 @@ WHERE  t1a IN (SELECT t2a
                WHERE  t1d = t2d)
 OFFSET 2;
 
+-- OFFSET in correlated subquery
+SELECT *
+FROM   t1
+WHERE  t1a IN (SELECT t2a
+               FROM   t2
+               WHERE  t1d = t2d
+               ORDER BY t2a
+               OFFSET 2)
+OFFSET 2;
+
 -- TC 03.02
 SELECT *
 FROM   t1
@@ -178,6 +299,15 @@ WHERE  t1c IN (SELECT t2c
                OFFSET 2)
 OFFSET 4;
 
+SELECT *
+FROM   t1
+WHERE  t1c IN (SELECT t2c
+               FROM   t2
+               WHERE  t2b < t1b
+               ORDER BY t2c
+               OFFSET 2)
+OFFSET 1;
+
 -- TC 03.03
 SELECT Count(DISTINCT( t1a )),
        t1b
@@ -199,6 +329,14 @@ WHERE  t1b NOT IN (SELECT t2b
                    WHERE  t2b > 6
                    OFFSET 2);
 
+-- OFFSET with NOT IN correlated
+SELECT count(*)
+FROM   t1
+WHERE  t1b NOT IN (SELECT t2b
+                   FROM   t2
+                   WHERE  t2b < t1b
+                   OFFSET 2);
+
 -- TC 03.05
 SELECT Count(DISTINCT( t1a )),
        t1b
@@ -210,3 +348,37 @@ WHERE  t1d NOT IN (SELECT t2d
 GROUP  BY t1b
 ORDER BY t1b NULLS last
 OFFSET 1;
+
+-- SPARK-46526: LIMIT over correlated predicate that references only the outer table.
+SELECT COUNT(DISTINCT(t1a))
+FROM t1
+WHERE t1d IN (SELECT t2d
+              FROM   t2
+              WHERE t1a IS NOT NULL
+              LIMIT 10);
+
+SELECT COUNT(DISTINCT(t1a))
+FROM t1
+WHERE t1d IN (SELECT MAX(t2d)
+              FROM   t2
+              WHERE t1a IS NOT NULL
+              LIMIT 10);
+
+SELECT COUNT(DISTINCT(t1a))
+FROM t1
+WHERE t1d IN (SELECT DISTINCT t2d
+              FROM   t2
+              WHERE t1a IS NOT NULL
+              LIMIT 10);
+
+set spark.sql.optimizer.decorrelateExistsIn.enabled = false;
+-- LIMIT is not supported in correlated IN, unless the DECORRELATE_EXISTS_AND_IN_SUBQUERIES
+-- is enabled.
+SELECT *
+FROM   t1
+WHERE  t1b NOT IN (SELECT t2b
+                   FROM   t2
+                   WHERE  t2b = t1b
+                   LIMIT  2
+                   OFFSET 2);
+set spark.sql.optimizer.decorrelateExistsIn.enabled = true;
