@@ -596,8 +596,9 @@ class DataFrameTimeWindowingSuite extends QueryTest with SharedSparkSession {
       df <- Seq(df1, df2)
       nullable <- Seq(true, false)
     } {
-      val dfWithDesiredNullability = new DataFrame(df.queryExecution, ExpressionEncoder(
-        StructType(df.schema.fields.map(_.copy(nullable = nullable)))))
+      val dfWithDesiredNullability = new classic.DataFrame(
+        df.queryExecution,
+        ExpressionEncoder(StructType(df.schema.fields.map(_.copy(nullable = nullable)))))
       // tumbling windows
       val windowedProject = dfWithDesiredNullability
         .select(window($"time", "10 seconds").as("window"), $"value")
