@@ -46,14 +46,14 @@ object InExpressionBenchmark extends SqlBasedBenchmark {
   private def runByteBenchmark(numItems: Int, numRows: Long, minNumIters: Int): Unit = {
     val name = s"$numItems bytes"
     val values = (Byte.MinValue until Byte.MinValue + numItems).map(v => s"${v}Y")
-    val df = spark.range(0, numRows).select($"id".cast(ByteType))
+    val df = spark.range(0, numRows).selectExpr("mod(id, 256) as id")
     runBenchmark(name, df, values, numRows, minNumIters)
   }
 
   private def runShortBenchmark(numItems: Int, numRows: Long, minNumIters: Int): Unit = {
     val name = s"$numItems shorts"
     val values = (1 to numItems).map(v => s"${v}S")
-    val df = spark.range(0, numRows).select($"id".cast(ShortType))
+    val df = spark.range(0, numRows).selectExpr("mod(id, 32768) as id")
     runBenchmark(name, df, values, numRows, minNumIters)
   }
 
@@ -64,14 +64,14 @@ object InExpressionBenchmark extends SqlBasedBenchmark {
     require(isLookupSwitch(rangeSize, numItems))
     val name = s"$numItems shorts (non-compact)"
     val values = (Short.MinValue until maxValue by step).map(v => s"${v}S")
-    val df = spark.range(0, numRows).select($"id".cast(ShortType))
+    val df = spark.range(0, numRows).selectExpr("mod(id, 32768) as id")
     runBenchmark(name, df, values, numRows, minNumIters)
   }
 
   private def runIntBenchmark(numItems: Int, numRows: Long, minNumIters: Int): Unit = {
     val name = s"$numItems ints"
     val values = 1 to numItems
-    val df = spark.range(0, numRows).select($"id".cast(IntegerType))
+    val df = spark.range(0, numRows).selectExpr("mod(id, 2147483648) as id")
     runBenchmark(name, df, values, numRows, minNumIters)
   }
 
@@ -82,7 +82,7 @@ object InExpressionBenchmark extends SqlBasedBenchmark {
     require(isLookupSwitch(rangeSize, numItems))
     val name = s"$numItems ints (non-compact)"
     val values = Int.MinValue until maxValue.toInt by step.toInt
-    val df = spark.range(0, numRows).select($"id".cast(IntegerType))
+    val df = spark.range(0, numRows).selectExpr("mod(id, 2147483648) as id")
     runBenchmark(name, df, values, numRows, minNumIters)
   }
 
