@@ -18,7 +18,7 @@
 package org.apache.spark.sql.execution.datasources.noop
 
 import org.apache.spark.sql.classic.DataFrame
-import org.apache.spark.sql.execution.streaming.MemoryStream
+import org.apache.spark.sql.execution.streaming.runtime.MemoryStream
 import org.apache.spark.sql.streaming.{StreamingQuery, StreamTest, Trigger}
 
 class NoopStreamSuite extends StreamTest {
