@@ -17,6 +17,7 @@
 
 package org.apache.spark.sql.execution.datasources.orc
 
+import java.lang.invoke.MethodHandles
 import java.util.{Map => JMap}
 import java.util.Random
 
@@ -58,9 +59,9 @@ class OrcEncryptionSuite extends OrcTest with SharedSparkSession {
 
   private val keyProviderCacheRef: JMap[String, KeyProvider] = {
     val clazz = classOf[CryptoUtils]
-    val field = clazz.getDeclaredField("keyProviderCache")
-    field.setAccessible(true)
-    field.get(null).asInstanceOf[JMap[String, KeyProvider]]
+    val lookup = MethodHandles.privateLookupIn(clazz, MethodHandles.lookup())
+    lookup.findStaticVarHandle(clazz, "keyProviderCache", classOf[JMap[_, _]])
+      .get().asInstanceOf[JMap[String, KeyProvider]]
   }
 
   test("Write and read an encrypted file") {
