@@ -23,13 +23,14 @@ import java.util.UUID
 import org.apache.hadoop.conf.Configuration
 import org.scalatest.Assertions
 
-import org.apache.spark.{SparkException, SparkUnsupportedOperationException}
+import org.apache.spark.{SparkException, SparkThrowable, SparkUnsupportedOperationException}
 import org.apache.spark.io.CompressionCodec
 import org.apache.spark.sql.{AnalysisException, DataFrame, Encoders, Row}
 import org.apache.spark.sql.catalyst.expressions.{BoundReference, GenericInternalRow}
 import org.apache.spark.sql.catalyst.plans.physical.HashPartitioning
 import org.apache.spark.sql.execution.datasources.v2.state.utils.SchemaUtil
-import org.apache.spark.sql.execution.streaming.{CommitLog, MemoryStream, OffsetSeqLog, StreamExecution}
+import org.apache.spark.sql.execution.streaming.checkpointing.{CommitLog, OffsetSeqLog}
+import org.apache.spark.sql.execution.streaming.runtime.{MemoryStream, StreamExecution}
 import org.apache.spark.sql.execution.streaming.state._
 import org.apache.spark.sql.functions.col
 import org.apache.spark.sql.internal.SQLConf
@@ -568,6 +569,114 @@ StateDataSourceReadSuite {
     testSnapshotOnJoinState("rocksdb", 1)
     testSnapshotOnJoinState("rocksdb", 2)
   }
+
+  /**
+   * Note that we cannot use the golden files approach for transformWithState. The new schema
+   * format keeps track of the schema file path as an absolute path which cannot be used with
+   * the getResource model used in other similar tests on runbot.
+   */
+  test("snapshotStartBatchId on join state v3") {
+    testSnapshotOnJoinStateV3()
+  }
+}
+
+class RocksDBWithCheckpointV2StateDataSourceReaderSuite extends StateDataSourceReadSuite {
+  override protected def newStateStoreProvider(): RocksDBStateStoreProvider =
+    new RocksDBStateStoreProvider
+
+  import testImplicits._
+
+  override def beforeAll(): Unit = {
+    super.beforeAll()
+    spark.conf.set(SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION, 2)
+    spark.conf.set(SQLConf.STATE_STORE_PROVIDER_CLASS.key,
+      newStateStoreProvider().getClass.getName)
+    spark.conf.set("spark.sql.streaming.stateStore.rocksdb.changelogCheckpointing.enabled",
+      "true")
+  }
+
+  // TODO: Remove this test once we allow migrations from checkpoint v1 to v2
+  test("reading checkpoint v2 store with version 1 should fail") {
+    withTempDir { tmpDir =>
+      val inputData = MemoryStream[(Int, Long)]
+      val query = getStreamStreamJoinQuery(inputData)
+      testStream(query)(
+        StartStream(checkpointLocation = tmpDir.getCanonicalPath),
+        AddData(inputData, (1, 1L), (2, 2L), (3, 3L), (4, 4L), (5, 5L)),
+        ProcessAllAvailable(),
+        Execute { _ => Thread.sleep(2000) },
+        StopStream
+      )
+
+      withSQLConf(SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION.key -> "1") {
+        // Verify reading state throws error when reading checkpoint v2 with version 1
+        val exc = intercept[IllegalStateException] {
+          val stateDf = spark.read.format("statestore")
+            .option(StateSourceOptions.BATCH_ID, 0)
+            .option(StateSourceOptions.OPERATOR_ID, 0)
+            .load(tmpDir.getCanonicalPath)
+          stateDf.collect()
+        }
+
+        checkError(exc.getCause.asInstanceOf[SparkThrowable],
+          "INVALID_LOG_VERSION.EXACT_MATCH_VERSION", "KD002",
+          Map(
+            "version" -> "2",
+            "matchVersion" -> "1"))
+      }
+    }
+  }
+}
+
+class RocksDBWithCheckpointV2StateDataSourceReaderSnapshotSuite extends StateDataSourceReadSuite {
+  override protected def newStateStoreProvider(): RocksDBStateStoreProvider =
+    new RocksDBStateStoreProvider
+
+  override def beforeAll(): Unit = {
+    super.beforeAll()
+    spark.conf.set(SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION, 2)
+    spark.conf.set(SQLConf.STATE_STORE_PROVIDER_CLASS.key,
+      newStateStoreProvider().getClass.getName)
+    spark.conf.set("spark.sql.streaming.stateStore.rocksdb.changelogCheckpointing.enabled",
+      "true")
+    // make sure we have a snapshot for every two delta files
+    // HDFS maintenance task will not count the latest delta file, which has the same version
+    // as the snapshot version
+    spark.conf.set(SQLConf.STATE_STORE_MIN_DELTAS_FOR_SNAPSHOT.key, 2)
+  }
+
+  test("ERROR: snapshot of version not found") {
+    testSnapshotNotFound()
+  }
+
+  test("provider.getReadStore(snapshotVersion, endVersion)") {
+    testGetReadStoreWithStartVersion()
+  }
+
+  test("option snapshotPartitionId") {
+    testSnapshotPartitionId()
+  }
+
+  test("snapshotStartBatchId on limit state") {
+    testSnapshotOnLimitState("rocksdb", checkpointFormatVersion = 2)
+  }
+
+  test("snapshotStartBatchId on aggregation state") {
+    testSnapshotOnAggregateState("rocksdb", checkpointFormatVersion = 2)
+  }
+
+  test("snapshotStartBatchId on deduplication state") {
+    testSnapshotOnDeduplicateState("rocksdb", checkpointFormatVersion = 2)
+  }
+
+  test("snapshotStartBatchId on join state") {
+    testSnapshotOnJoinState("rocksdb", 1, checkpointFormatVersion = 2)
+    testSnapshotOnJoinState("rocksdb", 2, checkpointFormatVersion = 2)
+  }
+
+  test("snapshotStartBatchId on join state v3") {
+    testSnapshotOnJoinStateV3()
+  }
 }
 
 abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Assertions {
@@ -869,6 +978,10 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
     testStreamStreamJoin(2)
   }
 
+  test("stream-stream join, state ver 3") {
+    testStreamStreamJoin(3)
+  }
+
   private def testStreamStreamJoin(stateVersion: Int): Unit = {
     def assertInternalColumnIsNotExposed(df: DataFrame): Unit = {
       val valueSchema = SchemaUtil.getSchemaAsDataType(df.schema, "value")
@@ -879,6 +992,12 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
       }
     }
 
+    // We should only test state version 3 with RocksDBStateStoreProvider
+    if (stateVersion == 3
+      && SQLConf.get.stateStoreProviderClass != classOf[RocksDBStateStoreProvider].getName) {
+      return
+    }
+
     withSQLConf(SQLConf.STREAMING_JOIN_STATE_FORMAT_VERSION.key -> stateVersion.toString) {
       withTempDir { tempDir =>
         runStreamStreamJoinQuery(tempDir.getAbsolutePath)
@@ -939,7 +1058,7 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
 
         val stateReadDfForRightKeyWithIndexToValue = stateReaderForRightKeyWithIndexToValue.load()
 
-        if (stateVersion == 2) {
+        if (stateVersion >= 2) {
           val resultDf4 = stateReadDfForRightKeyWithIndexToValue
             .selectExpr("key.field0 AS key_0", "key.index AS key_index",
               "value.rightId AS rightId", "CAST(value.rightTime AS integer) AS rightTime",
@@ -1094,22 +1213,29 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
 
   protected def testGetReadStoreWithStartVersion(): Unit = {
     withTempDir { tempDir =>
+      val versionToCkptId = scala.collection.mutable.Map[Long, Option[String]]()
       val provider = getNewStateStoreProvider(tempDir.getAbsolutePath)
       for (i <- 1 to 4) {
-        val store = provider.getStore(i - 1)
+        val store = provider.getStore(i - 1, versionToCkptId.getOrElse(i - 1, None))
         put(store, "a", i, i)
         store.commit()
+
+        val ssInfo = store.getStateStoreCheckpointInfo()
+        versionToCkptId(ssInfo.batchVersion) = ssInfo.stateStoreCkptId
+
         provider.doMaintenance()
       }
 
       val result =
         provider.asInstanceOf[SupportsFineGrainedReplay]
-          .replayReadStateFromSnapshot(2, 3)
+          .replayReadStateFromSnapshot(2, 3,
+            versionToCkptId.getOrElse(2, None), versionToCkptId.getOrElse(3, None))
 
       assert(get(result, "a", 1).get == 1)
       assert(get(result, "a", 2).get == 2)
       assert(get(result, "a", 3).get == 3)
       assert(get(result, "a", 4).isEmpty)
+      result.release()
 
       provider.close()
     }
@@ -1163,10 +1289,13 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
     checkAnswer(stateSnapshotDf, stateDf)
   }
 
-  protected def testSnapshotOnLimitState(providerName: String): Unit = {
+  protected def testSnapshotOnLimitState(
+      providerName: String,
+      checkpointFormatVersion: Int = 1): Unit = {
     /** The golden files are generated by:
       withSQLConf({
         SQLConf.STREAMING_MAINTENANCE_INTERVAL.key -> "100"
+        SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION.key -> checkpointFormatVersion.toString
       }) {
         val inputData = MemoryStream[(Int, Long)]
         val query = inputData.toDF().limit(10)
@@ -1187,17 +1316,27 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
         )
       }
      */
+
+    val versionSuffix = if (checkpointFormatVersion == 2) {
+      "-checkpoint-v2"
+    } else {
+      ""
+    }
+
     val resourceUri = this.getClass.getResource(
-      s"/structured-streaming/checkpoint-version-4.0.0/$providerName/limit/"
+      s"/structured-streaming/checkpoint-version-4.0.0$versionSuffix/$providerName/limit/"
     ).toURI
 
     testSnapshotStateDfAgainstStateDf(new File(resourceUri))
   }
 
-  protected def testSnapshotOnAggregateState(providerName: String): Unit = {
+  protected def testSnapshotOnAggregateState(
+      providerName: String,
+      checkpointFormatVersion: Int = 1): Unit = {
     /** The golden files are generated by:
       withSQLConf({
         SQLConf.STREAMING_MAINTENANCE_INTERVAL.key -> "100"
+        SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION.key -> checkpointFormatVersion.toString
       }) {
         val inputData = MemoryStream[(Int, Long)]
         val query = inputData.toDF().groupBy("_1").count()
@@ -1218,17 +1357,26 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
         )
       }
      */
+    val versionSuffix = if (checkpointFormatVersion == 2) {
+      "-checkpoint-v2"
+    } else {
+      ""
+    }
+
     val resourceUri = this.getClass.getResource(
-      s"/structured-streaming/checkpoint-version-4.0.0/$providerName/dedup/"
+      s"/structured-streaming/checkpoint-version-4.0.0$versionSuffix/$providerName/dedup/"
     ).toURI
 
     testSnapshotStateDfAgainstStateDf(new File(resourceUri))
   }
 
-  protected def testSnapshotOnDeduplicateState(providerName: String): Unit = {
+  protected def testSnapshotOnDeduplicateState(
+      providerName: String,
+      checkpointFormatVersion: Int = 1): Unit = {
     /** The golden files are generated by:
       withSQLConf({
         SQLConf.STREAMING_MAINTENANCE_INTERVAL.key -> "100"
+        SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION.key -> checkpointFormatVersion.toString
       }) {
         val inputData = MemoryStream[(Int, Long)]
         val query = inputData.toDF().dropDuplicates("_1")
@@ -1249,18 +1397,28 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
         )
       }
      */
+    val versionSuffix = if (checkpointFormatVersion == 2) {
+      "-checkpoint-v2"
+    } else {
+      ""
+    }
+
     val resourceUri = this.getClass.getResource(
-      s"/structured-streaming/checkpoint-version-4.0.0/$providerName/dedup/"
+      s"/structured-streaming/checkpoint-version-4.0.0$versionSuffix/$providerName/dedup/"
     ).toURI
 
     testSnapshotStateDfAgainstStateDf(new File(resourceUri))
   }
 
-  protected def testSnapshotOnJoinState(providerName: String, stateVersion: Int): Unit = {
+  protected def testSnapshotOnJoinState(
+      providerName: String,
+      stateVersion: Int,
+      checkpointFormatVersion: Int = 1): Unit = {
     /** The golden files are generated by:
       withSQLConf({
         SQLConf.STREAMING_JOIN_STATE_FORMAT_VERSION.key -> stateVersion.toString
         SQLConf.STREAMING_MAINTENANCE_INTERVAL.key -> "100"
+        SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION.key -> checkpointFormatVersion.toString
       }) {
         val inputData = MemoryStream[(Int, Long)]
         val query = getStreamStreamJoinQuery(inputData)
@@ -1278,8 +1436,15 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
         )
       }
      */
+    val versionSuffix = if (checkpointFormatVersion == 2) {
+      "-checkpoint-v2"
+    } else {
+      ""
+    }
+
     val resourceUri = this.getClass.getResource(
-      s"/structured-streaming/checkpoint-version-4.0.0/$providerName/join$stateVersion/"
+      s"/structured-streaming/checkpoint-version-4.0.0$versionSuffix/" +
+        s"$providerName/join$stateVersion/"
     ).toURI
 
     val resourceDir = new File(resourceUri)
@@ -1297,4 +1462,42 @@ abstract class StateDataSourceReadSuite extends StateDataSourceTestBase with Ass
 
     checkAnswer(stateSnapshotDf, stateDf)
   }
+
+  protected def testSnapshotOnJoinStateV3(): Unit = {
+    withTempDir { tmpDir =>
+      withSQLConf(
+        SQLConf.STREAMING_JOIN_STATE_FORMAT_VERSION.key -> "3",
+        SQLConf.STREAMING_MAINTENANCE_INTERVAL.key -> "100"
+      ) {
+        val inputData = MemoryStream[(Int, Long)]
+        val query = getStreamStreamJoinQuery(inputData)
+        testStream(query)(
+          StartStream(checkpointLocation = tmpDir.getCanonicalPath),
+          AddData(inputData, (1, 1L), (2, 2L), (3, 3L), (4, 4L), (5, 5L)),
+          ProcessAllAvailable(),
+          Execute { _ => Thread.sleep(2000) },
+          AddData(inputData, (6, 6L), (7, 7L), (8, 8L), (9, 9L), (10, 10L)),
+          ProcessAllAvailable(),
+          Execute { _ => Thread.sleep(2000) },
+          AddData(inputData, (11, 11L), (12, 12L), (13, 13L), (14, 14L), (15, 15L)),
+          ProcessAllAvailable(),
+          Execute { _ => Thread.sleep(5000) },
+          StopStream
+        )
+
+        val stateSnapshotDf = spark.read.format("statestore")
+          .option("snapshotPartitionId", 2)
+          .option("snapshotStartBatchId", 0)
+          .option("joinSide", "left")
+          .load(tmpDir.getCanonicalPath)
+
+        val stateDf = spark.read.format("statestore")
+          .option("joinSide", "left")
+          .load(tmpDir.getCanonicalPath)
+          .filter(col("partition_id") === 2)
+
+        checkAnswer(stateSnapshotDf, stateDf)
+      }
+    }
+  }
 }
