@@ -17,12 +17,13 @@
 
 package org.apache.spark.sql.execution.joins
 
-import org.apache.spark.sql.{DataFrame, Row}
-import org.apache.spark.sql.catalyst.expressions.{And, Expression, LessThan}
+import org.apache.spark.sql.Row
+import org.apache.spark.sql.catalyst.expressions.{And, EqualTo, Expression, LessThan}
 import org.apache.spark.sql.catalyst.optimizer.{BuildLeft, BuildRight}
 import org.apache.spark.sql.catalyst.planning.ExtractEquiJoinKeys
 import org.apache.spark.sql.catalyst.plans._
 import org.apache.spark.sql.catalyst.plans.logical.{Join, JoinHint}
+import org.apache.spark.sql.classic.DataFrame
 import org.apache.spark.sql.execution.{SparkPlan, SparkPlanTest}
 import org.apache.spark.sql.execution.exchange.EnsureRequirements
 import org.apache.spark.sql.internal.SQLConf
@@ -30,6 +31,7 @@ import org.apache.spark.sql.test.{SharedSparkSession, SQLTestData}
 import org.apache.spark.sql.types.{DoubleType, IntegerType, StructType}
 
 class OuterJoinSuite extends SparkPlanTest with SharedSparkSession with SQLTestData {
+  import testImplicits.toRichColumn
   setupTestData()
 
   private val EnsureRequirements = new EnsureRequirements()
@@ -61,7 +63,7 @@ class OuterJoinSuite extends SparkPlanTest with SharedSparkSession with SQLTestD
     )), new StructType().add("c", IntegerType).add("d", DoubleType))
 
   private lazy val condition = {
-    And((left.col("a") === right.col("c")).expr,
+    And(EqualTo(left.col("a").expr, right.col("c").expr),
       LessThan(left.col("b").expr, right.col("d").expr))
   }
 
@@ -87,7 +89,7 @@ class OuterJoinSuite extends SparkPlanTest with SharedSparkSession with SQLTestD
     )), new StructType().add("c", IntegerType).add("d", DoubleType))
 
   private lazy val uniqueCondition = {
-    And((uniqueLeft.col("a") === uniqueRight.col("c")).expr,
+    And(EqualTo(uniqueLeft.col("a").expr, uniqueRight.col("c").expr),
       LessThan(uniqueLeft.col("b").expr, uniqueRight.col("d").expr))
   }
 
