@@ -24,10 +24,11 @@ import scala.util.Random
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.fs._
 
-import org.apache.spark.SparkFunSuite
+import org.apache.spark.{SparkException, SparkFunSuite}
 import org.apache.spark.sql.catalyst.plans.SQLHelper
 import org.apache.spark.sql.catalyst.util.quietly
-import org.apache.spark.sql.execution.streaming.CheckpointFileManager.CancellableFSDataOutputStream
+import org.apache.spark.sql.execution.streaming.checkpointing.{CheckpointFileManager, FileContextBasedCheckpointFileManager, FileSystemBasedCheckpointFileManager, HDFSMetadataLog}
+import org.apache.spark.sql.execution.streaming.checkpointing.CheckpointFileManager.CancellableFSDataOutputStream
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.test.SharedSparkSession
 
@@ -39,14 +40,14 @@ abstract class CheckpointFileManagerTests extends SparkFunSuite {
 
   protected def createManager(path: Path): CheckpointFileManager
 
-  private implicit class RichCancellableStream(stream: CancellableFSDataOutputStream) {
+  protected implicit class RichCancellableStream(stream: CancellableFSDataOutputStream) {
     def writeContent(i: Int): CancellableFSDataOutputStream = {
       stream.writeInt(i)
       stream
     }
   }
 
-  private implicit class RichFSDataInputStream(stream: FSDataInputStream) {
+  protected implicit class RichFSDataInputStream(stream: FSDataInputStream) {
     def readContent(): Int = {
       val res = stream.readInt()
       stream.close()
@@ -111,6 +112,7 @@ abstract class CheckpointFileManagerTests extends SparkFunSuite {
         fm.open(path)
       }
       fm.delete(path) // should not throw exception
+      fm.close()
     }
   }
 }
@@ -145,6 +147,56 @@ class CheckpointFileManagerSuite extends SharedSparkSession {
       }
     }
   }
+
+  test("SPARK-52824: CheckpointFileManager.create() does not throw InvocationTargetException") {
+    withSQLConf(
+      SQLConf.STREAMING_CHECKPOINT_FILE_MANAGER_CLASS.parent.key ->
+        classOf[ConstructorFailureTestManager].getName) {
+      val ex = intercept[SparkException] {
+        CheckpointFileManager.create(new Path("/"), spark.sessionState.newHadoopConf())
+      }
+      checkError(
+        ex,
+        condition = "CANNOT_LOAD_CHECKPOINT_FILE_MANAGER.ERROR_LOADING_CLASS",
+        parameters = Map(
+          "path" -> "/",
+          "className" -> classOf[ConstructorFailureTestManager].getName,
+          "msg" -> "java.lang.IllegalStateException: error")
+      )
+    }
+  }
+
+  test("SPARK-52824: CheckpointFileManager.create() throws uncategorized error") {
+    val hadoopConf = spark.sessionState.newHadoopConf()
+    // Set invalid fs.defaultFS to trigger uncategorized error from URI.create
+    hadoopConf.set("fs.defaultFS", "|invalid/")
+    val ex = intercept[SparkException] {
+      CheckpointFileManager.create(new Path("/"), hadoopConf)
+    }
+    checkError(
+      ex,
+      condition = "CANNOT_LOAD_CHECKPOINT_FILE_MANAGER.UNCATEGORIZED",
+      parameters = Map("path" -> "/")
+    )
+  }
+
+  test("SPARK-52824: CheckpointFileManager.create() throws error when class cannot be found") {
+    withSQLConf(
+      SQLConf.STREAMING_CHECKPOINT_FILE_MANAGER_CLASS.parent.key ->
+        "notarealclass") {
+      val ex = intercept[SparkException] {
+        CheckpointFileManager.create(new Path("/"), spark.sessionState.newHadoopConf())
+      }
+      checkError(
+        ex,
+        condition = "CANNOT_LOAD_CHECKPOINT_FILE_MANAGER.ERROR_LOADING_CLASS",
+        parameters = Map(
+          "path" -> "/",
+          "className" -> "notarealclass",
+          "msg" -> "java.lang.ClassNotFoundException: notarealclass")
+      )
+    }
+  }
 }
 
 abstract class CheckpointFileManagerTestsOnLocalFs
@@ -222,6 +274,11 @@ object CreateAtomicTestManager {
   @volatile var cancelCalledInCreateAtomic = false
 }
 
+/** A fake implementation to test constructor failure */
+class ConstructorFailureTestManager(path: Path, hadoopConf: Configuration)
+  extends FileSystemBasedCheckpointFileManager(path, hadoopConf) {
+  throw new IllegalStateException("error")
+}
 
 /**
  * CheckpointFileManagerSuiteFileSystem to test fallback of the CheckpointFileManager
