@@ -22,8 +22,9 @@ import org.apache.hadoop.fs.Path
 import org.apache.spark.SparkRuntimeException
 import org.apache.spark.sql.{Column, Row}
 import org.apache.spark.sql.execution.datasources.v2.state.{StateDataSourceUnspecifiedRequiredOption, StateSourceOptions}
-import org.apache.spark.sql.execution.streaming.{CheckpointFileManager, LongOffset, MemoryStream, OffsetSeq, OffsetSeqLog}
-import org.apache.spark.sql.execution.streaming.StreamingCheckpointConstants.DIR_NAME_OFFSETS
+import org.apache.spark.sql.execution.streaming.checkpointing.{CheckpointFileManager, OffsetSeq, OffsetSeqLog}
+import org.apache.spark.sql.execution.streaming.runtime.{LongOffset, MemoryStream}
+import org.apache.spark.sql.execution.streaming.runtime.StreamingCheckpointConstants.DIR_NAME_OFFSETS
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.streaming.{OutputMode, RunningCountStatefulProcessor, StreamTest, TimeMode}
@@ -229,6 +230,86 @@ class OperatorStateMetadataSuite extends StreamTest with SharedSparkSession {
     }
   }
 
+  test(
+    "SPARK-51779 Stateful operator metadata v2 for streaming join with virtual column families") {
+    withTempDir { checkpointDir =>
+      withSQLConf(
+        SQLConf.STATE_STORE_PROVIDER_CLASS.key -> classOf[RocksDBStateStoreProvider].getName,
+        SQLConf.STREAMING_JOIN_STATE_FORMAT_VERSION.key -> "3"
+      ) {
+        val input1 = MemoryStream[Int]
+        val input2 = MemoryStream[Int]
+
+        val df1 = input1.toDF().select($"value" as "key", ($"value" * 2) as "leftValue")
+        val df2 = input2.toDF().select($"value" as "key", ($"value" * 3) as "rightValue")
+        val joined = df1.join(df2, "key")
+
+        testStream(joined)(
+          StartStream(checkpointLocation = checkpointDir.getCanonicalPath),
+          AddData(input1, 1),
+          CheckAnswer(),
+          AddData(input2, 1, 10),
+          CheckNewAnswer((1, 2, 3)),
+          StopStream
+        )
+
+        val expectedStateStoreInfo =
+          Array(StateStoreMetadataV2("default", 0, numShufflePartitions, List.empty))
+
+        val expectedProperties =
+          StreamingJoinOperatorProperties(useVirtualColumnFamilies = true)
+
+        val expectedMetadata = OperatorStateMetadataV2(
+          OperatorInfoV1(0, "symmetricHashJoin"),
+          expectedStateStoreInfo,
+          expectedProperties.json
+        )
+        checkOperatorStateMetadata(checkpointDir.toString, 0, expectedMetadata, 2)
+
+        // Verify that the state store metadata is not available for invalid batches.
+        val ex = intercept[Exception] {
+          val invalidBatchId = OperatorStateMetadataUtils.getLastOffsetBatch(
+            spark,
+            checkpointDir.toString
+          ) + 1
+          checkOperatorStateMetadata(
+            checkpointDir.toString,
+            0,
+            expectedMetadata,
+            2,
+            Some(invalidBatchId)
+          )
+        }
+
+        checkError(
+          ex.asInstanceOf[SparkRuntimeException],
+          "STDS_FAILED_TO_READ_OPERATOR_METADATA",
+          Some("42K03"),
+          Map("checkpointLocation" -> ".*", "batchId" -> ".*"),
+          matchPVals = true
+        )
+
+        val ex1 = intercept[Exception] {
+          checkOperatorStateMetadata(checkpointDir.toString, 0, expectedMetadata, 2, Some(-1))
+        }
+
+        checkError(
+          ex1.asInstanceOf[SparkRuntimeException],
+          "STDS_FAILED_TO_READ_OPERATOR_METADATA",
+          Some("42K03"),
+          Map("checkpointLocation" -> ".*", "batchId" -> ".*"),
+          matchPVals = true
+        )
+
+        val df = spark.read.format("state-metadata").load(checkpointDir.toString)
+        checkAnswer(
+          df,
+          Seq(Row(0, "symmetricHashJoin", "default", 5, 0L, 1L, expectedProperties.json))
+        )
+      }
+    }
+  }
+
   test("Stateful operator metadata for streaming session window") {
     withTempDir { checkpointDir =>
       val input = MemoryStream[(String, Long)]
