@@ -27,12 +27,14 @@ import org.scalatest.Tag
 import org.apache.spark.{SparkContext, SparkException, TaskContext}
 import org.apache.spark.sql.{DataFrame, ForeachWriter}
 import org.apache.spark.sql.catalyst.expressions.UnsafeRow
-import org.apache.spark.sql.execution.streaming.{CommitLog, MemoryStream, StreamExecution}
+import org.apache.spark.sql.execution.streaming.checkpointing.CommitLog
+import org.apache.spark.sql.execution.streaming.runtime.{MemoryStream, StreamExecution}
+import org.apache.spark.sql.execution.streaming.state.StateStoreCoordinatorSuite.withCoordinatorRef
 import org.apache.spark.sql.execution.streaming.state.StateStoreTestsHelper
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.streaming._
-import org.apache.spark.sql.streaming.OutputMode.Update
+import org.apache.spark.sql.streaming.OutputMode.{Append, Update}
 import org.apache.spark.sql.test.TestSparkSession
 import org.apache.spark.sql.types.StructType
 
@@ -89,6 +91,8 @@ case class CkptIdCollectingStateStoreWrapper(innerStore: StateStore) extends Sta
 
   override def abort(): Unit = innerStore.abort()
 
+  override def release(): Unit = {}
+
   // Implement methods from StateStore (current trait)
 
   override def removeColFamilyIfExists(colFamilyName: String): Boolean = {
@@ -119,6 +123,13 @@ case class CkptIdCollectingStateStoreWrapper(innerStore: StateStore) extends Sta
     innerStore.put(key, value, colFamilyName)
   }
 
+  override def putList(
+      key: UnsafeRow,
+      values: Array[UnsafeRow],
+      colFamilyName: String = StateStore.DEFAULT_COL_FAMILY_NAME): Unit = {
+    innerStore.putList(key, values, colFamilyName)
+  }
+
   override def remove(
       key: UnsafeRow,
       colFamilyName: String = StateStore.DEFAULT_COL_FAMILY_NAME): Unit = {
@@ -132,6 +143,13 @@ case class CkptIdCollectingStateStoreWrapper(innerStore: StateStore) extends Sta
     innerStore.merge(key, value, colFamilyName)
   }
 
+  override def mergeList(
+      key: UnsafeRow,
+      values: Array[UnsafeRow],
+      colFamilyName: String = StateStore.DEFAULT_COL_FAMILY_NAME): Unit = {
+    innerStore.mergeList(key, values, colFamilyName)
+  }
+
   override def commit(): Long = innerStore.commit()
   override def metrics: StateStoreMetrics = innerStore.metrics
   override def getStateStoreCheckpointInfo(): StateStoreCheckpointInfo = {
@@ -157,7 +175,6 @@ class CkptIdCollectingStateStoreProviderWrapper extends StateStoreProvider {
       hadoopConf: Configuration,
       useMultipleValuesPerKey: Boolean = false,
       stateSchemaProvider: Option[StateSchemaProvider] = None): Unit = {
-    hadoopConf.set(StreamExecution.RUN_ID_KEY, UUID.randomUUID().toString)
     innerProvider.init(
       stateStoreId,
       keySchema,
@@ -180,14 +197,44 @@ class CkptIdCollectingStateStoreProviderWrapper extends StateStoreProvider {
   }
 
   override def getReadStore(version: Long, uniqueId: Option[String] = None): ReadStateStore = {
-    new WrappedReadStateStore(
-      CkptIdCollectingStateStoreWrapper(innerProvider.getReadStore(version, uniqueId)))
+    // Don't wrap the read store with WrappedReadStateStore since it makes upgrading difficult
+    // Just return the wrapped store directly
+    CkptIdCollectingStateStoreWrapper(innerProvider.getReadStore(version, uniqueId))
   }
 
-  override def doMaintenance(): Unit = innerProvider.doMaintenance()
+  override def upgradeReadStoreToWriteStore(
+      readStore: ReadStateStore,
+      version: Long,
+      uniqueId: Option[String] = None): StateStore = {
+    // Following the pattern from RocksDBStateStoreProvider, we verify version and id match
+    assert(version == readStore.version,
+      s"Can only upgrade readStore to writeStore with the same version," +
+        s" readStoreVersion: ${readStore.version}, writeStoreVersion: ${version}")
+    assert(this.stateStoreId == readStore.id, "Can only upgrade readStore to writeStore with" +
+      " the same stateStoreId")
+
+    // Extract the inner store from our wrapper
+    val innerReadStore = readStore match {
+      case wrapper: CkptIdCollectingStateStoreWrapper => wrapper.innerStore
+      case _ =>
+        throw new IllegalStateException(
+          s"Expected CkptIdCollectingStateStoreWrapper" +
+            s" but got ${readStore.getClass}")
+    }
+
+    // Delegate to inner provider to upgrade the store
+    val upgradedStore = innerProvider.upgradeReadStoreToWriteStore(
+      innerReadStore, version, uniqueId)
+
+    // Wrap the upgraded store with CkptIdCollectingStateStoreWrapper
+    CkptIdCollectingStateStoreWrapper(upgradedStore)
+  }
 
   override def supportedCustomMetrics: Seq[StateStoreCustomMetric] =
     innerProvider.supportedCustomMetrics
+
+  override def supportedInstanceMetrics: Seq[StateStoreInstanceMetric] =
+    innerProvider.supportedInstanceMetrics
 }
 
 class RocksDBStateStoreCheckpointFormatV2Suite extends StreamTest
@@ -577,6 +624,29 @@ class RocksDBStateStoreCheckpointFormatV2Suite extends StreamTest
     validateBaseCheckpointInfo()
   }
 
+  def validateCheckpointInfoGlobalLimit(
+      numBatches: Int,
+      numStateStores: Int,
+      batchVersionSet: Set[Long]): Unit = {
+    val checkpointInfoList = CkptIdCollectingStateStoreWrapper.getStateStoreCheckpointInfos
+    // We have 6 batches, 1 partitions (since global limit), and 1 state store per batch
+    assert(checkpointInfoList.size == numBatches * numStateStores * 1)
+    checkpointInfoList.foreach { l =>
+      assert(l.stateStoreCkptId.isDefined)
+      if (batchVersionSet.contains(l.batchVersion)) {
+        assert(l.baseStateStoreCkptId.isDefined)
+      }
+    }
+    assert(checkpointInfoList.count(_.partitionId == 0) == numBatches * numStateStores)
+    // Since we use global limit, there should be no partition 1
+    assert(checkpointInfoList.count(_.partitionId == 1) == 0)
+    for (i <- 1 to numBatches) {
+      // Since we use global limit, there should be only one store per batch
+      assert(checkpointInfoList.count(_.batchVersion == i) == numStateStores * 1)
+    }
+    validateBaseCheckpointInfo()
+  }
+
   /**
    * Verify lineage for each partition across batches. Below should satisfy because
    * these ids are stored in the following manner:
@@ -1139,6 +1209,54 @@ class RocksDBStateStoreCheckpointFormatV2Suite extends StreamTest
     validateCheckpointInfo(6, 1, Set(2, 4, 6))
   }
 
+
+  // No matter the number of shuffle partitions, global limit should always use one partition
+  Seq(1, 2, 10, 200).foreach { shufflePartitions =>
+    testWithCheckpointInfoTracked(
+      s"checkpointFormatVersion2 validate StreamingGlobalLimit with " +
+      s"shufflePartitions = $shufflePartitions") {
+      withTempDir { checkpointDir =>
+        withSQLConf((SQLConf.SHUFFLE_PARTITIONS.key, shufflePartitions.toString)) {
+          val inputData = MemoryStream[Int]
+          val aggregated = inputData
+            .toDF()
+            .limit(10)
+
+          testStream(aggregated, Append)(
+            StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
+            AddData(inputData, 3),
+            CheckLastBatch(3),
+            AddData(inputData, 3, 2),
+            CheckLastBatch(3, 2),
+            StopStream
+          )
+
+          // Test recovery
+          testStream(aggregated, Append)(
+            StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
+            AddData(inputData, 4, 1, 3),
+            CheckLastBatch(4, 1, 3),
+            AddData(inputData, 5, 4, 4),
+            CheckLastBatch(5, 4, 4),
+            StopStream
+          )
+
+          // crash recovery again
+          testStream(aggregated, Append)(
+            StartStream(checkpointLocation = checkpointDir.getAbsolutePath),
+            AddData(inputData, 4, 7),
+            CheckLastBatch(4),
+            AddData(inputData, 5),
+            CheckLastBatch(),
+            StopStream
+          )
+
+          validateCheckpointInfoGlobalLimit(6, 1, Set(2, 3, 4, 5, 6))
+        }
+      }
+    }
+  }
+
   test("checkpointFormatVersion2 validate transformWithState") {
     withTempDir { checkpointDir =>
       val inputData = MemoryStream[String]
@@ -1173,28 +1291,30 @@ class RocksDBStateStoreCheckpointFormatV2Suite extends StreamTest
   test("checkpointFormatVersion2 racing commits don't return incorrect checkpointInfo") {
     val sqlConf = new SQLConf()
     sqlConf.setConf(SQLConf.STATE_STORE_CHECKPOINT_FORMAT_VERSION, 2)
-
-    withTempDir { checkpointDir =>
-      val provider = new CkptIdCollectingStateStoreProviderWrapper()
-      provider.init(
-        StateStoreId(checkpointDir.toString, 0, 0),
-        StateStoreTestsHelper.keySchema,
-        StateStoreTestsHelper.valueSchema,
-        PrefixKeyScanStateEncoderSpec(StateStoreTestsHelper.keySchema, 1),
-        useColumnFamilies = false,
-        new StateStoreConf(sqlConf),
-        new Configuration
-      )
-
-      val store1 = provider.getStore(0)
-      val store1NewVersion = store1.commit()
-      val store2 = provider.getStore(1)
-      val store2NewVersion = store2.commit()
-      val store1CheckpointInfo = store1.getStateStoreCheckpointInfo()
-      val store2CheckpointInfo = store2.getStateStoreCheckpointInfo()
-
-      assert(store1CheckpointInfo.batchVersion == store1NewVersion)
-      assert(store2CheckpointInfo.batchVersion == store2NewVersion)
+    val sc = spark.sparkContext
+    withCoordinatorRef(sc) { _ =>
+      withTempDir { checkpointDir =>
+        val hadoopConf = new Configuration()
+        hadoopConf.set(StreamExecution.RUN_ID_KEY, UUID.randomUUID().toString)
+        val provider = new CkptIdCollectingStateStoreProviderWrapper()
+        provider.init(
+          StateStoreId(checkpointDir.toString, 0, 0),
+          StateStoreTestsHelper.keySchema,
+          StateStoreTestsHelper.valueSchema,
+          PrefixKeyScanStateEncoderSpec(StateStoreTestsHelper.keySchema, 1),
+          useColumnFamilies = false,
+          new StateStoreConf(sqlConf),
+          hadoopConf
+        )
+        val store1 = provider.getStore(0)
+        val store1NewVersion = store1.commit()
+        val store2 = provider.getStore(1)
+        val store2NewVersion = store2.commit()
+        val store1CheckpointInfo = store1.getStateStoreCheckpointInfo()
+        val store2CheckpointInfo = store2.getStateStoreCheckpointInfo()
+        assert(store1CheckpointInfo.batchVersion == store1NewVersion)
+        assert(store2CheckpointInfo.batchVersion == store2NewVersion)
+      }
     }
   }
 }
