@@ -27,14 +27,13 @@ import scala.reflect.ClassTag
 import scala.util.control.ControlThrowable
 
 import com.google.common.util.concurrent.UncheckedExecutionException
-import org.apache.commons.io.FileUtils
 import org.apache.hadoop.conf.Configuration
 import org.scalatest.time.SpanSugar._
 
 import org.apache.spark.{SparkConf, SparkContext, TaskContext, TestUtils}
 import org.apache.spark.scheduler.{SparkListener, SparkListenerJobStart}
 import org.apache.spark.sql.{AnalysisException, Encoders, Row, SQLContext, TestStrategy}
-import org.apache.spark.sql.catalyst.plans.logical.{Range, RepartitionByExpression}
+import org.apache.spark.sql.catalyst.plans.logical.Range
 import org.apache.spark.sql.catalyst.streaming.{InternalOutputModes, StreamingRelationV2}
 import org.apache.spark.sql.catalyst.types.DataTypeUtils.toAttributes
 import org.apache.spark.sql.catalyst.util.DateTimeUtils
@@ -43,6 +42,8 @@ import org.apache.spark.sql.classic.ClassicConversions._
 import org.apache.spark.sql.execution.{LocalLimitExec, SimpleMode, SparkPlan}
 import org.apache.spark.sql.execution.command.ExplainCommand
 import org.apache.spark.sql.execution.streaming._
+import org.apache.spark.sql.execution.streaming.operators.stateful.StreamingLocalLimitExec
+import org.apache.spark.sql.execution.streaming.runtime.{LongOffset, MemoryStream, StreamExecution, StreamingExecutionRelation, StreamingQueryWrapper, StreamingRelation}
 import org.apache.spark.sql.execution.streaming.sources.{ContinuousMemoryStream, ForeachBatchUserFuncException, MemorySink}
 import org.apache.spark.sql.execution.streaming.state.{KeyStateEncoderSpec, StateSchemaProvider, StateStore, StateStoreConf, StateStoreId, StateStoreProvider}
 import org.apache.spark.sql.expressions.Window
@@ -115,7 +116,7 @@ class StreamSuite extends StreamTest {
     val memoryStream = MemoryStream[Int]
     val executionRelation = StreamingExecutionRelation(
       memoryStream, toAttributes(memoryStream.encoder.schema), None)(
-      memoryStream.sqlContext.sparkSession)
+      memoryStream.sparkSession)
     assert(executionRelation.computeStats().sizeInBytes ==
       spark.sessionState.conf.defaultSizeInBytes)
   }
@@ -767,7 +768,7 @@ class StreamSuite extends StreamTest {
     val dir1 = Utils.createTempDir().getCanonicalFile // not using withTempDir {}, makes test flaky
     // Copy the checkpoint to a temp dir to prevent changes to the original.
     // Not doing this will lead to the test passing on the first run, but fail subsequent runs.
-    FileUtils.copyDirectory(checkpointDir, dir1)
+    Utils.copyDirectory(checkpointDir, dir1)
     // Checkpoint data was generated by a query with 10 shuffle partitions.
     // In order to test reading from the checkpoint, the checkpoint must have two or more batches,
     // since the last batch may be rerun.
@@ -793,7 +794,7 @@ class StreamSuite extends StreamTest {
     // 2 - Check recovery with wrong num shuffle partitions
     prepareMemoryStream()
     val dir2 = Utils.createTempDir().getCanonicalFile
-    FileUtils.copyDirectory(checkpointDir, dir2)
+    Utils.copyDirectory(checkpointDir, dir2)
     // Since the number of partitions is greater than 10, should throw exception.
     withSQLConf(SQLConf.SHUFFLE_PARTITIONS.key -> "15") {
       var streamingQuery: StreamingQuery = null
@@ -1115,7 +1116,7 @@ class StreamSuite extends StreamTest {
       )
       require(execPlan != null)
 
-      val localLimits = execPlan.collect {
+      val localLimits = StreamingQueryPlanTraverseHelper.collectFromUnfoldedPlan(execPlan) {
         case l: LocalLimitExec => l
         case l: StreamingLocalLimitExec => l
       }
@@ -1298,37 +1299,6 @@ class StreamSuite extends StreamTest {
     }
   }
 
-  test("SPARK-34482: correct active SparkSession for logicalPlan") {
-    withSQLConf(
-      SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "true",
-      SQLConf.COALESCE_PARTITIONS_INITIAL_PARTITION_NUM.key -> "10") {
-      val df = spark.readStream.format(classOf[FakeDefaultSource].getName).load()
-      var query: StreamExecution = null
-      try {
-        query =
-          df.repartition($"a")
-            .writeStream
-            .format("memory")
-            .queryName("memory")
-            .start()
-            .asInstanceOf[StreamingQueryWrapper]
-            .streamingQuery
-        query.awaitInitialization(streamingTimeout.toMillis)
-        val plan = query.logicalPlan
-        val numPartition = plan
-          .find { _.isInstanceOf[RepartitionByExpression] }
-          .map(_.asInstanceOf[RepartitionByExpression].numPartitions)
-        // Before the fix of SPARK-34482, the numPartition is the value of
-        // `COALESCE_PARTITIONS_INITIAL_PARTITION_NUM`.
-        assert(numPartition.get === spark.sessionState.conf.getConf(SQLConf.SHUFFLE_PARTITIONS))
-      } finally {
-        if (query != null) {
-          query.stop()
-        }
-      }
-    }
-  }
-
   test("isInterruptionException should correctly unwrap classic py4j InterruptedException") {
     val e1 = new py4j.Py4JException(
       """
