@@ -20,10 +20,12 @@ package org.apache.spark.sql.streaming
 import java.time.Duration
 
 import org.apache.spark.sql.Encoders
-import org.apache.spark.sql.execution.streaming.{ListStateImplWithTTL, MapStateImplWithTTL, MemoryStream, ValueStateImplWithTTL}
+import org.apache.spark.sql.execution.streaming.operators.stateful.transformwithstate.ttl.ListStateImplWithTTL
+import org.apache.spark.sql.execution.streaming.runtime.MemoryStream
 import org.apache.spark.sql.execution.streaming.state.RocksDBStateStoreProvider
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.streaming.util.StreamManualClock
+import org.apache.spark.tags.SlowSQLTest
 
 // MultiStatefulVariableTTLProcessor is a StatefulProcessor that consumes a stream of
 // strings and returns a stream of <string, count> pairs.
@@ -36,25 +38,22 @@ import org.apache.spark.sql.streaming.util.StreamManualClock
 // used to add a record into the secondary index for every appendList call.
 class MultiStatefulVariableTTLProcessor(ttlConfig: TTLConfig)
   extends StatefulProcessor[String, String, (String, Long)]{
-  @transient private var _listState: ListStateImplWithTTL[String] = _
+  @transient private var _listState: ListState[String] = _
   // Map from index to count
-  @transient private var _mapState: MapStateImplWithTTL[Long, Long] = _
+  @transient private var _mapState: MapState[Long, Long] = _
   // Counts the number of times the string has occurred. It should always be
   // equal to the size of the list state at the start and end of handleInputRows.
-  @transient private var _valueState: ValueStateImplWithTTL[Long] = _
+  @transient private var _valueState: ValueState[Long] = _
 
   override def init(
       outputMode: OutputMode,
       timeMode: TimeMode): Unit = {
     _listState = getHandle
       .getListState("listState", Encoders.STRING, ttlConfig)
-      .asInstanceOf[ListStateImplWithTTL[String]]
     _mapState = getHandle
         .getMapState("mapState", Encoders.scalaLong, Encoders.scalaLong, ttlConfig)
-        .asInstanceOf[MapStateImplWithTTL[Long, Long]]
     _valueState = getHandle
         .getValueState("valueState", Encoders.scalaLong, ttlConfig)
-        .asInstanceOf[ValueStateImplWithTTL[Long]]
   }
   override def handleInputRows(
       key: String,
@@ -94,14 +93,13 @@ class MultiStatefulVariableTTLProcessor(ttlConfig: TTLConfig)
 class ListStateTTLProcessor(ttlConfig: TTLConfig)
   extends StatefulProcessor[String, InputEvent, OutputEvent] {
 
-  @transient private var _listState: ListStateImplWithTTL[Int] = _
+  @transient private var _listState: ListState[Int] = _
 
   override def init(
       outputMode: OutputMode,
       timeMode: TimeMode): Unit = {
     _listState = getHandle
       .getListState("listState", Encoders.scalaInt, ttlConfig)
-      .asInstanceOf[ListStateImplWithTTL[Int]]
   }
 
   override def handleInputRows(
@@ -111,7 +109,8 @@ class ListStateTTLProcessor(ttlConfig: TTLConfig)
     var results = List[OutputEvent]()
 
     inputRows.foreach { row =>
-      val resultIter = processRow(row, _listState)
+      val resultIter = processRow(row,
+        _listState.asInstanceOf[ListStateImplWithTTL[Int]])
       resultIter.foreach { r =>
         results = r :: results
       }
@@ -167,6 +166,7 @@ class ListStateTTLProcessor(ttlConfig: TTLConfig)
  * Test suite for testing list state with TTL.
  * We use the base TTL suite with a list state processor.
  */
+@SlowSQLTest
 class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
   with StateStoreMetricsTest {
 
@@ -202,26 +202,32 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
         AddData(inputStream, "k1"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k1", 1)),
+        assertNumStateRows(total = 3, updated = 3),
 
         AddData(inputStream, "k2"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k2", 1)),
+        assertNumStateRows(total = 6, updated = 3),
 
         AddData(inputStream, "k1"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k1", 2)),
+        assertNumStateRows(total = 7, updated = 3),
 
         AddData(inputStream, "k2"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k2", 2)),
+        assertNumStateRows(total = 8, updated = 3),
 
         AddData(inputStream, "k1"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k1", 3)),
+        assertNumStateRows(total = 9, updated = 3),
 
         AddData(inputStream, "k2"),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(("k2", 3)),
+        assertNumStateRows(total = 10, updated = 3)
 
         // For each unique key that occurs t times, the MultiStatefulVariableTTLProcessor maintains:
         //    - Map state: t records in the primary, and t records in the TTL index
@@ -239,7 +245,6 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
         // The number of updated rows is the total across the last time assertNumStateRows
         // was called, and we only update numRowsUpdated for primary key updates. We ran 6 batches
         // and each wrote 3 primary keys, so the total number of updated rows is 6 * 3 = 18.
-        assertNumStateRows(total = 10, updated = 18)
       )
     }
   }
@@ -268,6 +273,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
         ),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(),
+        assertNumStateRows(total = 1, updated = 3),
         // get ttl values
         AddData(inputStream, InputEvent("k1", "get_ttl_value_from_state", -1, null)),
         AdvanceManualClock(1 * 1000),
@@ -276,6 +282,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           OutputEvent("k1", 2, isTTLValue = true, 62000),
           OutputEvent("k1", 3, isTTLValue = true, 62000)
         ),
+        assertNumStateRows(total = 1, updated = 0),
         // advance clock to add elements with later TTL
         AdvanceManualClock(45 * 1000), // batch timestamp: 48000
         AddData(inputStream,
@@ -285,6 +292,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
         ),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(),
+        assertNumStateRows(total = 1, updated = 3),
         Execute { q =>
           assert(q.lastProgress.stateOperators(0).numRowsUpdated === 3)
         },
@@ -299,6 +307,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           OutputEvent("k1", 5, isTTLValue = true, 109000),
           OutputEvent("k1", 6, isTTLValue = true, 109000)
         ),
+        assertNumStateRows(total = 1, updated = 0),
         AddData(inputStream, InputEvent("k1", "get", -1, null)),
         // advance clock to expire the first three elements
         AdvanceManualClock(15 * 1000), // batch timestamp: 65000
@@ -307,6 +316,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           OutputEvent("k1", 5, isTTLValue = false, -1),
           OutputEvent("k1", 6, isTTLValue = false, -1)
         ),
+        assertNumStateRows(total = 1, updated = 0),
         Execute { q =>
           assert(q.lastProgress.stateOperators(0).numRowsRemoved === 3)
         },
@@ -318,11 +328,13 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           OutputEvent("k1", 5, isTTLValue = false, -1),
           OutputEvent("k1", 6, isTTLValue = false, -1)
         ),
+        assertNumStateRows(total = 1, updated = 0),
         AddData(inputStream, InputEvent("k1", "get_values_in_ttl_state", -1, null)),
         AdvanceManualClock(1 * 1000),
         CheckNewAnswer(
           OutputEvent("k1", -1, isTTLValue = true, 109000)
-        )
+        ),
+        assertNumStateRows(total = 1, updated = 0)
       )
     }
   }
@@ -362,6 +374,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           // advance clock to trigger processing
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 3),
 
           // get ttl values
           AddData(inputStream, InputEvent("k1", "get_ttl_value_from_state", -1, null)),
@@ -371,6 +384,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = true, 182000),
             OutputEvent("k1", 3, isTTLValue = true, 182000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           AddData(inputStream, InputEvent("k1", "get", -1, null)),
           AdvanceManualClock(1 * 1000),
@@ -379,6 +393,8 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = false, -1),
             OutputEvent("k1", 3, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           StopStream
         )
 
@@ -403,6 +419,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           // advance clock to trigger processing
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 3),
 
           // get all elements without enforcing ttl
           AddData(inputStream, InputEvent("k1", "get_without_enforcing_ttl", -1, null)),
@@ -415,6 +432,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 5, isTTLValue = false, -1),
             OutputEvent("k1", 6, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           AddData(inputStream, InputEvent("k1", "get_ttl_value_from_state", -1, null)),
           AdvanceManualClock(1 * 1000),
@@ -426,6 +444,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 5, isTTLValue = true, 20000),
             OutputEvent("k1", 6, isTTLValue = true, 20000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
           StopStream
         )
 
@@ -440,6 +459,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           // advance clock to trigger processing
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 3),
 
           // advance clock to expire the middle three elements
           AddData(inputStream, InputEvent("k1", "get_values_in_ttl_state", -1, null)),
@@ -447,6 +467,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           CheckNewAnswer(
             OutputEvent("k1", -1, isTTLValue = true, 20000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           // progress batch timestamp from 9000 to 54000, expiring the middle
           // three elements.
@@ -462,6 +483,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 8, isTTLValue = false, -1),
             OutputEvent("k1", 9, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           AddData(inputStream, InputEvent("k1", "get_without_enforcing_ttl", -1, null)),
           AdvanceManualClock(1 * 1000),
@@ -473,6 +495,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 8, isTTLValue = false, -1),
             OutputEvent("k1", 9, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           AddData(inputStream, InputEvent("k1", "get_values_in_ttl_state", -1, null)),
           AdvanceManualClock(1 * 1000),
@@ -517,14 +540,17 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           AddData(inputStream, InputEvent("k1", "append", 1)),
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 1),
 
           AddData(inputStream, InputEvent("k1", "append", 2)),
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 1),
 
           AddData(inputStream, InputEvent("k1", "append", 3)),
           AdvanceManualClock(1 * 1000), // Time is 3000
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 1),
 
           // Add a separate key; this should not be affected by k1 expiring.
           // It will have an expiration of 64000.
@@ -538,12 +564,14 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = true, 62000),
             OutputEvent("k1", 3, isTTLValue = true, 63000)
           ),
+          assertNumStateRows(total = 2, updated = 1),
 
           AddData(inputStream, InputEvent("k1", "get_values_in_min_state", -1, null)),
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer( // Time is 5000 for this micro-batch
             OutputEvent("k1", -1, isTTLValue = true, 61000)
           ),
+          assertNumStateRows(total = 2, updated = 0),
 
           // The k1 records expire at 63000, and the current time is 5000. So, we advance the
           // clock by 63 - 5 = 58 seconds to expire those.
@@ -555,7 +583,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           //
           // It's important to check with assertNumStateRows, since the InputEvents
           // only return values for the current grouping key, not the entirety of RocksDB.
-          assertNumStateRows(total = 1, updated = 4),
+          assertNumStateRows(total = 1, updated = 0),
 
           // The k1 calls should both return no values. However, the k2 calls should return
           // one record each. We put these into one AddData call since we want them all to
@@ -620,6 +648,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           // advance clock to trigger processing
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 3),
 
           // get ttl values
           AddData(inputStream,
@@ -636,6 +665,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             // From the get_values_in_min_state call
             OutputEvent("k1", -1, isTTLValue = true, 121000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
 
           AddData(inputStream, InputEvent("k1", "get", -1, null)),
           AdvanceManualClock(1 * 1000),
@@ -644,6 +674,8 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = false, -1),
             OutputEvent("k1", 3, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           StopStream
         )
 
@@ -659,6 +691,7 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
           // advance clock to trigger processing
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(),
+          assertNumStateRows(total = 1, updated = 3),
 
           // get ttl values
           AddData(inputStream, InputEvent("k1", "get_ttl_value_from_state", -1, null)),
@@ -671,12 +704,15 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 5, isTTLValue = true, 65000),
             OutputEvent("k1", 6, isTTLValue = true, 65000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           AddData(inputStream, InputEvent("k1", "get_values_in_ttl_state", -1, null)),
           AdvanceManualClock(1 * 1000),
-
           CheckNewAnswer(
             OutputEvent("k1", -1, isTTLValue = true, 65000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           // expire end values, batch timestamp from 7000 to 67000
           AdvanceManualClock(60 * 1000),
           AddData(inputStream, InputEvent("k1", "get", -1, null)),
@@ -686,6 +722,8 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = false, -1),
             OutputEvent("k1", 3, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           AddData(inputStream, InputEvent("k1", "get_without_enforcing_ttl", -1, null)),
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(
@@ -693,11 +731,63 @@ class TransformWithListStateTTLSuite extends TransformWithStateTTLTest
             OutputEvent("k1", 2, isTTLValue = false, -1),
             OutputEvent("k1", 3, isTTLValue = false, -1)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
           AddData(inputStream, InputEvent("k1", "get_values_in_ttl_state", -1, null)),
           AdvanceManualClock(1 * 1000),
           CheckNewAnswer(
             OutputEvent("k1", -1, isTTLValue = true, 121000)
           ),
+          assertNumStateRows(total = 1, updated = 0),
+
+          StopStream
+        )
+      }
+    }
+  }
+
+  test("SPARK-53069: stopping and restarting the query maintains correct state total rows") {
+    withSQLConf(SQLConf.STATE_STORE_PROVIDER_CLASS.key ->
+      classOf[RocksDBStateStoreProvider].getName,
+      SQLConf.SHUFFLE_PARTITIONS.key -> "1") {
+      withTempDir { checkpointLocation =>
+        val ttlConfig = TTLConfig(ttlDuration = Duration.ofMinutes(10))
+        val inputStream = MemoryStream[String]
+        val result = inputStream.toDS()
+          .groupByKey(x => x)
+          .transformWithState(
+            new MultiStatefulVariableTTLProcessor(ttlConfig),
+            TimeMode.ProcessingTime(),
+            OutputMode.Append())
+        val clock = new StreamManualClock
+
+        testStream(result)(
+          StartStream(Trigger.ProcessingTime("1 second"), triggerClock = clock,
+            checkpointLocation = checkpointLocation.getAbsolutePath),
+
+          AddData(inputStream, "k1"),
+          AdvanceManualClock(1 * 1000),
+          CheckNewAnswer(("k1", 1)),
+          assertNumStateRows(total = 3, updated = 3),
+
+          StopStream,
+          StartStream(Trigger.ProcessingTime("1 second"), triggerClock = clock,
+            checkpointLocation = checkpointLocation.getAbsolutePath),
+
+          AddData(inputStream, "k1"),
+          AdvanceManualClock(1 * 1000),
+          CheckNewAnswer(("k1", 2)),
+          assertNumStateRows(total = 4, updated = 3),
+
+          StopStream,
+          StartStream(Trigger.ProcessingTime("1 second"), triggerClock = clock,
+            checkpointLocation = checkpointLocation.getAbsolutePath),
+
+          AddData(inputStream, "k1"),
+          AdvanceManualClock(1 * 1000),
+          CheckNewAnswer(("k1", 3)),
+          assertNumStateRows(total = 5, updated = 3),
+
           StopStream
         )
       }
