@@ -30,7 +30,7 @@ import org.apache.hive.service.cli.operation.ExecuteStatementOperation
 import org.apache.hive.service.cli.session.HiveSession
 import org.apache.hive.service.rpc.thrift.{TCLIServiceConstants, TColumnDesc, TPrimitiveTypeEntry, TRowSet, TTableSchema, TTypeDesc, TTypeEntry, TTypeId, TTypeQualifiers, TTypeQualifierValue}
 
-import org.apache.spark.internal.{Logging, LogKeys, MDC}
+import org.apache.spark.internal.{Logging, LogKeys}
 import org.apache.spark.internal.LogKeys._
 import org.apache.spark.sql.{DataFrame, Row, SparkSession}
 import org.apache.spark.sql.catalyst.util.CharVarcharUtils
@@ -54,7 +54,7 @@ private[hive] class SparkExecuteStatementOperation(
   // a global timeout value, we use the user-specified value.
   // This code follows the Hive timeout behaviour (See #29933 for details).
   private val timeout = {
-    val globalTimeout = session.sessionState.conf.getConf(SQLConf.THRIFTSERVER_QUERY_TIMEOUT)
+    val globalTimeout = conf.getConf(SQLConf.THRIFTSERVER_QUERY_TIMEOUT)
     if (globalTimeout > 0 && (queryTimeout <= 0 || globalTimeout < queryTimeout)) {
       globalTimeout
     } else {
@@ -63,13 +63,13 @@ private[hive] class SparkExecuteStatementOperation(
   }
   private var timeoutExecutor: ScheduledExecutorService = _
 
-  private val forceCancel = session.sessionState.conf.getConf(SQLConf.THRIFTSERVER_FORCE_CANCEL)
+  private val forceCancel = conf.getConf(SQLConf.THRIFTSERVER_FORCE_CANCEL)
 
   private val redactedStatement = {
-    val substitutorStatement = SQLConf.withExistingConf(session.sessionState.conf) {
+    val substitutorStatement = SQLConf.withExistingConf(conf) {
       new VariableSubstitution().substitute(statement)
     }
-    SparkUtils.redact(session.sessionState.conf.stringRedactionPattern, substitutorStatement)
+    SparkUtils.redact(conf.stringRedactionPattern, substitutorStatement)
   }
 
   private var result: DataFrame = _
@@ -89,10 +89,10 @@ private[hive] class SparkExecuteStatementOperation(
 
   def getNextRowSet(order: FetchOrientation, maxRowsL: Long): TRowSet = withLocalProperties {
     try {
-      session.sparkContext.setJobGroup(statementId, redactedStatement, forceCancel)
+      sparkContext.setJobGroup(statementId, redactedStatement, forceCancel)
       getNextRowSetInternal(order, maxRowsL)
     } finally {
-      session.sparkContext.clearJobGroup()
+      sparkContext.clearJobGroup()
     }
   }
 
@@ -209,7 +209,7 @@ private[hive] class SparkExecuteStatementOperation(
     }
   }
 
-  private def execute(): Unit = {
+  private def execute(): Unit = withClassLoader { classLoader =>
     try {
       synchronized {
         if (getStatus.getState.isTerminal) {
@@ -222,16 +222,12 @@ private[hive] class SparkExecuteStatementOperation(
           setState(OperationState.RUNNING)
         }
       }
-      // Always use the latest class loader provided by executionHive's state.
-      val executionHiveClassLoader = session.sharedState.jarClassLoader
-      Thread.currentThread().setContextClassLoader(executionHiveClassLoader)
-
       // Always set the session state classloader to `executionHiveClassLoader` even for sync mode
       if (!runInBackground) {
-        parentSession.getSessionState.getConf.setClassLoader(executionHiveClassLoader)
+        parentSession.getSessionState.getConf.setClassLoader(classLoader)
       }
 
-      session.sparkContext.setJobGroup(statementId, redactedStatement, forceCancel)
+      sparkContext.setJobGroup(statementId, redactedStatement, forceCancel)
       result = session.sql(statement)
       logDebug(result.queryExecution.toString())
       HiveThriftServer2.eventManager.onStatementParsed(statementId,
@@ -252,8 +248,8 @@ private[hive] class SparkExecuteStatementOperation(
         // then they may both call cleanup() before Spark Jobs are started. But before background
         // task interrupted, it may have started some spark job, so we need to cancel again to
         // make sure job was cancelled when background thread was interrupted
-        if (statementId != null) {
-          session.sparkContext.cancelJobGroup(statementId,
+        if (!sparkContext.isStopped && statementId != null) {
+          sparkContext.cancelJobGroup(statementId,
             "The corresponding Thriftserver query has failed.")
         }
         val currentState = getStatus().getState()
@@ -271,7 +267,7 @@ private[hive] class SparkExecuteStatementOperation(
           e match {
             case _: HiveSQLException => throw e
             case _ => throw HiveThriftServerErrors.runningQueryError(
-              e, session.sessionState.conf.errorMessageFormat)
+              e, conf.errorMessageFormat)
           }
         }
     } finally {
@@ -281,7 +277,7 @@ private[hive] class SparkExecuteStatementOperation(
           HiveThriftServer2.eventManager.onStatementFinish(statementId)
         }
       }
-      session.sparkContext.clearJobGroup()
+      sparkContext.clearJobGroup()
     }
   }
 
@@ -318,7 +314,7 @@ private[hive] class SparkExecuteStatementOperation(
     }
     // RDDs will be cleaned automatically upon garbage collection.
     if (statementId != null) {
-      session.sparkContext.cancelJobGroup(statementId)
+      sparkContext.cancelJobGroup(statementId)
     }
     // Shutdown the timeout thread if any, while cleaning up this operation
     if (timeoutExecutor != null &&
@@ -344,6 +340,7 @@ object SparkExecuteStatementOperation {
     case _: StringType => TTypeId.STRING_TYPE
     case _: DecimalType => TTypeId.DECIMAL_TYPE
     case DateType => TTypeId.DATE_TYPE
+    case _: TimeType => TTypeId.STRING_TYPE
     // TODO: Shall use TIMESTAMPLOCALTZ_TYPE, keep AS-IS now for
     // unnecessary behavior change
     case TimestampType => TTypeId.TIMESTAMP_TYPE
