@@ -21,8 +21,8 @@ import java.io.File
 import java.sql.{DriverManager, ResultSet, Statement}
 import java.util
 
-import scala.collection.JavaConverters._
 import scala.concurrent.duration._
+import scala.jdk.CollectionConverters._
 import scala.util.Try
 
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars
@@ -30,12 +30,13 @@ import org.apache.hadoop.hive.ql.metadata.Hive
 import org.apache.hadoop.hive.ql.session.SessionState
 import org.apache.hive.jdbc.HttpBasicAuthInterceptor
 import org.apache.hive.service.auth.PlainSaslHelper
-import org.apache.hive.service.cli.thrift.{ThriftCLIService, ThriftCLIServiceClient}
+import org.apache.hive.service.cli.thrift.{ThriftBinaryCLIService, ThriftCLIService, ThriftCLIServiceClient}
 import org.apache.hive.service.rpc.thrift.TCLIService.Client
 import org.apache.http.impl.client.HttpClientBuilder
 import org.apache.thrift.protocol.TBinaryProtocol
 import org.apache.thrift.transport.{THttpClient, TSocket}
 
+import org.apache.spark.SparkException
 import org.apache.spark.sql.test.SharedSparkSession
 import org.apache.spark.util.Utils
 
@@ -127,21 +128,30 @@ trait SharedThriftServer extends SharedSparkSession {
 
   private def startThriftServer(attempt: Int): Unit = {
     logInfo(s"Trying to start HiveThriftServer2: mode=$mode, attempt=$attempt")
-    val sqlContext = spark.newSession().sqlContext
+    val sparkSession = spark.newSession()
     // Set the HIVE_SERVER2_THRIFT_PORT and HIVE_SERVER2_THRIFT_HTTP_PORT to 0, so it could
     // randomly pick any free port to use.
     // It's much more robust than set a random port generated by ourselves ahead
-    sqlContext.setConf(ConfVars.HIVE_SERVER2_THRIFT_PORT.varname, "0")
-    sqlContext.setConf(ConfVars.HIVE_SERVER2_THRIFT_HTTP_PORT.varname, "0")
-    sqlContext.setConf(ConfVars.HIVE_SERVER2_TRANSPORT_MODE.varname, mode.toString)
-    sqlContext.setConf(ConfVars.SCRATCHDIR.varname, tempScratchDir.getAbsolutePath)
-    sqlContext.setConf(ConfVars.HIVE_START_CLEANUP_SCRATCHDIR.varname, "true")
+    val sessionConf = sparkSession.sessionState.conf
+    sessionConf.setConfString(ConfVars.HIVE_SERVER2_THRIFT_PORT.varname, "0")
+    sessionConf.setConfString(ConfVars.HIVE_SERVER2_THRIFT_HTTP_PORT.varname, "0")
+    sessionConf.setConfString(ConfVars.HIVE_SERVER2_TRANSPORT_MODE.varname, mode.toString)
+    sessionConf.setConfString("hive.exec.scratchdir", tempScratchDir.getAbsolutePath)
+    sessionConf.setConfString(ConfVars.HIVE_START_CLEANUP_SCRATCHDIR.varname, "true")
 
     try {
-      hiveServer2 = HiveThriftServer2.startWithContext(sqlContext)
+      // Set exitOnError to false to avoid exiting the JVM process and tearing down the SparkContext
+      // instance in case of any exceptions here. Otherwise, the following retries are doomed to
+      // fail on a stopped context.
+      hiveServer2 = HiveThriftServer2.startWithSparkSession(sparkSession, exitOnError = false)
       hiveServer2.getServices.asScala.foreach {
         case t: ThriftCLIService =>
           serverPort = t.getPortNumber
+          if (t.isInstanceOf[ThriftBinaryCLIService] && mode == ServerMode.http) {
+            logError("A previous Hive's SessionState is leaked, aborting this retry")
+            throw SparkException.internalError("HiveThriftServer2 started in binary mode " +
+              "while the test case is expecting HTTP mode")
+          }
           logInfo(s"Started HiveThriftServer2: mode=$mode, port=$serverPort, attempt=$attempt")
         case _ =>
       }
