@@ -145,8 +145,8 @@ object ArrayDataAgg extends Aggregator[Array[Double], Array[Double], Array[Doubl
     s1
   }
   def finish(s: Array[Double]): Array[Double] = s
-  def bufferEncoder: Encoder[Array[Double]] = ExpressionEncoder[Array[Double]]
-  def outputEncoder: Encoder[Array[Double]] = ExpressionEncoder[Array[Double]]
+  def bufferEncoder: Encoder[Array[Double]] = ExpressionEncoder[Array[Double]]()
+  def outputEncoder: Encoder[Array[Double]] = ExpressionEncoder[Array[Double]]()
 }
 
 abstract class UDAQuerySuite extends QueryTest with SQLTestUtils with TestHiveSingleton {
@@ -210,8 +210,7 @@ abstract class UDAQuerySuite extends QueryTest with SQLTestUtils with TestHiveSi
     spark.udf.register("longProductSum", udaf(LongProductSumAgg))
     spark.udf.register("arraysum", udaf(ArrayDataAgg))
     spark.udf.register("reduceOptionPair", udaf(Reduce[Option[(Int, Int)]](
-      (opt1, opt2) =>
-        opt1.zip(opt2).map { case ((a1, b1), (a2, b2)) => (a1 + a2, b1 + b2) }.headOption)))
+      (opt1, opt2) => opt1.zip(opt2).map { case ((a1, b1), (a2, b2)) => (a1 + a2, b1 + b2) })))
   }
 
   override def afterAll(): Unit = {
@@ -256,7 +255,7 @@ abstract class UDAQuerySuite extends QueryTest with SQLTestUtils with TestHiveSi
             |GROUP BY key
           """.stripMargin)
       },
-      errorClass = "AGGREGATE_FUNCTION_WITH_NONDETERMINISTIC_EXPRESSION",
+      condition = "AGGREGATE_FUNCTION_WITH_NONDETERMINISTIC_EXPRESSION",
       parameters = Map("sqlExpr" -> "\"mydoublesum(((value + (1.5 * key)) + rand()))\""),
       context = ExpectedContext(
         fragment = "value + 1.5 * key + rand()",
