@@ -120,10 +120,10 @@ sealed abstract class State[S] {
   /**
    * Get the state as a `scala.Option`. It will be `Some(state)` if it exists, otherwise `None`.
    */
-  @inline final def getOption(): Option[S] = if (exists) Some(get()) else None
+  @inline final def getOption(): Option[S] = if (exists()) Some(get()) else None
 
   @inline final override def toString(): String = {
-    getOption.map { _.toString }.getOrElse("<state not set>")
+    getOption().map { _.toString }.getOrElse("<state not set>")
   }
 }
 
