@@ -20,7 +20,7 @@ package org.apache.spark.streaming
 import java.io.File
 import java.util.concurrent.ConcurrentLinkedQueue
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 import scala.reflect.ClassTag
 
 import org.scalatest.PrivateMethodTester._
@@ -66,17 +66,17 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
         shouldBeTimingOut: Boolean = false
       ): Unit = {
       if (expectedData.isDefined) {
-        assert(state.exists)
+        assert(state.exists())
         assert(state.get() === expectedData.get)
         assert(state.getOption() === expectedData)
-        assert(state.getOption.getOrElse(-1) === expectedData.get)
+        assert(state.getOption().getOrElse(-1) === expectedData.get)
       } else {
-        assert(!state.exists)
+        assert(!state.exists())
         intercept[NoSuchElementException] {
           state.get()
         }
         assert(state.getOption() === None)
-        assert(state.getOption.getOrElse(-1) === -1)
+        assert(state.getOption().getOrElse(-1) === -1)
       }
 
       assert(state.isTimingOut() === shouldBeTimingOut)
@@ -161,7 +161,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
 
     // state maintains running count, and updated count is returned
     val mappingFunc = (key: String, value: Option[Int], state: State[Int]) => {
-      val sum = value.getOrElse(0) + state.getOption.getOrElse(0)
+      val sum = value.getOrElse(0) + state.getOption().getOrElse(0)
       state.update(sum)
       sum
     }
@@ -206,7 +206,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
 
     // state maintains running count, key string doubled and returned
     val mappingFunc = (batchTime: Time, key: String, value: Option[Int], state: State[Int]) => {
-      val sum = value.getOrElse(0) + state.getOption.getOrElse(0)
+      val sum = value.getOrElse(0) + state.getOption().getOrElse(0)
       state.update(sum)
       Some(key * 2)
     }
@@ -298,7 +298,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
       )
 
     val mappingFunc = (time: Time, key: String, value: Option[Int], state: State[Int]) => {
-      val sum = value.getOrElse(0) + state.getOption.getOrElse(0)
+      val sum = value.getOrElse(0) + state.getOption().getOrElse(0)
       val output = (key, sum)
       state.update(sum)
       Some(output)
@@ -336,7 +336,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
       )
 
     val mappingFunc = (time: Time, key: String, value: Option[Int], state: State[Int]) => {
-      val sum = value.getOrElse(0) + state.getOption.getOrElse(0)
+      val sum = value.getOrElse(0) + state.getOption().getOrElse(0)
       val output = (key, sum)
       state.update(sum)
       None.asInstanceOf[Option[Int]]
@@ -385,7 +385,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
       )
 
     val mappingFunc = (time: Time, key: String, value: Option[Int], state: State[Int]) => {
-      if (state.exists) {
+      if (state.exists()) {
         state.remove()
         Some(key)
       } else {
@@ -413,7 +413,7 @@ class MapWithStateSuite extends SparkFunSuite with LocalStreamingContext
       if (value.isDefined) {
         state.update(1)
       }
-      if (state.isTimingOut) {
+      if (state.isTimingOut()) {
         Some(key)
       } else {
         None
